@@ -22,7 +22,7 @@
 
 #ifndef lint
 static const char rcsid[] = 
-       "$Id: admin.c,v 1.5 2005/08/24 21:25:32 slay Exp $";
+       "$Id: admin.c 2 2006-04-03 21:04:25Z tomac $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -33,7 +33,15 @@ static const char rcsid[] =
 
 #include <stdio.h>
 #include <errno.h>
+
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
 #include <sys/socket.h>       
 
 #ifdef HAVE_NETINET_IN_SYSTM_H
@@ -91,10 +99,10 @@ static const char rcsid[] =
 
 
 int8_t 
-admin_init(struct filter *ip_filter)
+admin_init(struct term_tty *node)
 {
    if (thread_create(&terms->admin_listen_th.id, &admin_th_listen, 
-        (void *)ip_filter) < 0)
+        (void *)node) < 0)
       return -1;
 
    return 0;
@@ -113,6 +121,7 @@ admin_th_listen(void *arg)
    struct sockaddr_in server_address;
    struct timeval timeout;
    struct filter *ip_filter;
+   struct term_tty *node;
    int32_t sock=0, sock2, on=1,n,ret;
    socklen_t clilen;
    pthread_t tid;
@@ -131,7 +140,8 @@ write_log(0,"\n admin_th_network_listen es %d\n",(int)pthread_self());
       admin_th_listen_exit(NULL,0);
    }
 
-   ip_filter = (struct filter *)arg;
+   node = (struct term_tty *)arg;
+   ip_filter = (struct filter *)node->ip_filter;
    
    if ( (sock = socket(AF_INET, SOCK_STREAM, 0)) == -1 )
    {
@@ -168,7 +178,7 @@ write_log(0,"\n admin_th_network_listen es %d\n",(int)pthread_self());
    }
 
    server_address.sin_family      = AF_INET;
-   server_address.sin_port        = htons(SERVER_PORT);
+   server_address.sin_port        = htons(node->port);
    server_address.sin_addr.s_addr = htonl(INADDR_ANY);
           
    if ( bind( sock, (struct sockaddr *)&server_address,
@@ -673,9 +683,19 @@ write_log(0,"vty peer %d mutex_unlock terms \n",pthread_self());
                 case '\t': if (term_states[term_node->state].key_able && 
                                 !vty->authing)
                            {
-                              fail = term_vty_complete_command(term_node);
-                               if (!fail)
-                                  fail = term_vty_flush(term_node);
+                             fail = term_vty_write(term_node,"\r\n",2);
+                             if (!fail)
+                                fail = term_vty_complete_command(term_node);
+                             if (!fail)
+                                 fail = term_vty_prompt(term_node);
+                             if (!fail && vty->repeat_command)
+                               fail=term_vty_write(term_node, vty->buf_command,
+                                                    vty->command_len);                                                         
+                             if (!fail)
+                                fail = term_vty_flush(term_node);
+                             if (!vty->repeat_command)
+                                term_vty_clear_command(term_node);
+                             vty->repeat_command=0;                           
                               continue;
                            }            
                 break;
@@ -905,14 +925,15 @@ write_log(0,"vty peer %d mutex_unlock terms \n",pthread_self());
 void
 admin_th_network_peer_exit(struct term_node *term_node, int32_t sock)
 {
-   u_int8_t i;
+   dlist_t *p;
+   struct interface_data *iface_data;
 
    if (term_node)
    {
-       for (i=0; i< MAX_INTERFACES; i++)
+      for (p = term_node->used_ints->list; p; p = dlist_next(term_node->used_ints->list, p))
        {
-          if (term_node->used_ints[i])
-             interfaces_del(i);
+          iface_data = (struct interface_data *) dlist_data(p);
+          interfaces_disable(iface_data->ifname);
        }
 
       attack_kill_th(term_node,ALL_ATTACK_THREADS);
