@@ -1,7 +1,7 @@
 /* dhcp.h
  * Definitions for Dynamic Host Configuration Protocol
  *
- * $Id: dhcp.h,v 1.7 2005/08/02 17:43:40 t0mac Exp $ 
+ * $Id: dhcp.h 2 2006-04-03 21:04:25Z tomac $ 
  *
  * Yersinia
  * By David Barroso <tomac@wasahero.org> and Alfredo Andres <slay@wasahero.org>
@@ -38,8 +38,6 @@
 
 #define DHCP_MAX_OPTIONS 100
 
-#define DHCP_TOTAL_FIELDS 23
-
 #define DHCP_SMAC        0
 #define DHCP_DMAC        1
 #define DHCP_SIP         2
@@ -170,31 +168,31 @@ static struct attack_param dhcp_tlv[] = {
 };
 
 static const struct tuple_type_desc dhcp_opcode[] = {
-    { LIBNET_DHCP_REQUEST, "(REQUEST)" },
-    { LIBNET_DHCP_REPLY, "(REPLY)" },
+    { LIBNET_DHCP_REQUEST, "REQUEST" },
+    { LIBNET_DHCP_REPLY, "REPLY" },
     { 0, NULL }
 };
 
 static const struct tuple_type_desc dhcp_message[] = {
-    { LIBNET_DHCP_MSGDISCOVER, "(DISCOVER)" },
-    { LIBNET_DHCP_MSGOFFER,    "(OFFER)" },
-    { LIBNET_DHCP_MSGREQUEST,  "(REQUEST)" },
-    { LIBNET_DHCP_MSGDECLINE,  "(DECLINE)" },
-    { LIBNET_DHCP_MSGACK,      "(ACK)" },     
-    { LIBNET_DHCP_MSGNACK,     "(NACK)" },
-    { LIBNET_DHCP_MSGRELEASE,  "(RELEASE)" },
-    { LIBNET_DHCP_MSGINFORM,   "(INFORM)" },
+    { LIBNET_DHCP_MSGDISCOVER, "DISCOVER" },
+    { LIBNET_DHCP_MSGOFFER,    "OFFER" },
+    { LIBNET_DHCP_MSGREQUEST,  "REQUEST" },
+    { LIBNET_DHCP_MSGDECLINE,  "DECLINE" },
+    { LIBNET_DHCP_MSGACK,      "ACK" },     
+    { LIBNET_DHCP_MSGNACK,     "NACK" },
+    { LIBNET_DHCP_MSGRELEASE,  "RELEASE" },
+    { LIBNET_DHCP_MSGINFORM,   "INFORM" },
     { 0,                       NULL }
 };
 
 static const struct tuple_type_desc dhcp_htype[] = {
-    { DHCP_HW_TYPE_E10MB, "(E10MB)" },
+    { DHCP_HW_TYPE_E10MB, "E10MB" },
     { 0,                  NULL }
 };
 
 static const struct tuple_type_desc dhcp_port[] = {
-    { DHCP_CLIENT_PORT, "(CLIENT)" },
-    { DHCP_SERVER_PORT, "(SERVER)" },
+    { DHCP_CLIENT_PORT, "CLIENT" },
+    { DHCP_SERVER_PORT, "SERVER" },
     { 0, NULL}
 };
 
@@ -204,6 +202,9 @@ static struct proto_features dhcp_features[] = {
     { -1, 0 }
 };
 
+#define MAX_SNAME 64
+#define MAX_FNAME 128
+
 /* DHCP stuff */
 struct dhcp_data { /* DHCP and Ethernet fields*/
     u_int8_t op;
@@ -218,8 +219,8 @@ struct dhcp_data { /* DHCP and Ethernet fields*/
     u_int32_t siaddr;
     u_int32_t giaddr;
     u_int8_t chaddr[ETHER_ADDR_LEN];
-    char *sname; /* max 64 bytes */
-    char *file; /* max 128 bytes */
+    /*char sname[MAX_SNAME];
+    char fname[MAX_FNAME]; */
     /* specific options */
     u_int8_t options[MAX_TLV*MAX_VALUE_LENGTH];
     u_int8_t options_len;
@@ -238,40 +239,133 @@ static const struct tuple_type_desc dhcp_tlv_desc[] = {
     { 0,              NULL }
 };
 
-static struct position_fields dhcp_fields[] = {
-   { DHCP_SMAC,   "Source MAC",      1,  1, 12, 17, 0, 0, FIELD_MAC, FIELD_FIRST, NULL},
-   { DHCP_DMAC,   "Destination MAC", 1, 46, 62, 17, 0, 0, FIELD_MAC, FIELD_NORMAL, NULL},
-   { DHCP_SIP,    "SIP",             2,  1,  5, 15, 1, 2, FIELD_IP, FIELD_NORMAL, NULL},
-   { DHCP_DIP,    "DIP",             2, 22, 26, 15, 1, 18, FIELD_IP, FIELD_NORMAL, NULL},
-   { DHCP_SPORT,  "SPort",           2, 43, 49, 5 , 0, 0, FIELD_DEC, FIELD_NORMAL, dhcp_port},
-   { DHCP_DPORT,  "DPort",           2, 56, 62, 5 , 0, 0, FIELD_DEC, FIELD_NORMAL, dhcp_port},
-   { DHCP_OP,     "Op",              3,  1,  4, 2 , 0, 0, FIELD_HEX, FIELD_NORMAL, dhcp_opcode},
-   { DHCP_HTYPE,  "Htype",           3,  7, 13, 2 , 0, 0, FIELD_HEX, FIELD_NORMAL, dhcp_htype},
-   { DHCP_HLEN,   "Hlen",            3, 16, 21, 2 , 0, 0, FIELD_HEX, FIELD_NORMAL, NULL},
-   { DHCP_HOPS,   "Hops",            3, 24, 29, 2 , 0, 0, FIELD_HEX, FIELD_NORMAL, NULL},
-   { DHCP_XID,    "Xid",             3, 32, 36, 8 , 0, 0, FIELD_HEX, FIELD_NORMAL, NULL},
-   { DHCP_SECS,   "Secs",            3, 45, 50, 4 , 0, 0, FIELD_HEX, FIELD_NORMAL, NULL},
-   { DHCP_FLAGS,  "Flags",           3, 55, 61, 4 , 0, 0, FIELD_HEX, FIELD_NORMAL, NULL},
-   { DHCP_CIADDR, "CI",              4,  1,  4, 15, 0, 0, FIELD_IP, FIELD_NORMAL, NULL},
-   { DHCP_YIADDR, "YI",              4, 21, 24, 15, 0, 0, FIELD_IP, FIELD_NORMAL, NULL},
-   { DHCP_SIADDR, "SI",              4, 41, 44, 15, 0, 0, FIELD_IP, FIELD_NORMAL, NULL},
-   { DHCP_GIADDR, "GI",              4, 61, 64, 15, 0, 0, FIELD_IP, FIELD_NORMAL, NULL},
-   { DHCP_CHADDR, "CH",              5,  1,  4, 17, 0, 0, FIELD_MAC, FIELD_NORMAL, NULL},
-   { DHCP_TLV,    "TLV",             5, 23, 23, MAX_TLV*MAX_VALUE_LENGTH, 0, 0, FIELD_TLV, FIELD_LAST,  dhcp_tlv_desc},
-   /* TLV values that appear in the main window */
-   { DHCP_MSG,    "MESSAGETYPE",     0,  0,  0,  0, 1, 40,FIELD_TLV, FIELD_NORMAL, dhcp_message},
-   { 0,           NULL,              0,  0,  0, 0 , 0, 0, FIELD_NONE, FIELD_NORMAL, NULL}
+
+/* Struct needed for using protocol fields within the network client */
+struct commands_param dhcp_comm_params[] = {
+    { DHCP_SMAC, "source",    "Source MAC", 6, 0, 0, FIELD_MAC, "Set source MAC address", 
+                                        " H:H:H:H:H:H    48 bit mac address", 17, 1, 0, NULL, NULL },
+    { DHCP_DMAC, "dest",      "Destination MAC", 6, 0, 0, FIELD_MAC, "Set destination MAC address", 
+                                        " H:H:H:H:H:H    48 bit mac address", 17, 1, 0, NULL, NULL },
+    { DHCP_SIP, "ipsource",  "SIP", 4, 0, 0, FIELD_IP, "Set source IP address", 
+                                        " A.A.A.A    IPv4 address", 15, 2, 1, NULL, NULL },                
+    { DHCP_DIP, "ipdest",    "DIP", 4, 0, 0, FIELD_IP, "Set destination IP address", 
+                                        " A.A.A.A    IPv4 address", 15, 2, 1, NULL, NULL },
+    { DHCP_SPORT, "sport",     "SPort", 2, 0, 65535, FIELD_DEC, "Set UDP source port", 
+                                        " <0-65535>    UDP source port", 5, 2, 0, NULL, dhcp_port },
+    { DHCP_DPORT, "dport",     "DPort", 2, 0, 65535, FIELD_DEC, "Set UDP destination port", 
+                                        " <0-65535>    UDP destination port", 5, 2, 0, NULL, dhcp_port },
+    { DHCP_OP, "opcode",    "Op", 1, 0, 255, FIELD_HEX, "Set dhcp operation code", 
+                                        " <00-FF>    host dynamic configuration operation code", 2, 3, 0, NULL, NULL },
+    { DHCP_HTYPE, "htype",     "Htype", 1, 0, 255, FIELD_HEX, "Set dhcp htype", 
+                                        " <00-FF>    dhcp htype", 2, 3, 0, NULL, dhcp_htype },
+    { DHCP_HLEN, "hlen",      "HLEN", 1, 0, 255, FIELD_HEX, "Set dhcp hlen", 
+                                        " <00-FF>    dhcp hlen", 2, 3, 0, NULL, NULL },
+    { DHCP_HOPS, "hops",      "Hops", 1, 0, 255, FIELD_HEX, "Set dhcp hops", 
+                                        " <00-FF>    dhcp hops", 2, 3, 0, NULL, NULL },
+    { DHCP_XID, "xid",       "Xid", 4, 0, 0xFFFFFFFF, FIELD_HEX, "Set dhcp xid", 
+                                        " <00-FFFFFFFF>    dhcp xid", 8, 3, 0, NULL, NULL },
+    { DHCP_SECS, "secs",      "Secs", 2, 0, 65535, FIELD_HEX, "Set dhcp secs", 
+                                        " <00-FFFF>    dhcp secs", 4, 3, 0, NULL, NULL },
+    { DHCP_FLAGS, "flags",     "Flags", 2, 0, 65535, FIELD_HEX, "Set dhcp flags", 
+                                        " <00-FFFF>    dhcp flags", 4, 3, 0, NULL, NULL },
+    { DHCP_CIADDR, "ci",        "CI", 4, 0, 0, FIELD_IP, "Set ci IP address", 
+                                        " A.A.A.A    IPv4 address", 15, 4, 0, NULL, NULL },                
+    { DHCP_YIADDR, "yi",        "YI", 4, 0, 0, FIELD_IP, "Set yi IP address", 
+                                        " A.A.A.A    IPv4 address", 15, 4, 0, NULL, NULL },
+    { DHCP_SIADDR, "si",        "SI", 4, 0, 0, FIELD_IP, "Set si IP address", 
+                                        " A.A.A.A    IPv4 address", 15, 4, 0, NULL, NULL },                
+    { DHCP_GIADDR, "gi",        "GI", 4, 0, 0, FIELD_IP, "Set gi IP address", 
+                                        " A.A.A.A    IPv4 address", 15, 4, 0, NULL, NULL },                
+    { DHCP_CHADDR, "ch",        "CH", 6, 0, 0, FIELD_MAC, "Set ch MAC address", 
+                                        " H:H:H:H:H:H    48 bit mac address", 17, 5, 0, NULL, NULL },
+    { 0, "defaults",  NULL, 0, 0, 0, FIELD_DEFAULT, "Set all values to default", 
+                                        " <cr>", 0, 0, 0, NULL, NULL }, 
+    /* { "fname",     "Filename", MAX_FNAME, 0, 0, FIELD_STR, "Set boot file name", 
+                                        " WORD    Boot file name", MAX_FNAME, NULL },*/
+    { 0, "interface", NULL, IFNAMSIZ, 0, 0, FIELD_IFACE, "Set network interface to use", 
+                                        " WORD    Network interface", IFNAMSIZ, 0, 0, NULL, NULL },
+    /* { "sname",     "Sname", MAX_SNAME, 0, 0, FIELD_STR, "Set server hostname", 
+                                        " WORD    Server hostname", MAX_SNAME, NULL },*/
+    { DHCP_TLV, "tlv",       "TLV", 0, 0, 0, FIELD_EXTRA, "", "", 0, 0, 0, NULL, NULL}
+};
+
+struct commands_param_extra dhcp_params_tlv[] = {
+/*    { LIBNET_DHCP_PAD,             "PAD" },*/           
+    { LIBNET_DHCP_SUBNETMASK,      "subnetmask", "SubnetMask", 4, 0, 0, FIELD_IP, "Set SubnetMaskr", " A.A.A.A IPv4 address", 15, 0, NULL },         
+/*    { LIBNET_DHCP_TIMEOFFSET,      "TIMEOFFSET" },    */ 
+    { LIBNET_DHCP_ROUTER,          "router", "Router", 4, 0, 0, FIELD_IP, "Set Router", " A.A.A.A IPv4 address", 15, 0, NULL },         
+    { LIBNET_DHCP_TIMESERVER,      "timeserver", "TimeServer", 4, 0, 0, FIELD_IP, "Set TimeServer", " A.A.A.A IPv4 address", 15, 0, NULL },         
+    { LIBNET_DHCP_NAMESERVER,      "namserver", "NameServer", 4, 0, 0, FIELD_IP, "Set NameServer", " A.A.A.A IPv4 address", 15, 0, NULL },         
+    { LIBNET_DHCP_DNS,             "dns", "DNS", 4, 0, 0, FIELD_IP, "Set DNS", " A.A.A.A IPv4 address", 15, 0, NULL },         
+    { LIBNET_DHCP_LOGSERV,         "logserver", "LogServer", 4, 0, 0, FIELD_IP, "Set LogServer", " A.A.A.A IPv4 address", 15, 0, NULL },         
+    { LIBNET_DHCP_COOKIESERV,      "cookieserver", "CookieServer", 4, 0, 0, FIELD_IP, "Set CookieServer", " A.A.A.A IPv4 address", 15, 0, NULL },         
+    { LIBNET_DHCP_LPRSERV,         "lprserver", "LPRServer", 4, 0, 0, FIELD_IP, "Set LPRServer", " A.A.A.A IPv4 address", 15, 0, NULL },         
+    { LIBNET_DHCP_IMPSERV,         "impserver", "Impserver", 4, 0, 0, FIELD_IP, "Set ImpServer", " A.A.A.A IPv4 address", 15, 0, NULL },         
+    { LIBNET_DHCP_RESSERV,         "resserver", "ResServer", 4, 0, 0, FIELD_IP, "Set ResServer", " A.A.A.A IPv4 address", 15, 0, NULL },         
+    { LIBNET_DHCP_HOSTNAME,        "hostname", "HostName", MAX_STRING_SIZE, 0, 0, FIELD_STR, "Set HostName", " WORD HostName", MAX_STRING_SIZE, 0, NULL },     
+/*    { LIBNET_DHCP_BOOTFILESIZE,    "BOOTFILESIZE" },   
+    { LIBNET_DHCP_DUMPFILE,        "DUMPFILE" },      */ 
+    { LIBNET_DHCP_DOMAINNAME,      "domainname", "DomainName", MAX_STRING_SIZE, 0, 0, FIELD_STR, "Set DomainName", " WORD DomainName", MAX_STRING_SIZE, 0, NULL },     
+/*    { LIBNET_DHCP_SWAPSERV,        "SWAPSERV" },       
+    { LIBNET_DHCP_ROOTPATH,        "ROOTPATH" },       
+    { LIBNET_DHCP_EXTENPATH,       "EXTENPATH" },      
+    { LIBNET_DHCP_IPFORWARD,       "IPFORWARD" },      
+    { LIBNET_DHCP_SRCROUTE,        "SRCROUTE" },       
+    { LIBNET_DHCP_POLICYFILTER,    "POLICYFILTER" },   
+    { LIBNET_DHCP_MAXASMSIZE,      "MAXASMSIZE" },     
+    { LIBNET_DHCP_IPTTL,           "IPTTL" },          
+    { LIBNET_DHCP_MTUTIMEOUT,      "MTUTIMEOUT" },     
+    { LIBNET_DHCP_MTUTABLE,        "MTUTABLE" },       
+    { LIBNET_DHCP_MTUSIZE,         "MTUSIZE" },        
+    { LIBNET_DHCP_LOCALSUBNETS,    "LOCALSUBNETS" },   
+    { LIBNET_DHCP_BROADCASTADDR,   "BROADCASTADDR" },  
+    { LIBNET_DHCP_DOMASKDISCOV,    "DOMASKDISCOV" },   
+    { LIBNET_DHCP_MASKSUPPLY,      "MASKSUPPLY" },     
+    { LIBNET_DHCP_DOROUTEDISC,     "DOROUTEDISC" },    
+    { LIBNET_DHCP_ROUTERSOLICIT,   "ROUTERSOLICIT" },  
+    { LIBNET_DHCP_STATICROUTE,     "STATICROUTE" },    
+    { LIBNET_DHCP_TRAILERENCAP,    "TRAILERENCAP" },   
+    { LIBNET_DHCP_ARPTIMEOUT,      "ARPTIMEOUT" },     
+    { LIBNET_DHCP_ETHERENCAP,      "ETHERENCAP" },    
+    { LIBNET_DHCP_TCPTTL,          "TCPTTL" },        
+    { LIBNET_DHCP_TCPKEEPALIVE,    "TCPKEEPALIVE" },  
+    { LIBNET_DHCP_TCPALIVEGARBAGE, "TCPALIVEGARBAGE" },
+    { LIBNET_DHCP_NISDOMAIN,       "NISDOMAIN" },      
+    { LIBNET_DHCP_NISSERVERS,      "NISSERVERS" },    
+    { LIBNET_DHCP_NISTIMESERV,     "NISTIMESERV" },   
+    { LIBNET_DHCP_VENDSPECIFIC,    "VENDSPECIFIC" },  
+    { LIBNET_DHCP_NBNS,            "NBNS" },          
+    { LIBNET_DHCP_NBDD,            "NBDD" },          
+    { LIBNET_DHCP_NBTCPIP,         "NBTCPIP" },       
+    { LIBNET_DHCP_NBTCPSCOPE,      "NBTCPSCOPE" },    
+    { LIBNET_DHCP_XFONT,           "XFONT" },         
+    { LIBNET_DHCP_XDISPLAYMGR,     "XDISPLAYMGR" },   
+    { LIBNET_DHCP_DISCOVERADDR,    "DISCOVERADDR" }, */ 
+    { LIBNET_DHCP_LEASETIME,       "leasetime", "LeaseTime", 4, 0, 0xFFFFFFFF, FIELD_HEX, "Set LeaseTime", " <0x00000000 - 0xFFFFFFFF", 8, 0, NULL },     
+/*    { LIBNET_DHCP_OPTIONOVERLOAD,  "OPTIONOVERLOAD" },*/
+    { LIBNET_DHCP_MESSAGETYPE,     "messagetype", "MessageType", 1, 0, 0xFF, FIELD_HEX, "Set MessageType", " <0x00 - 0xFF>", 2, 1, dhcp_message },
+    { LIBNET_DHCP_SERVIDENT,       "servident", "ServIdent", 4, 0, 0, FIELD_IP, "Set ServIdent", " A.A.A.A IPv4 address", 15, 0, NULL },         
+/*    { LIBNET_DHCP_PARAMREQUEST,    "PARAMREQUEST" },  
+    { LIBNET_DHCP_MESSAGE,         "MESSAGE" },       
+    { LIBNET_DHCP_MAXMSGSIZE,      "MAXMSGSIZE" },*/    
+    { LIBNET_DHCP_RENEWTIME,       "renewtime", "RenewTime", 4, 0, 0xFFFFFFFF, FIELD_HEX, "Set RenewTime", " <0x00000000 - 0xFFFFFFFF", 8, 0, NULL },     
+    { LIBNET_DHCP_REBINDTIME,      "rebindtime", "RebindTime", 4, 0, 0xFFFFFFFF, FIELD_HEX, "Set RebindTime", " <0x00000000 - 0xFFFFFFFF", 8, 0, NULL },     
+    { LIBNET_DHCP_CLASSSID,         "classsid", "ClassSID", MAX_STRING_SIZE, 0, 0, FIELD_STR, "Set ClassSID", " WORD ClassSID", MAX_STRING_SIZE, 0, NULL },     
+/*    { LIBNET_DHCP_CLIENTID,        "CLIENTID" },      
+    { LIBNET_DHCP_NISPLUSDOMAIN,   "NISPLUSDOMAIN" }, 
+    { LIBNET_DHCP_NISPLUSSERVERS,  "NISPLUSSERVERS" }, 
+    { LIBNET_DHCP_MOBILEIPAGENT,   "MOBILEIPAGENT" }, 
+    { LIBNET_DHCP_SMTPSERVER,      "SMTPSERVER" },    
+    { LIBNET_DHCP_POP3SERVER,      "POP3SERVER" },    
+    { LIBNET_DHCP_NNTPSERVER,      "NNTPSERVER" },    
+    { LIBNET_DHCP_WWWSERVER,       "WWWSERVER" },     
+    { LIBNET_DHCP_FINGERSERVER,    "FINGERSERVER" },  
+    { LIBNET_DHCP_IRCSERVER,       "IRCSERVER" },     
+    { LIBNET_DHCP_STSERVER,        "STSERVER" },      
+    { LIBNET_DHCP_STDASERVER,      "STDASERVER" },   */ 
+    { LIBNET_DHCP_END,             "end", "End", 0, 0, 0, FIELD_HEX, "Set End", "", 0, 0, NULL }
 };
 
-#define DHCP_ATTACK_SEND_RAW           0
-#define DHCP_ATTACK_SEND_DISCOVER      1
-#define DHCP_ATTACK_DOS_SEND_DISCOVER  2
-#define DHCP_ATTACK_SEND_OFFER         3
-#define DHCP_ATTACK_SEND_REQUEST       4
-#define DHCP_ATTACK_SEND_DECLINE       5 
-#define DHCP_ATTACK_SEND_INFORM        6
-#define DHCP_ATTACK_ROGUE_SERVER       7
-#define DHCP_ATTACK_DOS_SEND_RELEASE   8
 
 void   dhcp_th_send_raw(void *);
 void   dhcp_th_send_raw_exit(struct attacks *);
@@ -325,15 +419,19 @@ static struct attack_param dhcp_dos_send_release_params[] = {
     { NULL, "End IP",                  4, 0,    0, FIELD_IP,  15, NULL }
 };
 
+#define DHCP_ATTACK_SEND_RAW           0
+#define DHCP_ATTACK_DOS_SEND_DISCOVER  1
+#define DHCP_ATTACK_ROGUE_SERVER       2
+#define DHCP_ATTACK_DOS_SEND_RELEASE   3
 
 static struct attack dhcp_attack[] = {
   { DHCP_ATTACK_SEND_RAW,          "sending RAW packet",        NONDOS, SINGLE,    dhcp_th_send_raw,      NULL, 0 },
-//  { DHCP_ATTACK_SEND_DISCOVER,     "sending DISCOVER packet",   NONDOS, dhcp_th_send_discover, NULL, 0 },
+/*  { DHCP_ATTACK_SEND_DISCOVER,     "sending DISCOVER packet",   NONDOS, dhcp_th_send_discover, NULL, 0 },*/
   { DHCP_ATTACK_DOS_SEND_DISCOVER, "sending DISCOVER packet",   DOS,    CONTINOUS, dhcp_th_dos_send_discover,NULL, 0 },
-//  { DHCP_ATTACK_SEND_OFFER,        "sending OFFER packet",      NONDOS, dhcp_th_send_offer,    NULL, 0 },
-//  { DHCP_ATTACK_SEND_REQUEST,      "sending REQUEST packet",    NONDOS, dhcp_th_send_request,  NULL, 0 },
-//  { DHCP_ATTACK_SEND_DECLINE,      "sending DECLINE packet",    NONDOS, dhcp_th_send_decline,  NULL, 0 },
-//  { DHCP_ATTACK_SEND_INFORM,       "sending INFORM packet",     NONDOS, dhcp_th_send_inform,   NULL, 0 },
+/*  { DHCP_ATTACK_SEND_OFFER,        "sending OFFER packet",      NONDOS, dhcp_th_send_offer,    NULL, 0 },
+  { DHCP_ATTACK_SEND_REQUEST,      "sending REQUEST packet",    NONDOS, dhcp_th_send_request,  NULL, 0 },
+  { DHCP_ATTACK_SEND_DECLINE,      "sending DECLINE packet",    NONDOS, dhcp_th_send_decline,  NULL, 0 },
+  { DHCP_ATTACK_SEND_INFORM,       "sending INFORM packet",     NONDOS, dhcp_th_send_inform,   NULL, 0 },*/
   { DHCP_ATTACK_ROGUE_SERVER,      "creating DHCP rogue server",NONDOS, CONTINOUS, dhcp_th_rogue_server,  dhcp_rogue_server_params,
                                                 SIZE_ARRAY(dhcp_rogue_server_params) },
   { DHCP_ATTACK_DOS_SEND_RELEASE,  "sending RELEASE packet",    DOS,    CONTINOUS, dhcp_th_dos_send_release, dhcp_dos_send_release_params,
@@ -354,20 +452,14 @@ int8_t dhcp_learn_offer(struct attacks *);
 int8_t dhcp_load_values(struct pcap_data *, void *);
 char **dhcp_get_printable_packet(struct pcap_data *);
 char **dhcp_get_printable_store(struct term_node *);
-char *dhcp_get_info(u_int8_t, char *);
 int8_t dhcp_update_field(int8_t, struct term_node *, void *);
 char  *dhcp_get_type_info(u_int16_t);
 int8_t dhcp_init_attribs(struct term_node *);
-int8_t dhcp_update_data(int8_t, int8_t, int8_t, struct term_node *);
 int8_t dhcp_edit_tlv(struct term_node *, u_int8_t, u_int8_t, u_int16_t, u_int8_t *);
-void   dhcp_help(void);
-int8_t dhcp_parser( struct term_node *, int8_t, char **, void *);
 int8_t dhcp_send_arp_request(struct attacks *, u_int32_t);
 int8_t dhcp_learn_mac(struct attacks *, u_int32_t, u_int8_t *);
-
-#ifndef HAVE_USLEEP
-extern int usleep(unsigned long);
-#endif
+int8_t dhcp_init_comms_struct(struct term_node *);
+int8_t dhcp_end(struct term_node *);
 
 extern void   thread_libnet_error( char *, libnet_t *);
 extern int8_t vrfy_bridge_id( char *, u_int8_t * );
@@ -376,12 +468,11 @@ extern int8_t thread_create(pthread_t *, void *, void *);
 extern void   write_log( u_int16_t mode, char *msg, ... );
 extern int8_t attack_th_exit(struct attacks *);
 extern void   attack_gen_mac(u_int8_t *);
-extern int8_t interfaces_get_packet(u_int8_t *, struct pcap_pkthdr *, u_int8_t *, u_int16_t, time_t);
+extern struct interface_data *interfaces_get_packet(list_t *, struct interface_data *, u_int8_t *, struct pcap_pkthdr *, u_int8_t *, u_int16_t, time_t);
 extern int8_t parser_vrfy_mac(char *, u_int8_t *);
 extern int8_t parser_get_inet_aton(char *, struct in_addr *);
 
 extern int8_t parser_command2index(register const struct attack *, register int8_t);
-extern struct interface_data interfaces[MAX_INTERFACES];
 extern struct terminals *terms;
 
 extern int8_t bin_data[];
