@@ -22,7 +22,7 @@
 
 #ifndef lint
 static const char rcsid[] = 
-       "$Id: dot1q.c,v 1.9 2005/09/13 12:10:39 t0mac Exp $";
+       "$Id: dot1q.c 17 2006-04-17 21:02:28Z tomac $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -78,12 +78,6 @@ static const char rcsid[] =
 #include <pthread.h>
 #endif
 
-#ifdef HAVE_GETOPT_LONG_ONLY
-#include <getopt.h>
-#else
-#include "getopt.h"
-#endif
-
 #include <stdarg.h>
 
 #include "dot1q.h"
@@ -92,13 +86,62 @@ static const char rcsid[] =
 void
 dot1q_register(void)
 {
-    protocol_register(PROTO_DOT1Q, "802.1Q", "802.1Q", sizeof(struct dot1q_data), dot1q_init_attribs, dot1q_learn_packet,
-	    dot1q_get_printable_packet, dot1q_get_printable_store,
-	    dot1q_load_values, dot1q_update_data, dot1q_attack, dot1q_help, 
-	    dot1q_parser, dot1q_fields, dot1q_get_info, dot1q_update_field, dot1q_features);
+   protocol_register(PROTO_DOT1Q, "802.1Q", "IEEE 802.1Q", "dot1q", 
+         sizeof(struct dot1q_data), dot1q_init_attribs, dot1q_learn_packet,
+         dot1q_get_printable_packet, dot1q_get_printable_store,
+         dot1q_load_values, dot1q_attack, dot1q_update_field, 
+         dot1q_features, dot1q_comm_params, SIZE_ARRAY(dot1q_comm_params), 
+         NULL, 0, NULL, dot1q_init_comms_struct, PROTO_VISIBLE, dot1q_end);
 }
 
-                  
+/*
+ * Inicializa la estructura que se usa para relacionar el tmp_data
+ * de cada nodo con los datos que se sacaran por pantalla cuando
+ * se accede al demonio de red.
+ * Teoricamente como esta funcion solo se llama desde term_add_node()
+ * la cual, a su vez, solo es llamada al tener el mutex bloqueado por
+ * lo que no veo necesario que sea reentrante. (Fredy). 
+ */
+int8_t
+dot1q_init_comms_struct(struct term_node *node)
+{
+    struct dot1q_data *dot1q_data;
+    void **comm_param;
+ 
+    comm_param = (void *)calloc(1,sizeof(void *)*SIZE_ARRAY(dot1q_comm_params));
+    
+    if (comm_param == NULL)
+    {
+       thread_error("dot1q_init_commands_struct calloc error",errno);
+       return -1;
+    }
+
+    dot1q_data = node->protocol[PROTO_DOT1Q].tmp_data;
+    
+    node->protocol[PROTO_DOT1Q].commands_param = comm_param;
+    
+    comm_param[0] =  &dot1q_data->mac_source; 
+    comm_param[1] =  &dot1q_data->mac_dest;
+    comm_param[2] =  &dot1q_data->vlan1; 
+    comm_param[3] =  &dot1q_data->priority1;
+    comm_param[4] =  &dot1q_data->cfi1; 
+    comm_param[5] =  &dot1q_data->tpi2; 
+    comm_param[6] =  &dot1q_data->vlan2; 
+    comm_param[7] =  &dot1q_data->priority2;
+    comm_param[8] =  &dot1q_data->cfi2;
+    comm_param[9] =  &dot1q_data->tpi3;
+    comm_param[10] = &dot1q_data->src_ip;  
+    comm_param[11] = &dot1q_data->dst_ip; 
+    comm_param[12] = &dot1q_data->ip_proto; 
+    comm_param[13] = &dot1q_data->icmp_payload;
+    comm_param[14] = NULL; 
+    comm_param[15] = NULL; 
+    
+    return 0;
+}
+
+
+
 void
 dot1q_th_send(void *arg)
 {
@@ -194,15 +237,19 @@ dot1q_send_icmp(struct attacks *attacks, u_int8_t double_encap)
 {
     libnet_ptag_t t;
     libnet_t *lhandler;
-    u_int32_t i;
     int32_t sent;
     u_int16_t *aux;        
     int32_t payload_size=0;
     struct dot1q_data *dot1q_data;
     u_int8_t *payload=NULL;
+    dlist_t *p;
+    struct interface_data *iface_data;
+    struct interface_data *iface_data2;
     
     dot1q_data = attacks->data;
 
+    dot1q_data->icmp_pay_len = strlen((const char *)dot1q_data->icmp_payload);
+    
       /* 802.1Q Double encap. */
     if (double_encap)
     {
@@ -221,9 +268,9 @@ dot1q_send_icmp(struct attacks *attacks, u_int8_t double_encap)
        *aux = htons(dot1q_data->tpi3);
     }
 
-    for ( i = 0; i < MAX_INTERFACES; i++) {
-        if (attacks->used_ints[i] == 1) {
-            lhandler = interfaces[i].libnet_handler;
+    for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p)) {
+       iface_data = (struct interface_data *) dlist_data(p);
+            lhandler = iface_data->libnet_handler;
 
             t = libnet_build_icmpv4_echo(
                 ICMP_ECHO,                     /* type */
@@ -270,7 +317,7 @@ dot1q_send_icmp(struct attacks *attacks, u_int8_t double_encap)
 
             t = libnet_build_802_1q(
                   dot1q_data->mac_dest,   /* dest mac */
-                  (attacks->mac_spoofing) ? dot1q_data->mac_source : interfaces[i].etheraddr,
+                  (attacks->mac_spoofing) ? dot1q_data->mac_source : iface_data->etheraddr,
                   dot1q_data->tpi1,       /* TPI */
                   dot1q_data->priority1,  /* priority (0 - 7) */
                   dot1q_data->cfi1,       /* CFI flag */
@@ -305,8 +352,8 @@ dot1q_send_icmp(struct attacks *attacks, u_int8_t double_encap)
 
             libnet_clear_packet(lhandler);
             protocols[PROTO_DOT1Q].packets_out++;
-            interfaces[i].packets_out[PROTO_DOT1Q]++;            
-        }
+            iface_data2 = interfaces_get_struct(iface_data->ifname);
+            iface_data2->packets_out[PROTO_DOT1Q]++;
     }
 
     if (payload)
@@ -328,9 +375,11 @@ dot1q_th_poison(void *arg)
     struct libnet_802_3_hdr *ether;
     struct timeval now;
     u_int8_t *packet=NULL, out=0, arp_mac[ETHER_ADDR_LEN];
-    u_int16_t *cursor, i;
+    u_int16_t *cursor;
     int32_t sent;    
     sigset_t mask;
+    dlist_t *p;
+    struct interface_data *iface_data;
 
     attacks = arg;
     
@@ -365,14 +414,15 @@ dot1q_th_poison(void *arg)
     
     while (!attacks->attack_th.stop && !out)
     {
-        interfaces_get_packet(&attacks->attack_th.stop, &header, packet,
+        interfaces_get_packet(attacks->used_ints, NULL, &attacks->attack_th.stop, &header, packet,
                                PROTO_DOT1Q, NO_TIMEOUT);
         if (attacks->attack_th.stop)
            break;   
 
         ether = (struct libnet_802_3_hdr *) packet;
         
-        if ( memcmp((attacks->mac_spoofing)?dot1q_data->mac_source:interfaces[0].etheraddr,
+        iface_data = (struct interface_data *) dlist_data(attacks->used_ints->list);
+        if ( memcmp((attacks->mac_spoofing)?dot1q_data->mac_source:iface_data->etheraddr,
                      ether->_802_3_dhost,6))
            continue; /* Not for the poisoned MAC... */
  
@@ -381,19 +431,19 @@ dot1q_th_poison(void *arg)
  
         cursor = (u_int16_t *) (packet + LIBNET_802_3_H);
 
-//        cursor++;
+/*        cursor++; */
 
 /* 802.3
         if (ntohs(*cursor) < 0x0800)
             do_802_3 = 1;
 */        
-        for ( i = 0; i < MAX_INTERFACES; i++) {
-            if (attacks->used_ints[i] == 1) {
-                lhandler = interfaces[i].libnet_handler;
+        for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p)) {
+           iface_data = (struct interface_data *) dlist_data(p);
+                lhandler = iface_data->libnet_handler;
 
                 t = libnet_build_ethernet(
                       arp_mac,  /* dest mac */
-                      (attacks->mac_spoofing) ? dot1q_data->mac_source : interfaces[i].etheraddr, /* src mac*/
+                      (attacks->mac_spoofing) ? dot1q_data->mac_source : iface_data->etheraddr, /* src mac*/
                       ETHERTYPE_VLAN,       /* type */
                       (u_int8_t *)cursor,   /* payload */
                       header.len - LIBNET_802_3_H, /* payload size */
@@ -422,8 +472,7 @@ dot1q_th_poison(void *arg)
 
                 libnet_clear_packet(lhandler);
                 protocols[PROTO_DOT1Q].packets_out++;
-                interfaces[i].packets_out[PROTO_DOT1Q]++;            
-            }
+                iface_data->packets_out[PROTO_DOT1Q]++;            
         }                                                                                         
                                                                                         
     } /*...!stop*/    
@@ -442,121 +491,122 @@ dot1q_th_poison(void *arg)
 int8_t
 dot1q_learn_mac(struct attacks *attacks, struct pcap_pkthdr *header, u_int8_t *arp_mac)
 {
-    struct dot1q_data *dot1q_data, dot1q_data_learned;
-    struct attack_param *param=NULL;
-    struct pcap_data pcap_aux;
-    struct libnet_802_3_hdr *ether;
-    u_int32_t i, arp_src, arp_dst;
-    u_int8_t mac_dest[ETHER_ADDR_LEN];
-    u_int8_t arp_mac_dest[ETHER_ADDR_LEN];
-    int8_t ret, gotit=0;
-    u_int8_t *packet=NULL;
-    u_int16_t *cursor, *arp_vlan;
-    
-    dot1q_data = attacks->data;
+   struct dot1q_data *dot1q_data, dot1q_data_learned;
+   struct attack_param *param=NULL;
+   struct pcap_data pcap_aux;
+   struct libnet_802_3_hdr *ether;
+   u_int32_t arp_src, arp_dst;
+   u_int8_t mac_dest[ETHER_ADDR_LEN];
+   u_int8_t arp_mac_dest[ETHER_ADDR_LEN];
+   int8_t ret, gotit=0;
+   u_int8_t *packet=NULL;
+   u_int16_t *cursor, *arp_vlan;
+   dlist_t *p;
+   struct interface_data *iface_data;
 
-    param = attacks->params;
-    arp_vlan = (u_int16_t *)param[DOT1Q_ARP_VLAN].value;
+   dot1q_data = attacks->data;
 
-    memcpy((void *)&arp_src, param[DOT1Q_ARP_IP_SRC].value, 4);
+   param = attacks->params;
+   arp_vlan = (u_int16_t *)param[DOT1Q_ARP_VLAN].value;
 
-    memcpy((void *)&arp_dst, param[DOT1Q_ARP_IP].value, 4);
+   memcpy((void *)&arp_src, param[DOT1Q_ARP_IP_SRC].value, 4);
 
-    memcpy((void *)mac_dest, (void *)"\xff\xff\xff\xff\xff\xff", ETHER_ADDR_LEN);
-    memcpy((void *)arp_mac_dest, (void *)"\x00\x00\x00\x00\x00\x00", ETHER_ADDR_LEN);
-    
-    header->ts.tv_sec = 0;
-    header->ts.tv_usec = 0;
-    
-    if ((packet = calloc(1, SNAPLEN)) == NULL)
-        return -1;
+   memcpy((void *)&arp_dst, param[DOT1Q_ARP_IP].value, 4);
 
-    while (!attacks->attack_th.stop && !gotit)
-    {
-        usleep(800000);
-        for ( i = 0; i < MAX_INTERFACES; i++) 
-        {
-            if (attacks->used_ints[i] == 1) 
-            {
-                ret = dot1q_send_arp(i, ARPOP_REQUEST,
-                                     (attacks->mac_spoofing)?dot1q_data->mac_source:interfaces[i].etheraddr,
-                                     mac_dest,
-                                     (u_int8_t *)&arp_src,
-                                     (u_int8_t *)&arp_dst,
-                                     arp_mac_dest,
-                                     *arp_vlan,
-                                     dot1q_data->priority1);
-                if (ret == -1)
-                {
-                   free(packet);
-                   return ret;
-                }
-            }
-        }
+   memcpy((void *)mac_dest, (void *)"\xff\xff\xff\xff\xff\xff", ETHER_ADDR_LEN);
+   memcpy((void *)arp_mac_dest, (void *)"\x00\x00\x00\x00\x00\x00", ETHER_ADDR_LEN);
 
-        interfaces_get_packet(&attacks->attack_th.stop, header, packet,
-                               PROTO_DOT1Q, NO_TIMEOUT);
+   header->ts.tv_sec = 0;
+   header->ts.tv_usec = 0;
 
-        if (attacks->attack_th.stop)
-           break;   
+   if ((packet = calloc(1, SNAPLEN)) == NULL)
+      return -1;
 
-        ether = (struct libnet_802_3_hdr *) packet;
+   while (!attacks->attack_th.stop && !gotit)
+   {
+      thread_usleep(800000);
+      for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p))
+      {
+         iface_data = (struct interface_data *) dlist_data(p);
+         ret = dot1q_send_arp(iface_data, ARPOP_REQUEST,
+               (attacks->mac_spoofing)?dot1q_data->mac_source:iface_data->etheraddr,
+               mac_dest,
+               (u_int8_t *)&arp_src,
+               (u_int8_t *)&arp_dst,
+               arp_mac_dest,
+               *arp_vlan,
+               dot1q_data->priority1);
+         if (ret == -1)
+         {
+            free(packet);
+            return ret;
+         }
+      }
 
-        if ( !memcmp((attacks->mac_spoofing)?dot1q_data->mac_source:interfaces[0].etheraddr, 
-                     ether->_802_3_shost, 6) )
-           continue; /* Oops!! Its our packet... */
-        
-        if ( memcmp((attacks->mac_spoofing)?dot1q_data->mac_source:interfaces[0].etheraddr,
-                     ether->_802_3_dhost,6))
-           continue; /* Not a response... */
-        
-        pcap_aux.header = header;
-        pcap_aux.packet = packet;
+      interfaces_get_packet(attacks->used_ints, NULL, &attacks->attack_th.stop, header, packet,
+            PROTO_DOT1Q, NO_TIMEOUT);
 
-        if (dot1q_load_values(&pcap_aux, &dot1q_data_learned) < 0)
-           continue;
+      if (attacks->attack_th.stop)
+         break;   
 
-        if (dot1q_data_learned.tpi2 != ETHERTYPE_ARP)
-           continue;
-           
-        cursor = (u_int16_t *) (packet + LIBNET_802_3_H);
+      ether = (struct libnet_802_3_hdr *) packet;
 
-        cursor++;
-         
-        if (ntohs(*cursor) < 0x0800) /* 802.3 */
-            cursor+=4;
-         
-        cursor+=2;
+      iface_data = (struct interface_data *) dlist_data(attacks->used_ints->list);
+      if ( !memcmp((attacks->mac_spoofing)?dot1q_data->mac_source:iface_data->etheraddr, 
+               ether->_802_3_shost, 6) )
+         continue; /* Oops!! Its our packet... */
 
-        if (ntohs(*cursor) != ETHERTYPE_IP)
-           continue;
- 
-        cursor+=2;
-         
-        if (ntohs(*cursor) != 2 )
-           continue;
+      if ( memcmp((attacks->mac_spoofing)?dot1q_data->mac_source:iface_data->etheraddr,
+               ether->_802_3_dhost,6))
+         continue; /* Not a response... */
 
-        cursor+=4;
+      pcap_aux.header = header;
+      pcap_aux.packet = packet;
 
-        if (memcmp((void *)cursor,(void *)&arp_dst,4))
-           continue;
+      if (dot1q_load_values(&pcap_aux, &dot1q_data_learned) < 0)
+         continue;
 
-        memcpy((void *)arp_mac,(void *)ether->_802_3_shost,6);
- 
-        write_log(0, " ARP Spoofing MAC = %02X:%02X:%02X:%02X:%02X:%02X\n",
-              ether->_802_3_shost[0], ether->_802_3_shost[1], ether->_802_3_shost[2],
-              ether->_802_3_shost[3], ether->_802_3_shost[4], ether->_802_3_shost[5]);
-         
-        gotit = 1;
-             
-    } /* !stop */
+      if (dot1q_data_learned.tpi2 != ETHERTYPE_ARP)
+         continue;
 
-    free(packet);
+      cursor = (u_int16_t *) (packet + LIBNET_802_3_H);
 
-    if (attacks->attack_th.stop)
-       return -1;   
-    
-    return 0;
+      cursor++;
+
+      if (ntohs(*cursor) < 0x0800) /* 802.3 */
+         cursor+=4;
+
+      cursor+=2;
+
+      if (ntohs(*cursor) != ETHERTYPE_IP)
+         continue;
+
+      cursor+=2;
+
+      if (ntohs(*cursor) != 2 )
+         continue;
+
+      cursor+=4;
+
+      if (memcmp((void *)cursor,(void *)&arp_dst,4))
+         continue;
+
+      memcpy((void *)arp_mac,(void *)ether->_802_3_shost,6);
+
+      write_log(0, " ARP Spoofing MAC = %02X:%02X:%02X:%02X:%02X:%02X\n",
+            ether->_802_3_shost[0], ether->_802_3_shost[1], ether->_802_3_shost[2],
+            ether->_802_3_shost[3], ether->_802_3_shost[4], ether->_802_3_shost[5]);
+
+      gotit = 1;
+
+   } /* !stop */
+
+   free(packet);
+
+   if (attacks->attack_th.stop)
+      return -1;   
+
+   return 0;
 }
 
 
@@ -566,60 +616,60 @@ dot1q_learn_mac(struct attacks *attacks, struct pcap_pkthdr *header, u_int8_t *a
 int8_t
 dot1q_return_mac(struct attacks *attacks, u_int8_t *arp_mac)
 {
-    struct attack_param *param = NULL;
-    struct dot1q_data *dot1q_data;
-    u_int32_t i, arp_ip, arp_dst;
-    u_int8_t a;
-    u_int8_t mac_dest[ETHER_ADDR_LEN];
-    u_int16_t *arp_vlan;
-    int8_t ret;
-    
-    dot1q_data = attacks->data;
+   struct attack_param *param = NULL;
+   struct dot1q_data *dot1q_data;
+   u_int32_t arp_ip, arp_dst;
+   u_int8_t a;
+   u_int8_t mac_dest[ETHER_ADDR_LEN];
+   u_int16_t *arp_vlan;
+   int8_t ret;
+   dlist_t *p;
+   struct interface_data *iface_data;
 
-    param = attacks->params;
-    arp_vlan = (u_int16_t *)param[DOT1Q_ARP_VLAN].value;
+   dot1q_data = attacks->data;
 
-    memcpy((void *)&arp_ip,param[DOT1Q_ARP_IP].value,4);
-    memcpy((void *)mac_dest, (void *)"\xff\xff\xff\xff\xff\xff", ETHER_ADDR_LEN);
-    //arp_ip = htonl(arp_ip);
+   param = attacks->params;
+   arp_vlan = (u_int16_t *)param[DOT1Q_ARP_VLAN].value;
 
-    arp_dst = 0;
+   memcpy((void *)&arp_ip,param[DOT1Q_ARP_IP].value,4);
+   memcpy((void *)mac_dest, (void *)"\xff\xff\xff\xff\xff\xff", ETHER_ADDR_LEN);
+   /*arp_ip = htonl(arp_ip);*/
 
-    for ( a=0; a < 3; a++)
-    {
-        for ( i=0; i < MAX_INTERFACES; i++) 
-        { 
-            if (attacks->used_ints[i] == 1) 
-            {
-                ret = dot1q_send_arp(i, ARPOP_REPLY,
-                                     arp_mac,
-                                     mac_dest,
-                                     (u_int8_t *)&arp_ip,
-                                     (u_int8_t *)&arp_dst,
-                                     mac_dest,
-                                     *arp_vlan,
-                                     dot1q_data->priority1);
-                if (ret == -1)
-                   return ret;
-            }
-        } 
-        sleep(1);
-    } 
+   arp_dst = 0;
 
-    return 0;
+   for ( a=0; a < 3; a++)
+   {
+      for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p))
+      { 
+         iface_data = (struct interface_data *) dlist_data(p);
+         ret = dot1q_send_arp(iface_data, ARPOP_REPLY,
+               arp_mac,
+               mac_dest,
+               (u_int8_t *)&arp_ip,
+               (u_int8_t *)&arp_dst,
+               mac_dest,
+               *arp_vlan,
+               dot1q_data->priority1);
+         if (ret == -1)
+            return ret;
+      } 
+      thread_usleep(999999);
+   } 
+
+   return 0;
 }
 
 
-void
+   void
 dot1q_th_poison_exit(struct attacks *attacks)
 {
 
-    if (attacks)
-       attack_th_exit(attacks);
-    
-    pthread_mutex_unlock(&attacks->attack_th.finished);
-    
-    pthread_exit(NULL);
+   if (attacks)
+      attack_th_exit(attacks);
+
+   pthread_mutex_unlock(&attacks->attack_th.finished);
+
+   pthread_exit(NULL);
 }
 
 
@@ -633,11 +683,13 @@ dot1q_send_arp_poison(void *arg)
     struct attacks *attacks;
     struct attack_param *param;
     struct dot1q_data *dot1q_data;
-    u_int32_t i, arp_ip, arp_dst;
+    u_int32_t arp_ip, arp_dst;
     u_int8_t out=0;
     u_int8_t mac_dest[ETHER_ADDR_LEN];
     u_int16_t *arp_vlan;
     int8_t ret;
+    dlist_t *p;
+    struct interface_data *iface_data;
 
     attacks = arg;
     
@@ -652,33 +704,31 @@ dot1q_send_arp_poison(void *arg)
 
     memcpy((void *)&arp_ip, param[DOT1Q_ARP_IP].value, 4);
     memcpy((void *)mac_dest, (void*)"\xff\xff\xff\xff\xff\xff", ETHER_ADDR_LEN);
-//    arp_ip = htonl(arp_ip);
+    /*    arp_ip = htonl(arp_ip);*/
 
     arp_dst = 0;
-    
+
     while (!attacks->helper_th.stop && !out)
     {
-        for ( i = 0; i < MAX_INTERFACES; i++) 
-        {
-            if (attacks->used_ints[i] == 1) 
-            {
-                ret = dot1q_send_arp(i, ARPOP_REPLY,
-                                     (attacks->mac_spoofing)?dot1q_data->mac_source:interfaces[i].etheraddr,
-                                     mac_dest,
-                                     (u_int8_t *)&arp_ip,
-                                     (u_int8_t *)&arp_dst,
-                                     mac_dest,
-                                     *arp_vlan,
-                                     dot1q_data->priority1);
-                if (ret == -1)
-                {
-                    out = 1;
-                    break;
-                }
-            }
-        } 
-        if (!out)
-           sleep(1);
+       for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p))
+       {
+          iface_data = (struct interface_data *) dlist_data(p);
+          ret = dot1q_send_arp(iface_data, ARPOP_REPLY,
+                (attacks->mac_spoofing)?dot1q_data->mac_source:iface_data->etheraddr,
+                mac_dest,
+                (u_int8_t *)&arp_ip,
+                (u_int8_t *)&arp_dst,
+                mac_dest,
+                *arp_vlan,
+                dot1q_data->priority1);
+          if (ret == -1)
+          {
+             out = 1;
+             break;
+          }
+       } 
+       if (!out)
+          thread_usleep(999999);
     }
 
     pthread_mutex_unlock(&attacks->helper_th.finished);
@@ -691,7 +741,7 @@ dot1q_send_arp_poison(void *arg)
  * Send ARP packet...
  */
 int8_t
-dot1q_send_arp(u_int32_t i, u_int16_t optype, 
+dot1q_send_arp(struct interface_data *iface, u_int16_t optype, 
                u_int8_t *mac_source, u_int8_t *mac_dest,
                u_int8_t *ip_source, u_int8_t *ip_dest, u_int8_t *arp_mac_dest, 
                u_int16_t vlan, u_int8_t priority)
@@ -700,7 +750,7 @@ dot1q_send_arp(u_int32_t i, u_int16_t optype,
     libnet_t *lhandler;
     int32_t sent;
     
-    lhandler = interfaces[i].libnet_handler;
+    lhandler = iface->libnet_handler;
     
     t = libnet_build_arp(
             ARPHRD_ETHER, /* hardware addr */
@@ -756,7 +806,7 @@ dot1q_send_arp(u_int32_t i, u_int16_t optype,
 
     libnet_clear_packet(lhandler);
     protocols[PROTO_DOT1Q].packets_out++;
-    interfaces[i].packets_out[PROTO_DOT1Q]++;
+    iface->packets_out[PROTO_DOT1Q]++;
 
     return 0;
 }
@@ -800,24 +850,32 @@ dot1q_init_attribs(struct term_node *node)
 
 
 int8_t
-dot1q_learn_packet(int8_t iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header)
+dot1q_learn_packet(struct attacks *attacks, char *iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header)
 {
     struct dot1q_data *dot1q_data;
     struct pcap_data pcap_aux;
     u_int8_t *packet, got_802_1q_pkt = 0;
     u_int16_t *cursor;
-    int8_t iface_learn;
+    dlist_t *p;
+    struct interface_data *iface_data;
     
     dot1q_data = data;
 
     if ((packet = calloc(1, SNAPLEN)) == NULL)
         return -1;
 
+    if (iface) {
+       p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, iface);
+       if (!p)
+          return -1;
+
+       iface_data = (struct interface_data *) dlist_data(p);
+    } else
+       iface_data = NULL;
+
     while (!got_802_1q_pkt && !(*stop))
     {
-        iface_learn = interfaces_get_packet(stop, header, packet, PROTO_DOT1Q, NO_TIMEOUT);
-        if ( (iface != ALL_INTS) && (iface != iface_learn))
-           continue;
+        interfaces_get_packet(attacks->used_ints, iface_data, stop, header, packet, PROTO_DOT1Q, NO_TIMEOUT);
            
         if (*stop)
         {
@@ -938,7 +996,7 @@ dot1q_get_printable_packet(struct pcap_data *data)
     if (data && (data->header->caplen < (14+8+8)) ) /* Undersized packet!! */
        return NULL;
 
-    if ((field_values = (char **) protocol_create_printable(DOT1Q_TOTAL_FIELDS, dot1q_fields)) == NULL) {
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_DOT1Q].nparams, protocols[PROTO_DOT1Q].parameters)) == NULL) {
 	    write_log(0, "Error in calloc\n");
 	    return NULL;
     }
@@ -1178,7 +1236,7 @@ dot1q_get_printable_packet(struct pcap_data *data)
         return (char **)field_values;
 
 #ifdef LBL_ALIGN 
-    snprintf(field_values[DOT1Q_IP_PROTO], 3, "%02d",(*(ip+9));
+    snprintf(field_values[DOT1Q_IP_PROTO], 3, "%02d",*(ip+9));
     memcpy((void *)&aux_long, (void *)(ip+12),4);
     aux = libnet_addr2name4(aux_long, LIBNET_DONT_RESOLVE);
     strncpy(field_values[DOT1Q_SRC_IP],aux,16);
@@ -1215,7 +1273,8 @@ dot1q_get_printable_store(struct term_node *node)
     /* smac + dmac + double + vlan1 + priority + cfi1 + tpi1 + vlan2 +
      * priority2 + cfi2 + tpi3 + src + dst + proto + arp + vlan + null = 17
      */
-    if ((field_values = (char **) protocol_create_printable(DOT1Q_TOTAL_FIELDS, dot1q_fields)) == NULL) {
+
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_DOT1Q].nparams, protocols[PROTO_DOT1Q].parameters)) == NULL) {
 	    write_log(0, "Error in calloc\n");
 	    return NULL;
     }
@@ -1261,167 +1320,11 @@ dot1q_get_printable_store(struct term_node *node)
     /* IP protocol */
     snprintf(field_values[DOT1Q_IP_PROTO], 3, "%02d",dot1q_tmp->ip_proto);
 
+    memcpy(field_values[DOT1Q_PAYLOAD], dot1q_tmp->icmp_payload, MAX_ICMP_PAYLOAD);
+
     return (char **)field_values;
 }
 
-char *
-dot1q_get_info(u_int8_t field, char *value)
-{
-    u_int8_t i;
-    u_int16_t val;
-    
-    i = 0;
-
-    switch (field) {
-        case DOT1Q_TPI2:
-        case DOT1Q_TPI3:
-            while (dot1q_tpi[i].desc) {
-                if (dot1q_tpi[i].type == strtoul(value, NULL, 16))
-                    return dot1q_tpi[i].desc;
-                i++;
-            }
-        break;
-        
-        case DOT1Q_IP_PROTO:
-            val = strtoul(value, NULL, 10);
-            while (dot1q_ip_proto[i].desc) 
-            {
-                if (dot1q_ip_proto[i].type == val)
-                    return dot1q_ip_proto[i].desc;
-                i++;
-            }
-            if (!val)
-               return "";
-        break;
-    }
-    
-    return "UNKN";
-}
-
-
-int8_t 
-dot1q_update_data(int8_t state, int8_t key, int8_t position, struct term_node *node)
-{
-    struct in_addr addr;
-    struct dot1q_data *dot1q_data;
-    u_int32_t aux_long;
-    u_int16_t i;
-    char **dot1q;
-
-    dot1q_data = node->protocol[PROTO_DOT1Q].tmp_data;
-
-    if ((dot1q = dot1q_get_printable_store(node)) == NULL) {
-	    write_log(0, "Error in dot1q_get_printable_store\n");
-	    return -1;
-    }
-
-    switch(state)
-    {
-        /* Source MAC */
-        case DOT1Q_SMAC:
-            dot1q[DOT1Q_SMAC][position] = key;
-            for (i=0; i < 6; i++) 
-                dot1q_data->mac_source[i] = strtoul(&dot1q[DOT1Q_SMAC][3*i], 
-                                                      (char **)NULL, 16);
-        break;
-
-        /* Destination MAC */
-        case DOT1Q_DMAC:
-            dot1q[DOT1Q_DMAC][position] = key;
-            for (i=0; i < 6; i++) 
-                dot1q_data->mac_dest[i] = strtoul(&dot1q[DOT1Q_DMAC][3*i], 
-                                                 (char **)NULL, 16);
-        break;
-
-        /* Vlan1 */
-        case DOT1Q_VLAN1:
-            dot1q[DOT1Q_VLAN1][position] = key;
-            dot1q_data->vlan1 = strtoul(dot1q[DOT1Q_VLAN1], (char **)NULL, 10);
-        break;
-        
-        /* Priority1 */
-        case DOT1Q_PRIORITY1:
-            dot1q[DOT1Q_PRIORITY1][position] = key;
-            dot1q_data->priority1 = strtoul(dot1q[DOT1Q_PRIORITY1], (char **)NULL,
-                                             16);
-        break;
-
-        /* CFI1 */
-        case DOT1Q_CFI1:
-            dot1q[DOT1Q_CFI1][position] = key;
-            dot1q_data->cfi1 = strtoul(dot1q[DOT1Q_CFI1], (char **)NULL, 16);
-        break;
-
-        /* Tag Protocol2 */
-        case DOT1Q_TPI2:
-            dot1q[DOT1Q_TPI2][position] = key;
-            dot1q_data->tpi2 = strtoul(dot1q[DOT1Q_TPI2], (char **)NULL, 16);
-        break;
-
-        /* Vlan2 */
-        case DOT1Q_VLAN2:
-            dot1q[DOT1Q_VLAN2][position] = key;
-            dot1q_data->vlan2 = strtoul(dot1q[DOT1Q_VLAN2], (char **)NULL, 10);
-        break;
-        
-        /* Priority2 */
-        case DOT1Q_PRIORITY2:
-            dot1q[DOT1Q_PRIORITY2][position] = key;
-            dot1q_data->priority2 = strtoul(dot1q[DOT1Q_PRIORITY2],
-                                                (char **)NULL, 16);
-        break;
-
-        /* CFI2 */
-        case DOT1Q_CFI2:
-            dot1q[DOT1Q_CFI2][position] = key;
-            dot1q_data->cfi2 = strtoul(dot1q[DOT1Q_CFI2], (char **)NULL, 16);
-        break;
-
-        /* Tag Protocol2 */
-        case DOT1Q_TPI3:
-            dot1q[DOT1Q_TPI3][position] = key;
-            dot1q_data->tpi3 = strtoul(dot1q[DOT1Q_TPI3], (char **)NULL, 16);
-        break;
-
-        /* Source IP */
-        case DOT1Q_SRC_IP:
-            dot1q[DOT1Q_SRC_IP][position] = key;
-            if (parser_get_inet_aton(dot1q[DOT1Q_SRC_IP], &addr) < 0) {
-                return -1;
-            }
-            aux_long = ntohl(addr.s_addr);
-            memcpy((void *)&dot1q_data->src_ip, (void *)&aux_long, 4);
-        break;
-
-        /* Destination IP */
-        case DOT1Q_DST_IP:
-            dot1q[DOT1Q_DST_IP][position] = key;
-            if (parser_get_inet_aton(dot1q[DOT1Q_DST_IP], &addr) < 0) {
-                write_log(0, "Error in inet_aton\n");
-                return -1;
-            }
-            aux_long = ntohl(addr.s_addr);
-            memcpy((void *)&dot1q_data->dst_ip, (void *)&aux_long, 4);
-        break;
-
-        /* IP protocol */
-        case DOT1Q_IP_PROTO:
-            dot1q[DOT1Q_IP_PROTO][position] = key;
-            dot1q_data->ip_proto = strtoul(dot1q[DOT1Q_IP_PROTO], (char **)NULL, 10);
-        break;
-    }
-
-    i = 0;
-    while(dot1q[i]) {
-	free(dot1q[i]);
-	i++;
-    }
-
-    free(dot1q);
-
-    return 0;
-}
-
 
 int8_t 
 dot1q_update_field(int8_t state, struct term_node *node, void *value)
@@ -1459,7 +1362,7 @@ dot1q_update_field(int8_t state, struct term_node *node, void *value)
         /* Tag Proto */
         case DOT1Q_TPI2:
 	        dot1q_data->tpi2 = *(u_int16_t *)value;
-//	    memcpy((void *)&dot1q_data->tpi2,value,2);
+/*	    memcpy((void *)&dot1q_data->tpi2,value,2);*/
         break;
         /* Priority */
         case DOT1Q_PRIORITY2:
@@ -1476,7 +1379,7 @@ dot1q_update_field(int8_t state, struct term_node *node, void *value)
 
         case DOT1Q_TPI3:
 	        dot1q_data->tpi3 = *(u_int16_t *)value;
-//	    memcpy((void *)&dot1q_data->tpi3,value,2);
+/*	    memcpy((void *)&dot1q_data->tpi3,value,2);*/
         break;
 		/* Source IP */
 	    case DOT1Q_SRC_IP:
@@ -1498,200 +1401,8 @@ dot1q_update_field(int8_t state, struct term_node *node, void *value)
 }
 
 
-void
-dot1q_help(void)
-{
-    write_log(2,"%s\n", bin_data);
-    write_log(2,"\nUsage: %s 802.1q [-hM] [-v version] [-i interface]\n", PACKAGE);
-    write_log(2,"       [-smac hw_addr] [-dmac hw_addr] [-vlan1 vlan_id]\n");
-    write_log(2,"       [-vlan2 vlan_id] [-priority1 vlan_priority]\n");
-    write_log(2,"       [-priority2 vlan_priority] [-ipsource ip_addr]\n");
-    write_log(2,"       [-ipdest ip_addr] [-attack attack]\n\n");
-    write_log(2,"           -h    This help screen.\n\n");         
-
-    write_log(2,"Please, see the man page for a full list of options and many examples.\n");
-    write_log(2,"Send your bugs & suggestions to the Yersinia developers <yersinia@wasahero.org>\n\n");  
-}
-
-
-/*
- * 802.1Q parser...
- */
 int8_t
-dot1q_parser( struct term_node *node, int8_t argc, char **args, void *tmp_data)
+dot1q_end(struct term_node *node)
 {
-    int8_t carac, indx, aux, tmp, ifaces;
-    int opt_ind;
-    struct dot1q_data *dot1q_data;
-    struct term_tty *term_tty=NULL;
-    struct attack *first_attack;
-    u_int16_t aux_short;
-    static struct option options[] =
-    {
-        { "help",        0, 0, 'h' },        
-        { "M",           0, 0, 'M' },        
-        { "attack",      1, 0, 0 },
-        { "X",           1, 0, 0 },
-        { "i",           1, 0, 1 },
-        { "vlan1",       1, 0, 2 },
-        { "smac",        1, 0, 3 },
-        { "dmac",        1, 0, 4 },
-        { "vlan2",       1, 0, 5 },
-        { "priority1",   1, 0, 6 },
-        { "priority2",   1, 0, 7 },
-        { "ipsource",    1, 0, 8 },
-        { "ipdest",      1, 0, 9 },
-        { 0, 0, 0, 0 }
-    };
-
-    dot1q_data = node->protocol[PROTO_DOT1Q].tmp_data;
-
-    ifaces = 0;
-
-    term_tty = node->specific;
-
-    while( (carac=getopt_long_only(argc,args,"hd",
-                    options, &opt_ind)) != EOF)
-    {
-        switch(carac) 
-        {
-            case 0:
-                aux=atoi(optarg);
-                aux--;  /* Because our array starts with zero but user starts with 1... :) */
-                indx = parser_command2index(dot1q_attack, aux);
-                if (dot1q_attack[indx].s != NULL )
-                    term_tty->attack = indx;
-                else
-                {
-                    write_log(2,"\n  Ouch!! Invalid attack!! Valid %s ATTACK types are:\n", PACKAGE);
-                    first_attack = dot1q_attack;
-                    while (first_attack->s != NULL) 
-                    {
-                        write_log(2,"         %d: %s attack %s\n",first_attack->v+1,
-                                (first_attack->type)?"DOS":"NONDOS",
-                                first_attack->s);
-                        ++first_attack;
-                    }
-                    write_log(2,"\n");
-                    return -1;
-                }
-                break;
-
-            case 1:
-                if ((tmp = interfaces_get(optarg)) == -1)
-                {
-                   write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                   return -1;
-                }
-                 /* Don't repeat interface...*/
-                if (!node->used_ints[tmp])
-                {
-                    if ((tmp = interfaces_add(optarg)) == -1)
-                    {
-                       write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                       return -1;
-                    }
-                    node->used_ints[tmp] = 1;
-                    ifaces++;
-                }
-            break;
-
-            case 2:
-                aux_short = atoi(optarg);
-                if (aux_short>4095)
-                {
-                   write_log(2, " VLAN1 id incorrect!!\n");
-                   return -1;
-                }
-                dot1q_data->vlan1 = aux_short;
-            break;
-
-            case 3:
-                if (parser_vrfy_mac(optarg, dot1q_data->mac_source)) {
-                    write_log(2, "Source MAC address incorrect!!\n");
-                    return -1;
-                }
-
-            case 4: 
-                if ( parser_vrfy_mac(optarg, dot1q_data->mac_dest) )
-                {
-                    write_log(2,"Destination MAC address incorrect!!\n");
-                    return -1;
-                }
-            break;
-
-            case 5:
-                aux_short = atoi(optarg);
-                if (aux_short>4095)
-                {
-                   write_log(2, " VLAN2 id incorrect!!\n");
-                   return -1;
-                }
-                dot1q_data->vlan2 = aux_short;
-            break;
-
-            case 6:
-                aux_short = atoi(optarg);
-                if (aux_short>7)
-                {
-                   write_log(2, " VLAN1 priority incorrect!!\n");
-                   return -1;
-                }
-                dot1q_data->priority1 = aux_short;
-            break;
-
-            case 7:
-                aux_short = atoi(optarg);
-                if (aux_short>7)
-                {
-                   write_log(2, " VLAN2 priority incorrect!!\n");
-                   return -1;
-                }
-                dot1q_data->priority2 = aux_short;
-            break;
-
-            case 8: /* IP source */
-
-            break;
-
-            case 9: /* IP destination */
-
-            break;
-
-            case 'h': 
-                  dot1q_help();
-                  return 0;
-            break;
-
-			case 'M':
-			   node->mac_spoofing = 0;
-			break;
-
-            case '?': return -1;
-            break;       
-        }
-    }
-
-    /* select a default device if not specified */
-    if (!ifaces)  {
-        if (strlen(interfaces[0].ifname)) {
-            write_log(2,"Warning: interface %s selected as the default one\n", &interfaces[0].ifname);
-            if ((tmp = interfaces_add(interfaces[0].ifname)) == -1)
-            {
-               write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n", interfaces[0].ifname);
-               return -1;
-            }
-            else
-                node->used_ints[tmp] = 1;
-                ifaces++;
-        } else {
-            write_log(2,"Hmm... you don't have any valid interface. Go and get a life!\n");
-            return -1;
-        }
-    }
-
    return 0;
-
 }
-
-/* vim:set tabstop=4:set expandtab:set shiftwidth=4:set textwidth=78: */
