@@ -22,7 +22,7 @@
 
 #ifndef lint
 static const char rcsid[] = 
-       "$Id: dtp.c,v 1.6 2005/09/13 12:10:39 t0mac Exp $";
+       "$Id: dtp.c 17 2006-04-17 21:02:28Z tomac $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -77,12 +77,6 @@ static const char rcsid[] =
 #include <pthread.h>
 #endif
 
-#ifdef HAVE_GETOPT_LONG_ONLY
-#include <getopt.h>
-#else
-#include "getopt.h"
-#endif
-
 #include <stdarg.h>
 
 #include "dtp.h"
@@ -91,26 +85,69 @@ static const char rcsid[] =
 void
 dtp_register(void)
 {
-    protocol_register(PROTO_DTP, "DTP", "Dynamic Trunking Protocol", sizeof(struct dtp_data), 
-                      dtp_init_attribs, dtp_learn_packet,
-	              dtp_get_printable_packet, dtp_get_printable_store,
-	              dtp_load_values, dtp_update_data, dtp_attack, dtp_help,
-                      dtp_parser, dtp_fields, dtp_get_info, dtp_update_field,
-                      dtp_features);
+   protocol_register(PROTO_DTP, "DTP", "Dynamic Trunking Protocol", "dtp",
+         sizeof(struct dtp_data), dtp_init_attribs, 
+         dtp_learn_packet, dtp_get_printable_packet, 
+         dtp_get_printable_store, dtp_load_values, 
+         dtp_attack, dtp_update_field,
+         dtp_features, dtp_comm_params, SIZE_ARRAY(dtp_comm_params),
+         NULL, 0, NULL, dtp_init_comms_struct, PROTO_VISIBLE, dtp_end);
 }
 
+/*
+ * Inicializa la estructura que se usa para relacionar el tmp_data
+ * de cada nodo con los datos que se sacaran por pantalla cuando
+ * se accede al demonio de red.
+ * Teoricamente esta funcion solo se llama desde term_add_node()
+ * la cual, a su vez, solo es llamada al tener el mutex bloqueado por
+ * lo que no veo necesario que sea reentrante. (Fredy). 
+ */
+int8_t
+dtp_init_comms_struct(struct term_node *node)
+{
+    struct dtp_data *dtp_data;
+    void **comm_param;
+ 
+    comm_param = (void *)calloc(1,sizeof(void *)*SIZE_ARRAY(dtp_comm_params));
+    
+    if (comm_param == NULL)
+    {
+       thread_error("dtp_init_commands_struct calloc error",errno);
+       return -1;
+    }
+
+    dtp_data = node->protocol[PROTO_DTP].tmp_data;
+    
+    node->protocol[PROTO_DTP].commands_param = comm_param;
+    
+    comm_param[DTP_SMAC]    = &dtp_data->mac_source; 
+    comm_param[DTP_DMAC]    = &dtp_data->mac_dest; 
+    comm_param[DTP_VERSION] = &dtp_data->version; 
+    comm_param[DTP_NEIGH]   = &dtp_data->neighbor; 
+    comm_param[DTP_STATUS]  = &dtp_data->status; 
+    comm_param[DTP_TYPE]    = &dtp_data->type; 
+    comm_param[DTP_DOMAIN]  = &dtp_data->domain; 
+    comm_param[7] = NULL; 
+    comm_param[8] = NULL; 
+
+    return 0;
+}
 
 void
 dtp_th_send(void *arg)
 {
     struct attacks *attacks=NULL;
     sigset_t mask;
+    struct dtp_data *dtp_data;
 
     attacks = arg;
     
     pthread_mutex_lock(&attacks->attack_th.finished);
 
     pthread_detach(pthread_self());
+
+    dtp_data = attacks->data;
+    dtp_data->dom_len = strlen(dtp_data->domain);
     
     sigfillset(&mask);
 
@@ -144,10 +181,13 @@ dtp_send(struct attacks *attacks)
 {
     libnet_ptag_t t;
     libnet_t *lhandler;
-    u_int32_t i, dtp_len, sent;
+    u_int32_t dtp_len, sent;
     struct dtp_data *dtp_data;
     u_int8_t *dtp_packet, *aux;
     u_int8_t cisco_data[]={ 0x00, 0x00, 0x0c, 0x20, 0x04 };
+    dlist_t *p;
+    struct interface_data *iface_data;
+    struct interface_data *iface_data2;
         
     dtp_data = attacks->data;
 
@@ -191,11 +231,11 @@ dtp_send(struct attacks *attacks)
     *aux = 0x0a; aux++;
     memcpy(aux,dtp_data->neighbor,ETHER_ADDR_LEN);
     
-    for ( i = 0; i < MAX_INTERFACES; i++) {
-        if (attacks->used_ints[i] == 1) {
-            lhandler = interfaces[i].libnet_handler;
+    for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p)) {
+       iface_data = (struct interface_data *) dlist_data(p);
+       lhandler = iface_data->libnet_handler;
 
-            t = libnet_build_802_2(
+       t = libnet_build_802_2(
                     0xaa,            /* DSAP */   
                     0xaa,            /* SSAP */
                     0x03,            /* control */
@@ -204,51 +244,51 @@ dtp_send(struct attacks *attacks)
                     lhandler,        /* libnet handle */
                     0);              /* libnet id */
 
-            if (t == -1) 
-            {
-                thread_libnet_error("Can't build ethernet header",lhandler);
-                libnet_clear_packet(lhandler);
-                free(dtp_packet);
-                return -1;
-            }  
+       if (t == -1) 
+       {
+           thread_libnet_error("Can't build ethernet header",lhandler);
+           libnet_clear_packet(lhandler);
+           free(dtp_packet);
+           return -1;
+       }  
 
-            t = libnet_build_802_3(
-                    dtp_data->mac_dest,       /* ethernet destination */
-                    (attacks->mac_spoofing) ? dtp_data->mac_source : interfaces[i].etheraddr,
-                    /* ethernet source */
-                    LIBNET_802_2_H + dtp_len, /* frame size */
-                    NULL,                     /* payload */
-                    0,                        /* payload size */
-                    lhandler,                 /* libnet handle */
-                    0);                       /* libnet id */
+       t = libnet_build_802_3(
+               dtp_data->mac_dest,       /* ethernet destination */
+               (attacks->mac_spoofing) ? dtp_data->mac_source : iface_data->etheraddr,
+               /* ethernet source */
+               LIBNET_802_2_H + dtp_len, /* frame size */
+               NULL,                     /* payload */
+               0,                        /* payload size */
+               lhandler,                 /* libnet handle */
+               0);                       /* libnet id */
 
-            if (t == -1)
-            {
-                thread_libnet_error("Can't build ethernet header",lhandler);
-                libnet_clear_packet(lhandler);
-                free(dtp_packet);                
-                return -1;
-            }
+       if (t == -1)
+       {
+           thread_libnet_error("Can't build ethernet header",lhandler);
+           libnet_clear_packet(lhandler);
+           free(dtp_packet);                
+           return -1;
+       }
 
-            /*
-             *  Write it to the wire.
-             */
-            sent = libnet_write(lhandler);
+       /*
+        *  Write it to the wire.
+        */
+       sent = libnet_write(lhandler);
 
-            if (sent == -1) {
-                thread_libnet_error("libnet_write error", lhandler);
-                libnet_clear_packet(lhandler);
-                free(dtp_packet);
-                return -1;
-            }
+       if (sent == -1) {
+           thread_libnet_error("libnet_write error", lhandler);
+           libnet_clear_packet(lhandler);
+           free(dtp_packet);
+           return -1;
+       }
 
-            libnet_clear_packet(lhandler);
-            protocols[PROTO_DTP].packets_out++;
-            interfaces[i].packets_out[PROTO_DTP]++;
-        }
+       libnet_clear_packet(lhandler);
+       protocols[PROTO_DTP].packets_out++;
+       iface_data2 = interfaces_get_struct(iface_data->ifname);
+       iface_data2->packets_out[PROTO_DTP]++;
     }
 
-    free(dtp_packet);    
+    free(dtp_packet);
 
     return 0;
 }
@@ -284,6 +324,8 @@ dtp_init_attribs(struct term_node *node)
 }
 
 
+
+
 /*****************************/
 /* Child/Thread loop sending */
 /* DTP packets every 30 secs */
@@ -390,15 +432,15 @@ dtp_th_nondos_do_trunk(void *arg)
     if ((packet = calloc(1, SNAPLEN)) == NULL)
         dtp_th_nondos_do_trunk_exit(attacks);
     
-    dtp_send(attacks);  sleep(1);
-    dtp_send(attacks);  sleep(1);
+    dtp_send(attacks);  thread_usleep(999999);
+    dtp_send(attacks);  thread_usleep(999999);
     dtp_send(attacks);
     
     thread_create(&attacks->helper_th.id, &dtp_send_negotiate, attacks);
     
     while (!attacks->attack_th.stop)
     {
-        interfaces_get_packet(&attacks->attack_th.stop, &header, packet,
+        interfaces_get_packet(attacks->used_ints, NULL, &attacks->attack_th.stop, &header, packet,
                                PROTO_DTP, NO_TIMEOUT);
         if (attacks->attack_th.stop)
            break;   
@@ -446,23 +488,32 @@ dtp_th_nondos_do_trunk_exit(struct attacks *attacks)
 
 
 int8_t
-dtp_learn_packet(int8_t iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header)
+dtp_learn_packet(struct attacks *attacks, char *iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header)
 {
     struct dtp_data *dtp_data;
     struct pcap_data pcap_aux;
     u_int8_t *packet, *cursor, got_dtp_packet = 0;
-    int8_t iface_learn;
+    dlist_t *p;
+    struct interface_data *iface_data;
     
     dtp_data = (struct dtp_data *)data;
 
     if ((packet = calloc(1, SNAPLEN)) == NULL)
         return -1;
 
+    if (iface) {
+       p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, iface);
+       if (!p)
+          return -1;
+
+       iface_data = (struct interface_data *) dlist_data(p);
+    } else {
+       iface_data = NULL;
+    }
+    
     while (!got_dtp_packet && !(*stop))
     {
-        iface_learn = interfaces_get_packet(stop, header, packet, PROTO_DTP, NO_TIMEOUT);
-        if ( (iface!=ALL_INTS) && (iface != iface_learn))
-           continue;
+        interfaces_get_packet(attacks->used_ints, iface_data, stop, header, packet, PROTO_DTP, NO_TIMEOUT);
            
         if (*stop)
         {
@@ -494,14 +545,14 @@ char **
 dtp_get_printable_packet(struct pcap_data *data)
 {
     struct libnet_802_3_hdr *ether;
-    u_int8_t *dtp_data, *ptr, *tlv_data; //, *aux;
+    u_int8_t *dtp_data, *ptr, *tlv_data; /*, *aux;*/
     u_int16_t tlv_type, tlv_len;
 #ifdef LBL_ALIGN
     u_int16_t aux_short;
 #endif
     char **field_values;
 
-    if ((field_values = (char **) protocol_create_printable(DTP_TOTAL_FIELDS, dtp_fields)) == NULL) {
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_DTP].nparams, protocols[PROTO_DTP].parameters)) == NULL) {
 	    write_log(0, "Error in calloc\n");
 	    return NULL;
     }
@@ -528,7 +579,7 @@ dtp_get_printable_packet(struct pcap_data *data)
     while(ptr < data->packet + data->header->caplen) 
     {        /* Undersized packet!! */
         if ( (ptr+4) > (data->packet + data->header->caplen))
-//           return NULL;
+/*           return NULL;*/
            break;
 
 #ifdef LBL_ALIGN
@@ -547,7 +598,7 @@ dtp_get_printable_packet(struct pcap_data *data)
         }
 
         if (!tlv_len) {
-//            write_log(0, "a ver...%X %X %X %X %X %X %X %X\n", *(ptr), *(ptr+1), *(ptr+2), *(ptr+3),*(ptr+4), *(ptr+5), *(ptr+6), *(ptr+7));
+/*            write_log(0, "a ver...%X %X %X %X %X %X %X %X\n", *(ptr), *(ptr+1), *(ptr+2), *(ptr+3),*(ptr+4), *(ptr+5), *(ptr+6), *(ptr+7));*/
            break;
         }
  
@@ -561,7 +612,7 @@ dtp_get_printable_packet(struct pcap_data *data)
             switch(tlv_type) 
             {
                 case DTP_TYPE_DOMAIN:
-                   if ((tlv_len-4) < 20 ) //DTP_DOMAIN_SIZE )
+                   if ((tlv_len-4) < 20 ) /*DTP_DOMAIN_SIZE )*/
                    {
                       memcpy(field_values[DTP_DOMAIN], ptr+4, tlv_len-4);
                       field_values[DTP_DOMAIN][(tlv_len-4)]=0;
@@ -614,7 +665,7 @@ dtp_get_printable_store(struct term_node *node)
     char **field_values;
     
     /* smac + dmac + version + domain + status + type + neighbor + null = 8 */
-    if ((field_values = (char **) protocol_create_printable(DTP_TOTAL_FIELDS, dtp_fields)) == NULL) {
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_DTP].nparams, protocols[PROTO_DTP].parameters)) == NULL) {
 	    write_log(0, "Error in calloc\n");
 	    return NULL;
     }
@@ -754,120 +805,6 @@ dtp_load_values(struct pcap_data *data, void *values)
 }
 
 
-
-int8_t 
-dtp_update_data(int8_t state, int8_t key, int8_t position, struct term_node *node)
-{
-    struct dtp_data *dtp_data;
-    u_int8_t aux=0;
-    u_int16_t i;
-    char **dtp;
-
-    dtp_data = node->protocol[PROTO_DTP].tmp_data;
-
-    if ((dtp = dtp_get_printable_store(node)) == NULL) {
-	    write_log(0, "Error in dtp_get_printable_store\n");
-	    return -1;
-    }
-
-    switch(state)
-    {
-        /* Source MAC */
-        case DTP_SMAC:
-            dtp[DTP_SMAC][position] = key;
-            for (i=0; i < 6; i++) 
-                dtp_data->mac_source[i] = strtoul(&dtp[DTP_SMAC][3*i], 
-                                                  (char **)NULL, 16);
-        break;
-
-        /* Destination MAC */
-        case DTP_DMAC:
-            dtp[DTP_DMAC][position] = key;
-            for (i=0; i < 6; i++) 
-                dtp_data->mac_dest[i] = strtoul(&dtp[DTP_DMAC][3*i], 
-                                               (char **)NULL, 16);
-        break;
-
-        /* Version */
-        case DTP_VERSION:
-            dtp[DTP_VERSION][position] = key;
-            dtp_data->version = strtoul(dtp[DTP_VERSION], (char **)NULL, 16);
-        break;
-
-        /* Status */
-        case DTP_STATUS:
-            dtp[DTP_STATUS][position] = key;
-            dtp_data->status = strtoul(dtp[DTP_STATUS], (char **)NULL, 16);
-        break;
-
-        /* Type */
-        case DTP_TYPE:
-            dtp[DTP_TYPE][position] = key;
-            dtp_data->type = strtoul(dtp[DTP_TYPE], (char **)NULL, 16);
-        break;
-
-        /* Domain */
-        case DTP_DOMAIN:
-            dtp[DTP_DOMAIN][position] = key;
-            memcpy((void *)dtp_data->domain, (void *)dtp[DTP_DOMAIN],
-                     DTP_DOMAIN_SIZE);
-            dtp_data->dom_len = strlen(dtp_data->domain);
-        break;
-
-        /* Neighborg */
-        case DTP_NEIGH:
-            dtp[DTP_NEIGH][position] = key;
-            for (i=0; i < 6; i++) 
-            {
-                if (i!=5)
-                {
-                    aux = dtp[DTP_NEIGH][2*i+2];
-                    dtp[DTP_NEIGH][2*i+2] = ':';
-                }
-                dtp_data->neighbor[i] = strtoul(&dtp[DTP_NEIGH][2*i], (char **)NULL, 16);
-                if (i!=5)
-                   dtp[DTP_NEIGH][2*i+2] = aux;
-            }
-        break;
-    }
-
-    i = 0;
-    while (dtp[i]) {
-	free(dtp[i]);
-	i++;
-    }
-
-    free(dtp);
-
-    return 0;
-}
-
-char *
-dtp_get_info(u_int8_t field, char *value)
-{
-    u_int8_t i;
-
-    i = 0;
-    switch (field) {
-        case DTP_STATUS:
-            while (dtp_status[i].desc) {
-                if (dtp_status[i].type == strtoul(value, NULL, 16))
-                    return dtp_status[i].desc;
-                i++;
-            }
-        break;
-        case DTP_TYPE:
-            while (dtp_type[i].desc) {
-                if (dtp_type[i].type == strtoul(value, NULL, 16))
-                    return dtp_type[i].desc;
-                i++;
-            }
-        break;
-    }
-
-    return "(Unknown)";
-}
-
 int8_t 
 dtp_update_field(int8_t state, struct term_node *node, void *value)
 {
@@ -913,167 +850,8 @@ dtp_update_field(int8_t state, struct term_node *node, void *value)
 }
 
 
-void
-dtp_help(void)
+int8_t
+dtp_end(struct term_node *node)
 {
-    write_log(2,"%s\n", bin_data);
-    write_log(2,"\nUsage: %s dtp [-hM] [-v version] [-i interface]\n", PACKAGE);
-    write_log(2,"       [-smac hw_addr] [-dmac hw_addr] [-neighbor hw_addr]\n");
-    write_log(2,"       [-domain domain] [-attack attack]\n\n");
-    write_log(2,"           -h    This help screen.\n\n");         
-
-    write_log(2,"Please, see the man page for a full list of options and many examples.\n");
-    write_log(2,"Send your bugs & suggestions to the Yersinia developers <yersinia@wasahero.org>\n\n");  
-}
-
-
-
-/*
- * DTP parser...
- */
-int8_t 
-dtp_parser( struct term_node *node, int8_t argc, char **args, void *tmp_data)
-{
-    int8_t carac, indx, aux, tmp, ifaces;
-    int opt_ind;
-    struct dtp_data *dtp_data;
-    struct term_tty *term_tty=NULL;
-    struct attack *first_attack;
-
-    static struct option options[] =
-    {
-        { "help",         0, 0, 'h' },        
-        { "M",            0, 0, 'M' },        
-        { "attack",       1, 0, 0 },
-        { "X",            1, 0, 0 },
-        { "i",            1, 0, 1 },
-        { "domain",       1, 0, 2 },
-        { "smac",         1, 0, 3 },
-        { "dmac",         1, 0, 4 },
-        { "neighbor",     1, 0, 5 },
-        { "v",            1, 0, 6 },
-        { "version",      1, 0, 6 },
-        { 0, 0, 0, 0 }
-    };
-
-    dtp_data = node->protocol[PROTO_DTP].tmp_data;
-
-    ifaces = 0;
-
-    term_tty = node->specific;
-
-    while( (carac=getopt_long_only(argc,args,"h",
-                    options, &opt_ind)) != EOF)
-    {
-        switch(carac) 
-        {
-            case 0:
-                aux=atoi(optarg);
-                aux--;  /* Because our array starts with zero but user starts with 1... :) */
-                indx = parser_command2index(dtp_attack, aux);
-                if (dtp_attack[indx].s != NULL )
-                    term_tty->attack = indx;
-                else
-                {
-                    write_log(2,"\n  Ouch!! Invalid attack!! Valid %s ATTACK types are:\n", PACKAGE);
-                    first_attack = dtp_attack;
-                    while (first_attack->s != NULL) 
-                    {
-                        write_log(2,"         %d: %s attack %s\n",first_attack->v+1,
-                                (first_attack->type)?"DOS":"NONDOS",
-                                first_attack->s);
-                        ++first_attack;
-                    }
-                    write_log(2,"\n");
-                    return -1;
-                }
-                break;
-
-            case 1:
-                if ((tmp = interfaces_get(optarg)) == -1)
-                {
-                   write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                   return -1;
-                }
-                 /* Don't repeat interface...*/
-                if (!node->used_ints[tmp])
-                {
-                    if ((tmp = interfaces_add(optarg)) == -1)
-                    {
-                       write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                       return -1;
-                    }
-                    node->used_ints[tmp] = 1;
-                    ifaces++;
-                }
-            break;
-
-            case 2:
-                strncpy(dtp_data->domain,optarg, (DTP_DOMAIN_SIZE-1));
-                if (strlen(optarg) > (DTP_DOMAIN_SIZE-1)) 
-                   dtp_data->dom_len = (DTP_DOMAIN_SIZE-1);
-                else
-                   dtp_data->dom_len = strlen(optarg);
-            break;
-
-            case 3:
-                if (parser_vrfy_mac(optarg, dtp_data->mac_source)) {
-                    write_log(2, "Source MAC address incorrect!!\n");
-                    return -1;
-                }
-
-            case 4: 
-                if ( parser_vrfy_mac(optarg, dtp_data->mac_dest) )
-                {
-                    write_log(2,"Destination MAC address incorrect!!\n");
-                    return -1;
-                }
-            break;
-
-            case 5: 
-                if ( parser_vrfy_mac(optarg, dtp_data->neighbor) )
-                {
-                    write_log(2,"Neighbor ID address incorrect!!\n");
-                    return -1;
-                }
-            break;
-
-            case 6:
-                dtp_data->version = atol(optarg);
-            break;
-
-            case 'h': dtp_help();
-                      return 0;
-            break;
-
-            case 'M': node->mac_spoofing = 0;
-                      break;
-
-            case '?': return -1;
-            break;       
-        }
-    }
-
-    /* select a default device if not specified */
-    if (!ifaces)  {
-        if (strlen(interfaces[0].ifname)) {
-            write_log(2,"Warning: interface %s selected as the default one\n", &interfaces[0].ifname);
-            if ((tmp = interfaces_add(interfaces[0].ifname)) == -1)
-            {
-               write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n", interfaces[0].ifname);
-               return -1;
-            }
-            else
-                node->used_ints[tmp] = 1;
-                ifaces++;
-        } else {
-            write_log(2,"Hmm... you don't have any valid interface. Go and get a life!\n");
-            return -1;
-        }
-    }
-
    return 0;
-
 }
-
-/* vim:set tabstop=4:set expandtab:set shiftwidth=4:set textwidth=78: */
