@@ -1,12 +1,15 @@
-/* $Id: md5.h,v 1.1.1.1 2005/04/09 20:17:53 t0mac Exp $ */
+/* $Id: md5.h 2 2006-04-03 21:04:25Z tomac $ */
 
 #ifndef MD5_H
 #define MD5_H
 
 #ifdef SOLARIS
-typedef u_long u_int32_t;
+typedef uint32_t u_int32_t;
+typedef uint16_t u_int16_t;
+typedef uint8_t  u_int8_t;
 #endif
 
+
 struct MD5Context {
 	u_int32_t buf[4];
 	u_int32_t bits[2];
