@@ -22,7 +22,7 @@
 
 #ifndef lint
 static const char rcsid[] = 
-       "$Id: terminal.c,v 1.6 2005/07/28 19:29:25 t0mac Exp $";
+       "$Id: terminal.c 2 2006-04-03 21:04:25Z tomac $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -123,11 +123,21 @@ term_init(void)
       return -1;
    }
 
+#ifdef HAS_CURSES
    if (pthread_mutex_init(&terms->gui_th.finished, NULL) != 0)
    {
       thread_error("term_init pthread_mutex_init mutex", errno);
       return -1;
    }
+#endif
+
+#ifdef HAVE_GTK
+   if (pthread_mutex_init(&terms->gui_gtk_th.finished, NULL) != 0)
+   {
+      thread_error("term_init pthread_mutex_init mutex", errno);
+      return -1;
+   }
+#endif
 
    if (pthread_mutex_init(&terms->admin_listen_th.finished, NULL) != 0)
    {
@@ -224,6 +234,8 @@ term_add_node(struct term_node **node,
           new->pcap_file.name    = NULL;
           new->pcap_file.pd      = NULL;
           new->pcap_file.pdumper = NULL;          
+          new->used_ints = (list_t *) calloc(1, sizeof(list_t));
+          new->used_ints->cmp = interfaces_compare;
           
           switch(type)
           {
@@ -254,6 +266,7 @@ term_add_node(struct term_node **node,
                  tty->daemonize = 0;
                  tty->debug = 0;
                  tty->interactive = 0;
+                 tty->gtk = 0;
                  tty->attack = -1;
                  tty->config_file[0] = '\0';
              break;
@@ -278,13 +291,14 @@ term_add_node(struct term_node **node,
           new->up = 1;
 
           /* initialize the tmp_data structure for each different protocol */
-
           for (i = 0; i < MAX_PROTOCOLS; i++) 
           {
+              if (!protocols[i].visible) 
+                 continue;
               new->protocol[i].pcap_file.name    = NULL;
               new->protocol[i].pcap_file.pd      = NULL;
               new->protocol[i].pcap_file.pdumper = NULL;
-              strncpy(new->protocol[i].name, protocols[i].name, MAX_PROTO_NAME);
+              strncpy(new->protocol[i].name, protocols[i].namep, MAX_PROTO_NAME);
               new->protocol[i].tmp_data = calloc(1, protocols[i].size);
               if (new->protocol[i].tmp_data == NULL)
               {
@@ -294,12 +308,27 @@ term_add_node(struct term_node **node,
 
               /* Copy the default values */
               memcpy((void *)new->protocol[i].tmp_data, (void *)protocols[i].default_values, protocols[i].size);
-
               new->protocol[i].proto = protocols[i].proto;
+              /* Initialize the default values */
+             if (protocols[i].init_attribs)
+                 protocols[i].init_attribs(new);
           }
+
           
+          /* Initialize the commands_struct for each protocol */
+          for (i = 0; i < MAX_PROTOCOLS; i++) 
+          {
+              if (!protocols[i].visible) 
+                 continue;
+              if (protocols[i].init_commands_struct)
+              {
+                  if ( protocols[i].init_commands_struct(new) == -1)
+                      return -1;
+              }
+          }
+
           *node = new;
-          
+                    
           return 0;
        }
    } /* for...*/
@@ -319,8 +348,8 @@ void
 term_delete_node(struct term_node *node, int8_t kill_th)
 {
    int8_t i, type_aux,number_aux;
-//   struct term_console *console;
-//   struct term_tty *tty;
+/*   struct term_console *console;
+   struct term_tty *tty;*/
    struct term_vty *vty;
 
    if (node->up)
@@ -363,20 +392,34 @@ term_delete_node(struct term_node *node, int8_t kill_th)
 
       for (i = 0; i < MAX_PROTOCOLS; i++)
       {
+          if (!protocols[i].visible)
+             continue;
           if (node->protocol[i].pcap_file.pdumper)
              interfaces_pcap_file_close(node,i);
           thread_free_r(node->protocol[i].tmp_data);
       }   
 
+      /* free the commands_struct for each protocol */
+      for (i = 0; i < MAX_PROTOCOLS; i++) 
+      {
+          if (!protocols[i].visible)
+             continue;
+          if (protocols[i].init_commands_struct)
+             thread_free_r(node->protocol[i].commands_param);
+      }
+
       /* Preserve terminal type and number...*/
       type_aux = node->type;
       number_aux = node->number;
 
+      dlist_delete(node->used_ints->list);
+      if (node->used_ints)
+         free(node->used_ints);
+
       memset(node,0,sizeof(struct term_node));
       
       node->type = type_aux;
       node->number = number_aux;
-
    } /* if (node->up) */
    
 }
@@ -589,7 +632,7 @@ term_vty_negotiate(struct term_node *node)
 
 /*
  * Write to terminal.
- * Return -1 on error. Return 0 if OK.
+ * Return -1 on error. Return 0 if OKk.
  */
 int8_t
 term_write(struct term_node *node, char *message, u_int16_t size)
@@ -900,13 +943,13 @@ term_vty_do_command(struct term_node *node)
                             term_vty_free_words(warray);
                             return -1;
                          }
-                         
-                         if (command_select_comm(node,warray,0,0) == -1)
+#ifdef HAVE_REMOTE_ADMIN                         
+                         if (command_entry_point(node,warray,0,0,0) == -1)
                          {
                             term_vty_free_words(warray);
                             return -1;
                          }
-                         
+#endif                         
                          term_vty_free_words(warray);
 
       break;
@@ -978,7 +1021,7 @@ term_vty_do_command(struct term_node *node)
                             return -1;                         
                          }
                          
-                         if (attack_filter_param(vty->attack_param[vty->substate].type,
+                         if (parser_filter_param(vty->attack_param[vty->substate].type,
                                                  vty->attack_param[vty->substate].value,
                                                  vty->attack_param[vty->substate].print,
                                                  vty->attack_param[vty->substate].size_print,
@@ -1085,7 +1128,11 @@ term_vty_exit(struct term_node *node)
 int8_t
 term_vty_clear_screen(struct term_node *node)
 {
+#ifdef HAVE_REMOTE_ADMIN
    return (command_cls(node, NULL, 0, 0, 0));
+#else
+   return 0;
+#endif   
 }
 
 
@@ -1322,7 +1369,11 @@ term_vty_history_next(struct term_node *node)
 int8_t
 term_vty_complete_command(struct term_node *node)
 {
-   return 0;
+   int8_t fail;
+   
+   fail = term_vty_help_tab(node,1);
+   
+   return fail;
 }
 
 
@@ -1404,9 +1455,20 @@ term_vty_backspace(struct term_node *node)
  */
 int8_t
 term_vty_help(struct term_node *node)
+{
+   int8_t fail;
+   
+   fail = term_vty_help_tab(node,0);
+   
+   return fail;
+}
+
+
+int8_t
+term_vty_help_tab(struct term_node *node, int8_t tab)
 {
    int8_t ret;
-   int8_t help_as_param=0;
+   int8_t as_param=0;
    struct words_array *warray;
    struct term_vty *vty = node->specific;
    
@@ -1415,7 +1477,7 @@ term_vty_help(struct term_node *node)
    if (vty->command_len)
    {
       if (vty->buf_command[vty->command_len-1] == SPACE)
-         help_as_param = 1;
+         as_param = 1;
    }
 
    vty->command_cursor = vty->command_len;
@@ -1424,7 +1486,7 @@ term_vty_help(struct term_node *node)
 
    if (warray == NULL)
    {
-      thread_error("term_vty_help calloc()",errno);
+      thread_error("term_vty_help_tab calloc()",errno);
       return -1;
    }
    
@@ -1433,20 +1495,14 @@ term_vty_help(struct term_node *node)
       term_vty_free_words(warray);
       return -1;
    }
-   
-   if (!warray->nwords) 
-   {
-      ret = command_help(node);
-      vty->repeat_command = 0;
-      term_vty_free_words(warray);
-      return ret;
-   }
- 
-   if (help_as_param)
-      ret = command_select_comm(node,warray,0,1);
+
+#ifdef HAVE_REMOTE_ADMIN
+   if (tab)
+      ret = command_entry_point(node,warray,0,as_param,1);
    else
-      ret = command_select_comm(node,warray,1,0);
-   
+      ret = command_entry_point(node,warray,1,as_param,0);   
+#endif
+
    term_vty_free_words(warray);
 
    return ret;   
@@ -1748,4 +1804,66 @@ term_vty_free_words(struct words_array *warray)
    free(warray);
 }
 
-/* vim:set tabstop=4:set expandtab:set shiftwidth=4:set textwidth=78: */
+
+
+/*
+ * Completion command for '*word'. Used if TAB pressed.
+ */
+int8_t
+term_vty_tab_subst(struct term_node *node, char *word, char *comm)
+{
+   struct term_vty *vty = node->specific;
+   char *aux, *aux2;
+   u_int16_t i, spaces, diff;
+   int16_t res;
+
+   spaces = 0;
+   i = vty->command_len-1;
+
+   while(1)
+   {
+      if (*(vty->buf_command+i) != SPACE)
+      {
+         aux=(vty->buf_command+i);
+         break;
+      }
+      spaces++;
+      i--;
+   }
+
+   diff = (strlen(comm)-strlen(word));
+
+   res = vty->command_len - spaces + diff + 1;
+
+   if (diff == 0) /* Exact word, just add another ' ' */
+   {
+      if (res<MAX_COMMAND)
+      {
+         if (vty->command_len < res)
+         {
+             *(aux+1) = ' ';
+            vty->command_len++;
+            vty->command_cursor++;
+            vty->buf_command[vty->command_len] = 0;
+         }
+      }
+      return 0;
+   }
+
+   aux2 = (comm+strlen(word));
+
+   if (res<MAX_COMMAND)
+   {
+      if (res > vty->command_len)
+      {
+         memcpy((aux+1),aux2,strlen(aux2));
+         vty->buf_command[vty->command_len+strlen(aux2)] = ' ';
+         vty->command_len+=strlen(aux2)+1;
+         vty->command_cursor+=strlen(aux2)+1;
+         vty->buf_command[vty->command_len] = 0;
+         return 0;
+      }
+   }
+
+   return 0;
+}
