@@ -22,7 +22,7 @@
 
 #ifndef lint
 static const char rcsid[] = 
-       "$Id: xstp.c,v 1.7 2005/09/13 12:10:39 t0mac Exp $";
+       "$Id: xstp.c 17 2006-04-17 21:02:28Z tomac $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -77,12 +77,6 @@ static const char rcsid[] =
 #include <pthread.h>
 #endif
 
-#ifdef HAVE_GETOPT_LONG_ONLY
-#include <getopt.h>
-#else
-#include "getopt.h"
-#endif
-
 #include <stdarg.h>
 
 #include "xstp.h"
@@ -91,10 +85,59 @@ static const char rcsid[] =
 void
 xstp_register(void)
 {
-    protocol_register(PROTO_STP, "STP", "Spanning Tree Protocol", sizeof(struct stp_data), 
-			xstp_init_attribs, xstp_learn_packet, xstp_get_printable_packet, xstp_get_printable_store,
-	    xstp_load_values, xstp_update_data, stp_attack, xstp_help, 
-	    xstp_parser, xstp_fields, xstp_get_info, xstp_update_field, xstp_features);
+   protocol_register(PROTO_STP, "STP", "Spanning Tree Protocol", "stp", 
+         sizeof(struct stp_data), xstp_init_attribs, xstp_learn_packet,
+         xstp_get_printable_packet, xstp_get_printable_store,
+         xstp_load_values, stp_attack, 
+         xstp_update_field,
+         xstp_features, xstp_comm_params,  SIZE_ARRAY(xstp_comm_params), 
+         NULL, 0, NULL, xstp_init_comms_struct, PROTO_VISIBLE, xstp_end);
+}
+
+/*
+ * Inicializa la estructura que se usa para relacionar el tmp_data
+ * de cada nodo con los datos que se sacaran por pantalla cuando
+ * se accede al demonio de red.
+ * Teoricamente como esta funcion solo se llama desde term_add_node()
+ * la cual, a su vez, solo es llamada al tener el mutex bloqueado por
+ * lo que no veo necesario que sea reentrante. (Fredy). 
+ */
+int8_t
+xstp_init_comms_struct(struct term_node *node)
+{
+    struct stp_data *stp_data;
+    void **comm_param;
+ 
+    comm_param = (void *)calloc(1,sizeof(void *)*SIZE_ARRAY(xstp_comm_params));
+    
+    if (comm_param == NULL)
+    {
+       thread_error("xstp_init_commands_struct calloc error",errno);
+       return -1;
+    }
+
+    stp_data = node->protocol[PROTO_STP].tmp_data;
+    
+    node->protocol[PROTO_STP].commands_param = comm_param;
+    
+    comm_param[XSTP_SMAC] = &stp_data->mac_source; 
+    comm_param[XSTP_DMAC] = &stp_data->mac_dest; 
+    comm_param[XSTP_ID] = &stp_data->id;
+    comm_param[XSTP_VER] = &stp_data->version; 
+    comm_param[XSTP_TYPE] = &stp_data->bpdu_type;
+    comm_param[XSTP_FLAGS] = &stp_data->flags; 
+    comm_param[XSTP_ROOTID] = &stp_data->root_id; 
+    comm_param[XSTP_PATHCOST] = &stp_data->root_pc;
+    comm_param[XSTP_BRIDGEID] = &stp_data->bridge_id;
+    comm_param[XSTP_PORTID] = &stp_data->port_id;
+    comm_param[XSTP_AGE] = &stp_data->message_age; 
+    comm_param[XSTP_MAX] = &stp_data->max_age;
+    comm_param[XSTP_HELLO] = &stp_data->hello_time;  
+    comm_param[XSTP_FWD] = &stp_data->forward_delay;
+    comm_param[14] = NULL;
+    comm_param[15] = NULL;
+
+    return 0;
 }
 
 
@@ -138,13 +181,14 @@ xstp_th_send_bpdu_conf_exit(struct attacks *attacks)
 int8_t
 xstp_send_all_bpdu_conf(struct attacks *attacks)
 {
-    u_int8_t i;
+    dlist_t *p;
 
-    for ( i = 0; i < MAX_INTERFACES; i++) 
+    for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p))
     {
-        if (attacks->used_ints[i] == 1) 
-           if (xstp_send_bpdu_conf(attacks->mac_spoofing,(struct stp_data *)(attacks->data),i) < 0)
-               return -1;
+        if (xstp_send_bpdu_conf(attacks->mac_spoofing,
+                  (struct stp_data *)(attacks->data), 
+                      (struct interface_data *) dlist_data(p)) < 0)
+            return -1;
     }
     
     return 0;
@@ -152,13 +196,14 @@ xstp_send_all_bpdu_conf(struct attacks *attacks)
 
 
 int8_t
-xstp_send_bpdu_conf(u_int8_t mac_spoofing, struct stp_data *stp_data, u_int8_t i)
+xstp_send_bpdu_conf(u_int8_t mac_spoofing, struct stp_data *stp_data, struct interface_data *iface)
 {
     libnet_ptag_t t;
     libnet_t *lhandler;
     int32_t sent;
-
-    lhandler = interfaces[i].libnet_handler;
+    struct interface_data *iface_data;
+    
+    lhandler = iface->libnet_handler;
 
     t = libnet_build_stp_conf(
           stp_data->id,                /* protocol id */
@@ -203,7 +248,7 @@ xstp_send_bpdu_conf(u_int8_t mac_spoofing, struct stp_data *stp_data, u_int8_t i
 
     t = libnet_build_802_3(
             stp_data->mac_dest,                 /* ethernet destination */
-            (mac_spoofing) ? stp_data->mac_source : interfaces[i].etheraddr,
+            (mac_spoofing) ? stp_data->mac_source : iface->etheraddr,
             /* ethernet source */
             LIBNET_802_2_H + LIBNET_STP_CONF_H + stp_data->rstp_len, /* frame size */
             NULL,                               /* payload */
@@ -231,7 +276,9 @@ xstp_send_bpdu_conf(u_int8_t mac_spoofing, struct stp_data *stp_data, u_int8_t i
 
     libnet_clear_packet(lhandler);
     protocols[PROTO_STP].packets_out++;
-    interfaces[i].packets_out[PROTO_STP]++;
+
+    iface_data = interfaces_get_struct(iface->ifname);
+    iface_data->packets_out[PROTO_STP]++;
     
     return 0;
 }
@@ -278,26 +325,26 @@ xstp_th_send_bpdu_tcn_exit(struct attacks *attacks)
 int8_t
 xstp_send_all_bpdu_tcn(struct attacks *attacks)
 {
-    u_int8_t i;
+   dlist_t *p;
 
-    for ( i = 0; i < MAX_INTERFACES; i++) 
+    for (p = attacks->used_ints->list; p; p = dlist_next(attacks->used_ints->list, p))
     {
-        if (attacks->used_ints[i] == 1)
-           if (xstp_send_bpdu_tcn(attacks->mac_spoofing,(struct stp_data *)(attacks->data),i) < 0)
-               return -1;
+        if (xstp_send_bpdu_tcn(attacks->mac_spoofing,(struct stp_data *)(attacks->data), (struct interface_data *)dlist_data(p)) < 0)
+            return -1;
     }
     return 0;
 }
 
 
 int8_t
-xstp_send_bpdu_tcn(u_int8_t mac_spoofing, struct stp_data *stp_data, u_int8_t i)
+xstp_send_bpdu_tcn(u_int8_t mac_spoofing, struct stp_data *stp_data, struct interface_data *iface)
 {
     libnet_ptag_t t;
     int32_t sent;
     libnet_t *lhandler;
-
-    lhandler = interfaces[i].libnet_handler;
+    struct interface_data *iface_data;
+    
+    lhandler = iface->libnet_handler;
 
     t = libnet_build_stp_tcn(
             stp_data->id,             /* protocol id */
@@ -332,7 +379,7 @@ xstp_send_bpdu_tcn(u_int8_t mac_spoofing, struct stp_data *stp_data, u_int8_t i)
 
     t = libnet_build_802_3(
             stp_data->mac_dest,                 /* ethernet destination */
-            (mac_spoofing) ? stp_data->mac_source : interfaces[i].etheraddr,
+            (mac_spoofing) ? stp_data->mac_source : iface->etheraddr,
             /* ethernet source */
             LIBNET_802_2_H + LIBNET_STP_TCN_H,  /* frame size */
             NULL,                               /* payload */
@@ -361,7 +408,9 @@ xstp_send_bpdu_tcn(u_int8_t mac_spoofing, struct stp_data *stp_data, u_int8_t i)
 
     libnet_clear_packet(lhandler);
     protocols[PROTO_STP].packets_out++;
-    interfaces[i].packets_out[PROTO_STP]++;
+    iface_data = interfaces_get_struct(iface->ifname);
+    iface_data->packets_out[PROTO_STP]++;
+
     return 0;
 }
 
@@ -497,7 +546,7 @@ xstp_th_dos_conf(void *arg)
         memcpy((void *)&stp_data->bridge_id[2],(void *)&stp_data->mac_source,6);
         xstp_send_all_bpdu_conf(attacks);
 #ifdef NEED_USLEEP
-        usleep(100000);
+        thread_usleep(100000);
 #endif
     }
 
@@ -552,7 +601,7 @@ xstp_th_dos_tcn(void *arg)
         attack_gen_mac(stp_data->mac_source);
         xstp_send_all_bpdu_tcn(attacks);
 #ifdef NEED_USLEEP
-        usleep(100000);
+        thread_usleep(100000);
 #endif
     }
     
@@ -610,7 +659,7 @@ xstp_th_nondos_role(void *arg)
     
     stp_data = attacks->data;
 
-    if (xstp_learn_packet(ALL_INTS, &attacks->attack_th.stop,stp_data,&header) < 0)
+    if (xstp_learn_packet(attacks, NULL, &attacks->attack_th.stop,stp_data,&header) < 0)
         xstp_th_nondos_role_exit(attacks);
    
     xstp_decrement_bridgeid(stp_data);
@@ -623,7 +672,7 @@ xstp_th_nondos_role(void *arg)
 
     while (!attacks->attack_th.stop)
     {
-        interfaces_get_packet(&attacks->attack_th.stop, &header, packet, PROTO_STP, NO_TIMEOUT);
+        interfaces_get_packet(attacks->used_ints, NULL, &attacks->attack_th.stop, &header, packet, PROTO_STP, NO_TIMEOUT);
         if (attacks->attack_th.stop)
            break;
            
@@ -682,8 +731,9 @@ xstp_send_hellos_mitm(void *arg)
     struct timeval hello;
     struct attacks *attacks;
     struct stp_data *stp_data, *stp_data2;
-    struct attack_param *param=NULL;
-    u_int8_t iface1, iface2;
+    struct attack_param *param = NULL;
+    dlist_t *p;
+    struct interface_data *iface1, *iface2;
 
     attacks = arg;
     
@@ -701,8 +751,10 @@ xstp_send_hellos_mitm(void *arg)
     stp_data2 = xstp_mitm_args->stp_data2;
     
     param = attacks->params;
-    memcpy((void *)&iface1,param[XSTP_MITM_IFACE1].value,1);
-    memcpy((void *)&iface2,param[XSTP_MITM_IFACE2].value,1);
+    p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, param[XSTP_MITM_IFACE1].value);
+    iface1 = (struct interface_data *) dlist_data(p);
+    p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, param[XSTP_MITM_IFACE2].value);
+    iface2 = (struct interface_data *) dlist_data(p);
 
     secs = 0;
     i = 0;
@@ -772,7 +824,8 @@ xstp_th_dos_mitm(void *arg)
     struct timeval now;
     u_int8_t flags_tmp, flags_tmp2;
     u_int8_t *packet=NULL, *stp_conf;
-    int8_t iface, iface1, iface2;
+    dlist_t *p;
+    struct interface_data *iface, *iface1, *iface2;
     sigset_t mask;
 
     attacks = arg;
@@ -800,17 +853,18 @@ xstp_th_dos_mitm(void *arg)
     xstp_mitm_args.stp_data2 = &stp_data2;
 
     param = attacks->params;
-    memcpy((void *)&iface1,param[XSTP_MITM_IFACE1].value,1);
-    memcpy((void *)&iface2,param[XSTP_MITM_IFACE2].value,1);
+    p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, param[XSTP_MITM_IFACE1].value);
+    iface1 = (struct interface_data *) dlist_data(p);
+    p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, param[XSTP_MITM_IFACE2].value);
+    iface2 = (struct interface_data *) dlist_data(p);
 
-
-    if (xstp_learn_packet(iface1, &attacks->attack_th.stop, stp_data, &header) < 0)
+    if (xstp_learn_packet(attacks, iface1->ifname, &attacks->attack_th.stop, stp_data, &header) < 0)
         xstp_th_dos_mitm_exit(attacks);
     xstp_decrement_bridgeid(stp_data);
    
     memcpy((void *)&stp_data2, (void *)stp_data,sizeof(struct stp_data));
     
-    if (xstp_learn_packet(iface2, &attacks->attack_th.stop, &stp_data2, &header) < 0)
+    if (xstp_learn_packet(attacks, iface2->ifname, &attacks->attack_th.stop, &stp_data2, &header) < 0)
         xstp_th_dos_mitm_exit(attacks);
     xstp_decrement_bridgeid(&stp_data2);
 
@@ -822,7 +876,7 @@ xstp_th_dos_mitm(void *arg)
 
     while (!attacks->attack_th.stop)
     {
-        iface = interfaces_get_packet(&attacks->attack_th.stop, &header, packet, PROTO_STP, NO_TIMEOUT);
+        iface = interfaces_get_packet(attacks->used_ints, NULL, &attacks->attack_th.stop, &header, packet, PROTO_STP, NO_TIMEOUT);
         if ( (iface!=iface1) && (iface!=iface2))
            continue;
            
@@ -927,7 +981,7 @@ xstp_th_nondos_other_role(void *arg)
     header.ts.tv_sec = now.tv_sec;
     header.ts.tv_usec = now.tv_sec;
     
-    if (xstp_learn_packet(ALL_INTS, &attacks->attack_th.stop, stp_data,&header) < 0)
+    if (xstp_learn_packet(attacks, NULL, &attacks->attack_th.stop, stp_data,&header) < 0)
         xstp_th_nondos_other_role_exit(attacks);
 
     xstp_increment_bridgeid(stp_data);
@@ -958,7 +1012,7 @@ xstp_th_nondos_other_role_exit(struct attacks *attacks)
  * If iface == ALL_INTS ge a packet from any interface 
  */
 int8_t
-xstp_learn_packet(int8_t iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header )
+xstp_learn_packet(struct attacks *attacks, char *iface, u_int8_t *stop, void *data, struct pcap_pkthdr *header )
 {
     struct stp_data *stp_data;
     struct libnet_802_3_hdr *ether;
@@ -967,20 +1021,29 @@ xstp_learn_packet(int8_t iface, u_int8_t *stop, void *data, struct pcap_pkthdr *
     u_int32_t aux_long;
 #endif    
     u_int8_t *packet, *stp_conf;
-    int8_t got_bpdu_conf = 0, iface_learn;
+    int8_t got_bpdu_conf = 0;
+    dlist_t *p;
+    struct interface_data *iface_data;
 
     stp_data = (struct stp_data *)data;
   
     if ((packet = calloc(1, SNAPLEN)) == NULL)
         return -1;
 
+    if (iface) {
+       p = dlist_search(attacks->used_ints->list, attacks->used_ints->cmp, iface);
+       if (!p)
+          return -1;
+
+       iface_data = (struct interface_data *) dlist_data(p);
+    } else {
+       iface_data = NULL;
+    }
+
     while (!got_bpdu_conf && ! (*stop) )
     {
-        iface_learn = interfaces_get_packet(stop, header, packet, PROTO_STP, NO_TIMEOUT);
+        interfaces_get_packet(attacks->used_ints, iface_data, stop, header, packet, PROTO_STP, NO_TIMEOUT);
 
-        if ( (iface != ALL_INTS) && (iface != iface_learn) )
-           continue;
-           
         if (*stop)
         {
             free(packet);
@@ -1141,7 +1204,7 @@ xstp_get_printable_packet(struct pcap_data *data)
 #endif
      char **field_values;
 
-    if ((field_values = (char **) protocol_create_printable((u_int8_t)XSTP_TOTAL_FIELDS, xstp_fields)) == NULL) {
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_STP].nparams, protocols[PROTO_STP].parameters)) == NULL) {
         write_log(0, "Error in calloc\n");
         return NULL;
     }
@@ -1153,6 +1216,7 @@ xstp_get_printable_packet(struct pcap_data *data)
     snprintf(field_values[XSTP_SMAC], 18, "%02X:%02X:%02X:%02X:%02X:%02X",
 	    ether->_802_3_shost[0], ether->_802_3_shost[1], ether->_802_3_shost[2],
 	    ether->_802_3_shost[3], ether->_802_3_shost[4], ether->_802_3_shost[5]);
+
     /* Destination MAC */
     snprintf(field_values[XSTP_DMAC], 18, "%02X:%02X:%02X:%02X:%02X:%02X",
 	    ether->_802_3_dhost[0], ether->_802_3_dhost[1], ether->_802_3_dhost[2],
@@ -1166,7 +1230,6 @@ xstp_get_printable_packet(struct pcap_data *data)
     snprintf(field_values[XSTP_ID], 5, "%04hX", ntohs(*(u_int16_t *)stp_data));
 #endif
 
-
     /* Version */
     snprintf(field_values[XSTP_VER], 3, "%02X", *((u_int8_t *)(stp_data+2)));
     /* BPDU Type */
@@ -1176,6 +1239,7 @@ xstp_get_printable_packet(struct pcap_data *data)
 	{
 		/* Flags */
 		snprintf(field_values[XSTP_FLAGS], 3, "%02X", *((u_int8_t *)stp_data+4));
+
 		/* Root ID */
 		snprintf(field_values[XSTP_ROOTID], 18, "%02X%02X.%02X%02X%02X%02X%02X%02X",
 			*(stp_data+5)&0xFF, *(stp_data+6)&0xFF, *(stp_data+7)&0xFF,
@@ -1249,7 +1313,8 @@ xstp_get_printable_store(struct term_node *node)
     /* smac + dmac + id + ver + type + flags + rootid + bridgeid + pathcost +
      * + portid + age + max + hello + fwd + null = 15
      */
-    if ((field_values = (char **) protocol_create_printable(XSTP_TOTAL_FIELDS, xstp_fields)) == NULL) {
+
+    if ((field_values = (char **) protocol_create_printable(protocols[PROTO_STP].nparams, protocols[PROTO_STP].parameters)) == NULL) {
 	    write_log(0, "Error in calloc\n");
 	    return NULL;
     }
@@ -1318,41 +1383,6 @@ xstp_get_printable_store(struct term_node *node)
 }
 
 
-char *
-xstp_get_info(u_int8_t field, char *value)
-{
-    u_int8_t i;
-
-    i = 0;
-    switch (field) {
-        case XSTP_VER:
-            while (xstp_version[i].desc) {
-                if (xstp_version[i].type == strtoul(value, NULL, 16))
-                    return xstp_version[i].desc;
-                i++;
-            }
-        break;
-        case XSTP_TYPE:
-            while (xstp_type[i].desc) {
-                if (xstp_type[i].type == strtoul(value, NULL, 16))
-                    return xstp_type[i].desc;
-                i++;
-            }
-        break;
-/*        case XSTP_FLAGS:
-            flags = strtoul(bpdu->flags, NULL, 16);
-            if (flags & STP_TOPOLOGY_CHANGE)
-            while (bpdu_flags[i].desc) {
-                if (bpdu_flags[i].type == strtoul(bpdu->flags, NULL, 16))
-                    return bpdu_flags[i].desc;
-                i++;
-            }
-*/
-    }
-
-    return "(Unknown)";
-}
-
 int8_t 
 xstp_update_field(int8_t state, struct term_node *node, void *value)
 {
@@ -1589,499 +1619,106 @@ xstp_load_values(struct pcap_data *data, void *values)
 }
 
 
-/*
- * Update the prog_pack structure with the hexadecimal charactered entered
- * by the user
- */
+
+
 int8_t 
-xstp_update_data(int8_t state, int8_t key, int8_t position, 
-        struct term_node *node)
+xstp_com_version(void *aux_node, void *value, char *printable)
 {
-    struct stp_data *stp_data;
-    u_int16_t i;
-    u_int8_t aux;
-    char **bpdu = NULL;
+   struct term_node *node = aux_node;
+   u_int8_t *version = value;
+   struct stp_data *stp_data;
+
+   stp_data = node->protocol[PROTO_STP].tmp_data;
+
+   if (*version == RSTP_VERSION)
+   {
+      stp_data->version  = RSTP_VERSION;
+      if (!stp_data->rstp_data)
+      {
+         stp_data->rstp_data = (u_int8_t *)calloc(1,1);
+         if (stp_data->rstp_data == NULL)
+         {
+             thread_error("xstp_com_version calloc error",errno);
+             return -1;
+         }
+/*memcpy((void *)stp_data->rstp_data, (void *)"\x00", 1);*/
+         stp_data->rstp_len = 1;
+      }
+   }
+   else
+   {
+       if (stp_data->rstp_data)
+       {
+          free(stp_data->rstp_data);
+          stp_data->rstp_len  = 0;
+       }
+   }
     
-    stp_data = node->protocol[PROTO_STP].tmp_data;
-
-    if ((bpdu = xstp_get_printable_store(node)) == NULL) {
-	    write_log(0, "Error in xstp_get_printable_store\n");
-	    return -1;
-    }
-
-    switch(state)
-    {
-        /* Source MAC */
-        case XSTP_SMAC:
-            bpdu[XSTP_SMAC][position] = key;
-            for (i=0; i < 6; i++) 
-                stp_data->mac_source[i] = strtoul(&bpdu[XSTP_SMAC][3*i], (char **)NULL, 16);
-        break;
-
-        /* Destination MAC */
-        case XSTP_DMAC:
-            bpdu[XSTP_DMAC][position] = key;
-            for (i=0; i < 6; i++) 
-                stp_data->mac_dest[i] = strtoul(&bpdu[XSTP_DMAC][3*i], (char **)NULL, 16);
-        break;
-
-        /* ID */
-        case XSTP_ID:
-            bpdu[XSTP_ID][position] = key;
-            sscanf(bpdu[XSTP_ID], "%hx", &stp_data->id);
-        break;
-
-        /* Version */
-        case XSTP_VER:
-            bpdu[XSTP_VER][position] = key;
-            stp_data->version = strtoul(bpdu[XSTP_VER], (char **)NULL, 16);
-        break;
-
-        /* BPDU Type */
-        case XSTP_TYPE:
-            bpdu[XSTP_TYPE][position] = key;
-            stp_data->bpdu_type = strtoul(bpdu[XSTP_TYPE], (char **)NULL, 16);
-        break;
-
-        /* Flags */
-        case XSTP_FLAGS:            
-            bpdu[XSTP_FLAGS][position] = key;
-            stp_data->flags = strtoul(bpdu[XSTP_FLAGS], (char **)NULL, 16);
-        break;
-
-        /* Root ID */
-        case XSTP_ROOTID:
-            bpdu[XSTP_ROOTID][position] = key;
-            for (i=0; i < 2; i++) 
-            {
-                 aux = bpdu[XSTP_ROOTID][2*i+2];
-                 bpdu[XSTP_ROOTID][2*i+2] = ':';
-                 stp_data->root_id[i] = strtoul(&bpdu[XSTP_ROOTID][2*i], (char **)NULL, 16);
-                 bpdu[XSTP_ROOTID][2*i+2] = aux;
-            }
-            for (i=0; i < 6; i++)   
-            {  
-                if (i!=5)
-                {
-                    aux = bpdu[XSTP_ROOTID][2*i+5+2];
-                    bpdu[XSTP_ROOTID][2*i+5+2] = ':';                  
-                }
-                stp_data->root_id[i+2] = strtoul(&bpdu[XSTP_ROOTID][2*i+5], (char **)NULL, 16);
-                if (i!=5)
-                   bpdu[XSTP_ROOTID][2*i+2+5] = aux;
-            }
-        break;
-
-        /* Bridge ID */
-        case XSTP_BRIDGEID:
-            bpdu[XSTP_BRIDGEID][position] = key;
-            for (i=0; i < 2; i++) 
-            {
-                 aux = bpdu[XSTP_BRIDGEID][2*i+2];
-                 bpdu[XSTP_BRIDGEID][2*i+2] = ':';
-                 stp_data->bridge_id[i] = strtoul(&bpdu[XSTP_BRIDGEID][2*i], (char **)NULL, 16);
-                 bpdu[XSTP_BRIDGEID][2*i+2] = aux;
-            }
-            for (i=0; i < 6; i++)   
-            {  
-                if (i!=5)
-                {
-                    aux = bpdu[XSTP_BRIDGEID][2*i+5+2];
-                    bpdu[XSTP_BRIDGEID][2*i+5+2] = ':';                  
-                }
-                stp_data->bridge_id[i+2] = strtoul(&bpdu[XSTP_BRIDGEID][2*i+5], (char **)NULL, 16);
-                if (i!=5)
-                   bpdu[XSTP_BRIDGEID][2*i+2+5] = aux;
-            }
-        break;
-
-        /* Root Pathcost */
-        case XSTP_PATHCOST:
-            bpdu[XSTP_PATHCOST][position] = key;
-            sscanf(bpdu[XSTP_PATHCOST], "%x", (u_int32_t *) &stp_data->root_pc);
-        break;
-
-        /* Port ID */
-        case XSTP_PORTID:
-            bpdu[XSTP_PORTID][position] = key;
-            sscanf(bpdu[XSTP_PORTID], "%hx", &stp_data->port_id);
-        break;
-
-        /* Message age */
-        case XSTP_AGE:
-            bpdu[XSTP_AGE][position] = key;
-            sscanf(bpdu[XSTP_AGE], "%hx", &stp_data->message_age);
-        break;
-
-        /* Max age */
-        case XSTP_MAX:
-            bpdu[XSTP_MAX][position] = key;
-            sscanf(bpdu[XSTP_MAX], "%hx", &stp_data->max_age);
-        break;
-
-        /* Hello time */
-        case XSTP_HELLO:
-            bpdu[XSTP_HELLO][position] = key;
-            sscanf(bpdu[XSTP_HELLO], "%hx", &stp_data->hello_time);
-        break;
-
-        /* Forward delay */
-        case XSTP_FWD:
-            bpdu[XSTP_FWD][position] = key;
-            sscanf(bpdu[XSTP_FWD], "%hx", &stp_data->forward_delay);
-        break;
-    }
-
-    i = 0;
-    while (bpdu[i]) {
-	free(bpdu[i]);
-	i++;
-    }
-
-    free(bpdu);
-    
-    return 0;
+   return 0;
 }
 
 
-void
-xstp_help(void)
+int8_t
+xstp_com_type(void *aux_node, void *value, char *printable)
 {
-    write_log(2,"%s\n", bin_data);
-    write_log(2,"\nUsage: %s stp [-hM] [-v version] [-i interface] [-type type]\n", PACKAGE);
-    write_log(2,"       [-source hw_addr] [-dest hw_addr] [-F flags] [-root id] [-cost pathcost]\n");
-    write_log(2,"       [-bridge id] [-port id] [-age secs] [-max secs] [-hello secs]\n");
-    write_log(2,"       [-role role] [-state state] [-forward secs]\n");
-    write_log(2,"       [-attack attack]\n\n");
-    write_log(2,"           -h    This help screen.\n\n");         
+   struct term_node *node = aux_node;
+   u_int8_t *type = value;
+   struct stp_data *stp_data;   
 
-    write_log(2,"Please, see the man page for a full list of options and many examples.\n");
-    write_log(2,"Send your bugs & suggestions to the Yersinia developers <yersinia@wasahero.org>\n\n");  
-}
+   stp_data = node->protocol[PROTO_STP].tmp_data;
 
-/*
- * A command line parser function for R/STP protocol
- */
-int8_t 
-xstp_parser( struct term_node *node, int8_t argc, char **args, void *tmp_data)
-{
-    int8_t carac, indx, aux, tmp, ifaces;
-    int opt_ind;
-    struct stp_data *stp_data;
-    struct term_tty *term_tty=NULL;
-    struct attack *first_attack;
-
-    static const struct option options[] =
-    {
-        { "help",         0, 0, 'h' },    
-        { "J",            0, 0, 'L' },
-        { "M",            0, 0, 'M' },
-        { "mac_spoofing", 0, 0, 'M' },
-        { "attack",       1, 0, 0 },
-        { "X",            1, 0, 0 },
-        { "bpdu_version", 1, 0, 1 },
-        { "v",            1, 0, 1 },
-        { "i",            1, 0, 2 },
-        { "type",         1, 0, 4 },
-        { "t",            1, 0, 4 },
-        { "source",       1, 0, 5 },
-        { "s",            1, 0, 5 },
-        { "dest",         1, 0, 6 },
-        { "d",            1, 0, 6 },
-        { "F",            1, 0, 7 },
-        { "root_id",      1, 0, 8 },
-        { "root",         1, 0, 8 },
-        { "root_pc",      1, 0, 9 },
-        { "cost",         1, 0, 9 },
-        { "bridge_id",    1, 0, 10 },
-        { "bridge",       1, 0, 10 },
-        { "port_id",      1, 0, 11 },
-        { "port",         1, 0, 11 },
-        { "role",         1, 0, 12 },
-        { "state",        1, 0, 13 },
-        { "message_age",  1, 0, 14 },
-        { "age",          1, 0, 14 },
-        { "max_age",      1, 0, 15 },
-        { "max",          1, 0, 15 },
-        { "hello",        1, 0, 16 },
-        { "forward",      1, 0, 17 },
-        { 0, 0, 0, 0 }
-    };
-
-    stp_data = node->protocol[PROTO_STP].tmp_data;
-
-    term_tty = node->specific;
-    ifaces = 0;
-
-    while( (carac=getopt_long_only(argc,args,"hLYy",
-                    options, &opt_ind)) != EOF)
-    {
-        switch(carac) 
+   switch(*type)
+   {
+      case 0:
+        stp_data->bpdu_type = BPDU_CONF_STP;
+        if (stp_data->rstp_data)
         {
-            case 0:
-                aux=atoi(optarg);
-                aux--;  /* Because our array starts with zero but user starts with 1... :) */
-                indx = parser_command2index(stp_attack, aux);
-                if (stp_attack[indx].s != NULL )
-                    term_tty->attack = indx;
-                else
-                {
-                    write_log(2,"\n  Ouch!! Invalid attack!! Valid %s ATTACK types are:\n", PACKAGE);
-                    first_attack = stp_attack;
-                    while (first_attack->s != NULL) 
-                    {
-                        write_log(2,"         %d: %s attack %s\n",first_attack->v+1,
-                                (first_attack->type)?"DOS":"NONDOS",
-                                first_attack->s);
-                        ++first_attack;
-                    }
-                    write_log(2,"\n");
-                    return -1;
-                }
-                break;
-
-            case 1:
-                parser_str_tolower(optarg);
-                if ( !strcmp("rstp",optarg) )
-                {
-                    stp_data->version   = RSTP_VERSION;
-                    memcpy((void *)stp_data->rstp_data, (void *)"\x00", 1);
-                    stp_data->rstp_len  = 1;
-                }
-                else
-                {
-                    if ( !strcmp("stp",optarg) )
-                        stp_data->version = STP_VERSION;
-                    else
-                    {
-                        write_log(2,"Ouch!! Invalid protocol: %s!!\n", optarg);
-                        return -1;
-                    }
-                }
-                break;
-
-            case 2:
-                if ((tmp = interfaces_get(optarg)) == -1)
-                {
-                   write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                   return -1;
-                }
-                 /* Don't repeat interface...*/
-                if (!node->used_ints[tmp])
-                {
-                    if ((tmp = interfaces_add(optarg)) == -1)
-                    {
-                       write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n",optarg);
-                       return -1;
-                    }
-                    node->used_ints[tmp] = 1;
-                    ifaces++;
-                }
-            break;
-
-            case 4: 
-                switch (*optarg)
-                {
-                    case '0': stp_data->bpdu_type = BPDU_CONF_STP;
-                              break;
-
-                    case '1': stp_data->version   = RSTP_VERSION;
-                              stp_data->bpdu_type = BPDU_CONF_RSTP;
-                              memcpy((void *)stp_data->rstp_data, (void *)"\x00", 1);
-                              stp_data->rstp_len  = 1;
-                              break;
-
-                    case '2': stp_data->bpdu_type = BPDU_TCN;
-                              break;
-
-                    default: write_log(2,"Ouch!! Valid %s BPDU types are:\n", PACKAGE);
-                             write_log(2,"    0: Configuration STP\n");
-                             write_log(2,"    1: Configuration RSTP\n");
-                             write_log(2,"    2: Topology Change Notification\n\n");
-                             return -1;
-                             break;
-                }
-                break;
-
-            case 5: 
-                if ( parser_vrfy_mac(optarg,stp_data->mac_source) )
-                {
-                   write_log(2,"Source MAC address incorrect!!\n");
-                   return -1;
-                }
-                break;
-
-            case 6: 
-                if ( parser_vrfy_mac(optarg, stp_data->mac_dest) )
-                {
-                   write_log(2,"Destination MAC address incorrect!!\n");
-                   return -1;
-                }
-                break;
-
-            case 7:
-                parser_str_tolower(optarg);
-                if ( (*optarg == '0') && (*(optarg+1) == 'x') && (strlen(optarg) >= 2) )
-                    stp_data->flags |= strtol((char *)optarg, (char **)NULL, 16);
-                else
-                    stp_data->flags |= atoi(optarg);
-                break;
-
-            case 8:
-                if (strlen(optarg) !=  22)
-                {
-                   write_log(2,"Root ID incorrect. Check sintax!\n");
-                   return -1;
-                }
-
-                if ( parser_vrfy_bridge_id(optarg, stp_data->root_id))
-                {
-                   write_log(2,"Root ID incorrect!!\n");
-                   return -1;
-                }
-                break;
-
-            case 9:
-                parser_str_tolower(optarg);
-                if ( (*optarg == '0') && (*(optarg+1) == 'x') && (strlen(optarg) >= 2) )
-                    stp_data->root_pc = strtol((char *)optarg, (char **)NULL, 16);
-                else
-                    stp_data->root_pc = atoi(optarg);
-                break;
-
-            case 10:
-                if (strlen(optarg) !=  22)
-                {
-                    write_log(2,"Bridge ID incorrect. Check sintax!\n");
-                    return -1;
-                }
-
-                if ( parser_vrfy_bridge_id(optarg, stp_data->bridge_id))
-                {
-                    write_log(2,"Bridge ID incorrect!!\n\n");
-                    return -1;
-                }
-                break;
-
-            case 11: 
-                parser_str_tolower(optarg);
-                if ( (*optarg == '0') && (*(optarg+1) == 'x') && (strlen(optarg) >= 2) )
-                    stp_data->port_id = strtol((char *)optarg, (char **)NULL, 16);
-                else
-                    stp_data->port_id = atoi(optarg);
-                break;
-
-            case 12: /* Port role */
-                parser_str_tolower(optarg);
-                if (!strncmp("unknown",optarg,strlen(optarg)))
-                {
-                    stp_data->flags |= RSTP_PORT_ROLE_UNKNOWN;
-                    break;
-                }
-                if (!strncmp("backup",optarg,strlen(optarg)))
-                {
-                    stp_data->flags |= RSTP_PORT_ROLE_BACKUP;
-                    break;
-                }
-                if (!strncmp("root",optarg,strlen(optarg)))
-                {
-                    stp_data->flags |= RSTP_PORT_ROLE_ROOT;
-                    break;
-                }
-                if (!strncmp("designated",optarg,strlen(optarg)))
-                {
-                    stp_data->flags |= RSTP_PORT_ROLE_DESIGNATED;
-                    break;
-                }
-                write_log(2,"Ouch!! Valid %s RSTP port roles are:\n \
-                        unknown: Unknown role\n \
-                        backup: Backup role\n \
-                        root: Root role\n \
-                        designated: Designated role\n", PACKAGE);
-                return -1;
-                break;
-
-            case 13: /* Port state */
-                parser_str_tolower(optarg);
-                if (!strncmp("proposal",optarg,strlen(optarg)))
-                {
-                    stp_data->flags |= RSTP_PROPOSAL;
-                    break;
-                }
-                if (!strncmp("learning",optarg,strlen(optarg)))
-                {
-                    stp_data->flags |= RSTP_LEARNING;
-                    break;
-                }                  
-                if (!strncmp("forwarding",optarg,strlen(optarg)))
-                {
-                    stp_data->flags |= RSTP_FORWARDING;
-                    break;
-                }                  
-                if (!strncmp("agreement",optarg,strlen(optarg)))
-                {
-                    stp_data->flags |= RSTP_AGREEMENT;
-                    break;
-                }
-                write_log(2,"Ouch!! Valid %s RSTP port states are:\n \
-                        proposal: Proposal state\n \
-                        learning: Learning state\n \
-                        forwarding: Forwarding state\n \
-                        agreement: Agreement state\n", PACKAGE);
-                return -1;
-                break;
-
-            case 14: 
-                stp_data->message_age = htons(atoi(optarg) * 256);
-                break;
-
-            case 15: 
-                stp_data->max_age = htons(atoi(optarg) * 256);
-                break;
-
-            case 16: 
-                stp_data->hello_time = htons(atoi(optarg) * 256);
-                break;
-
-            case 17: 
-                stp_data->forward_delay = htons(atoi(optarg) * 256);
-                break;
-
-            case 'h': xstp_help();
-                      return 0;
-            break;
-            
-            case 'L': /*init_pcap = 1;*/
-                      break;
-
-            case 'M': node->mac_spoofing = 0;
-                      break;
-          
-            case '?': write_log(2,"Unknown option!!\n");
-                      return -1;
-                      break;       
+           free(stp_data->rstp_data);
+           stp_data->rstp_len  = 0;
         }
-    }
-
-    /* select a default device if not specified */
-    if (!ifaces)  {
-        if (strlen(interfaces[0].ifname)) {
-            write_log(2,"Warning: interface %s selected as the default one\n", &interfaces[0].ifname);
-            if ((tmp = interfaces_add(interfaces[0].ifname)) == -1)
-            {
-               write_log(2,"Unable to use interface %s!! (Maybe nonexistent?)\n\n", interfaces[0].ifname);
+      break;
+      case 1:
+        stp_data->version   = RSTP_VERSION;
+        stp_data->bpdu_type = BPDU_CONF_RSTP;
+        if (!stp_data->rstp_data)
+        {
+           stp_data->rstp_data = (u_int8_t *)calloc(1,1);
+           if (stp_data->rstp_data == NULL)
+           {
+               thread_error("xstp_com_version calloc error",errno);
                return -1;
-            }
-            else
-                node->used_ints[tmp] = 1;
-                ifaces++;
-        } else {
-            write_log(2,"Hmm... you don't have any valid interface. Go and get a life!\n");
-            return -1;
+           }
         }
-    }
+/*        memcpy((void *)stp_data->rstp_data, (void *)"\x00", 1);*/
+        stp_data->rstp_len  = 1;
+      break;
+      case 2:
+        stp_data->bpdu_type = BPDU_TCN;
+        if (stp_data->rstp_data)
+        {
+           free(stp_data->rstp_data);
+           stp_data->rstp_len  = 0;
+        }
+      break;
+   }
+   
+   return 0;
+}
+
+
+int8_t 
+xstp_com_other(void *aux_node, void *value, char *printable)
+{
+   u_int16_t *aux16 = value;
+
+  *aux16 = htons( ((*aux16) * 256) ); 
 
    return 0;
 }
 
-/* vim:set tabstop=4:set expandtab:set shiftwidth=4:set textwidth=78: */
+
+int8_t
+xstp_end(struct term_node *node)
+{
+   return 0;
+}
