@@ -22,7 +22,7 @@
 
 #ifndef lint
 static const char rcsid[] = 
-"$Id: yersinia.c,v 1.12 2005/09/13 20:58:53 t0mac Exp $";
+"$Id: yersinia.c 15 2006-04-10 20:42:06Z tomac $";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,10 @@ static const char rcsid[] =
 #include <sys/types.h>
 #endif
 
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
@@ -85,6 +89,10 @@ static const char rcsid[] =
 #include <pthread.h>
 #endif
 
+#ifdef HAVE_GTK
+#include <gtk/gtk.h>
+#endif
+
 #include <termios.h>
 
 #include <stdarg.h>
@@ -97,181 +105,195 @@ static const char rcsid[] =
 int
 main( int argc, char **argv )
 {
-    struct cl_args *cl_args;
-    struct term_node *tty_node = NULL;
-    pid_t pid;
-    pid_t parent_id;
-#ifdef HAVE_PTHREAD_SETCONCURRENCY
+   struct cl_args *cl_args;
+   struct term_node *tty_node = NULL;
+   pid_t pid;
+   pid_t parent_id;
+#if defined(HAVE_PTHREAD_SETCONCURRENCY) && !defined(LINUX)
    int concurrent;
 #endif
-   
+
    handle_signals_parent();
 
    tcgetattr(0, &term_parent);   
- 
+
    parent_id = getpid();
-   
-    if ((pid = fork()) < 0)
-    {
-       exit(1);
-    }
-    else
-    {
-       if (pid != 0)
-       {
-          wait(NULL);
-          tcsetattr(0, TCSANOW, &term_parent);
-          exit(0);
-       }
-    }
 
-    fatal_error = 4;
+   if ((pid = fork()) < 0)
+   {
+      exit(1);
+   }
+   else
+   {
+      if (pid != 0)
+      {
+         wait(NULL);
+         tcsetattr(0, TCSANOW, &term_parent);
+         exit(0);
+      }
+   }
 
-    /* Disable all signals while initializing data...*/
-    handle_signals();
+   fatal_error = 4;
 
-    setvbuf(stdout, NULL, _IONBF, 0);
-    
-    tty_tmp = (struct term_tty *)calloc(1,sizeof(struct term_tty));
+   /* Disable all signals while initializing data...*/
+   handle_signals();
 
-    if (tty_tmp == NULL)
-    {
-       printf("Out of memory on calloc tty_tmp\n");
-       clean_exit();
-    }
+   setvbuf(stdout, NULL, _IONBF, 0);
 
-    tty_tmp->term = (struct termios *)calloc(1,sizeof(struct termios));      
+   tty_tmp = (struct term_tty *)calloc(1,sizeof(struct term_tty));
 
-    if (tty_tmp->term == NULL)
-    {
-       printf("Out of memory on calloc tty_tmp->term\n");
-       clean_exit();
-    }
+   if (tty_tmp == NULL)
+   {
+      printf("Out of memory on calloc tty_tmp\n");
+      clean_exit();
+   }
 
-    /* default values */
-    tty_tmp->attack = -1;
-    tty_tmp->mac_spoofing = -1;
-    tty_tmp->splash = -1;
-    strncpy(tty_tmp->username, VTY_USER, MAX_USERNAME);
-    strncpy(tty_tmp->password, VTY_PASS, MAX_PASSWORD);
-    strncpy(tty_tmp->e_password, VTY_ENABLE, MAX_PASSWORD);
-    tty_tmp->ip_filter = NULL;
-    
-    cl_args = (struct cl_args *)calloc(1,sizeof(struct cl_args));
+   tty_tmp->term = (struct termios *)calloc(1,sizeof(struct termios));      
 
-    if (cl_args == NULL)
-    {
-       printf("Out of memory on calloc cl_args\n");
-       clean_exit();
-    }
-    
-    if ( argc == 1 )                                                          
-    {
-        printf("GNU %s %s %s\n", PACKAGE, VERSION,
-                    "$Date: 2005/09/13 20:58:53 $");
-        printf("Try '%s -h' to display the help.\n",PACKAGE);
-        clean_exit();
-    }
+   if (tty_tmp->term == NULL)
+   {
+      printf("Out of memory on calloc tty_tmp->term\n");
+      clean_exit();
+   }
 
-    if (getuid() != 0) 
-    {
-        printf("You must be root to run %s %s\n", PACKAGE, VERSION);
-        clean_exit();
-    }
-
-    if (term_init() < 0)
-       g00dbye();
-
-    /* Register all the protocols */
-    protocol_init();
-
-    cl_args->proto_index = -1;
-
-    if (parser_initial(tty_tmp, cl_args, argc, argv) < 0) {
-        clean_exit();
-    } 
-
-    /* initialize logfile */
-    init_log();
-
-#ifdef HAVE_PTHREAD_SETCONCURRENCY
-    concurrent = pthread_getconcurrency();
-
-    concurrent = 15;//(MAX_TERMS*MAX_PROTOCOLS*MAX_THREAD_ATTACK*2)+3;
-
-    if (pthread_setconcurrency(concurrent) != 0)
-    {
-       thread_error("init pthread_setconcurrency()",errno);
-       g00dbye();
-    }
+   /* default values */
+   tty_tmp->interactive = 0;
+   tty_tmp->gtk = 0;
+   tty_tmp->attack = -1;
+   tty_tmp->mac_spoofing = -1;
+   tty_tmp->splash = -1;
+   strncpy(tty_tmp->username, VTY_USER, MAX_USERNAME);
+   strncpy(tty_tmp->password, VTY_PASS, MAX_PASSWORD);
+   strncpy(tty_tmp->e_password, VTY_ENABLE, MAX_PASSWORD);
+   tty_tmp->port = VTY_PORT;
+   tty_tmp->ip_filter = NULL;
+#ifdef HAVE_GTK
+   tty_tmp->buffer_log = NULL;
 #endif
-    
-    if (interfaces_init(&terms->pcap_listen_th) < 0 )
-       g00dbye();
-    
-    /* Establish TERM signal handler...*/
-    posix_signal(SIGTERM, final);
 
-    /* Parse config file */
-    if (strlen(tty_tmp->config_file))
-        if (parser_read_config_file(tty_tmp) < 0)
-            g00dbye();
+   cl_args = (struct cl_args *)calloc(1,sizeof(struct cl_args));
+
+   if (cl_args == NULL)
+   {
+      printf("Out of memory on calloc cl_args\n");
+      clean_exit();
+   }
+
+   if ( argc == 1 )                                                          
+   {
+      printf("GNU %s %s %s\n", PACKAGE, VERSION,
+            "$Date: 2006/03/23 08:40:14 $");
+      printf("Try '%s -h' to display the help.\n",PACKAGE);
+      clean_exit();
+   }
+
+   if (getuid() != 0) 
+   {
+      printf("You must be root to run %s %s\n", PACKAGE, VERSION);
+      clean_exit();
+   }
+
+   if (term_init() < 0)
+      g00dbye();
+
+   /* Register all the protocols */
+   protocol_init();
+
+   cl_args->proto_index = -1;
+
+   if (parser_initial(tty_tmp, cl_args, argc, argv) < 0) {
+      clean_exit();
+   } 
+
+   init_log();
+
+#if defined(HAVE_PTHREAD_SETCONCURRENCY) && !defined(LINUX)
+/*   concurrent = pthread_getconcurrency();*/
+
+   concurrent = 15;/*(MAX_TERMS*MAX_PROTOCOLS*MAX_THREAD_ATTACK*2)+3;*/
+
+   if (pthread_setconcurrency(concurrent) != 0)
+   {
+      thread_error("init pthread_setconcurrency()",errno);
+      g00dbye();
+   }
+#endif
+
+   if (interfaces_init(&terms->pcap_listen_th) < 0 )
+      g00dbye();
+
+   /* Establish TERM signal handler...*/
+   posix_signal(SIGTERM, final);
+
 
 #ifdef HAVE_REMOTE_ADMIN
-    if (tty_tmp->daemonize)
-    {
-       if (admin_init(tty_tmp->ip_filter) < 0)
-          g00dbye();
-    }
+   if (tty_tmp->daemonize)
+   {
+      if (admin_init(tty_tmp) < 0)
+         g00dbye();
+   }
 #endif 
 
    if (thread_create(&terms->uptime_th.id, &th_uptime, (void *)NULL) < 0)
       g00dbye();
 
-    /* Command line and ncurses cannot be choosed simultaneously...*/
-    if (!tty_tmp->interactive && (cl_args->proto_index != -1)) 
-    {
+   /* Command line and ncurses cannot be choosed simultaneously...*/
+   if ((!tty_tmp->interactive) && (!tty_tmp->gtk) && (cl_args->proto_index != -1)) 
+   {
+      terms->work_state = INITIAL;
+      tty_node = term_type[TERM_TTY].list;
+      if (thread_create(&tty_node[0].thread.id, &th_tty_peer, 
+               (void *)cl_args) < 0)
+         g00dbye();
 
-       tty_node = term_type[TERM_TTY].list;
-       if (thread_create(&tty_node[0].thread.id, &th_tty_peer, 
-                           (void *)cl_args) < 0)
-          g00dbye();
-
-       write_log(0, "thread.id es %hd\n", tty_node[0].thread.id);
-       while(tty_node[0].thread.id)
-          sleep(1);
-    }
+      while(terms->work_state != STOPPED)
+         thread_usleep(100000);
+   }
 
 #ifdef HAS_CURSES
-    if (tty_tmp->interactive)
-    {
-        if (thread_create(&terms->gui_th.id, &ncurses_gui, NULL) < 0 )
-           g00dbye();
-            /* Wait until the ncurses GUI is over */
-        while(terms->gui_th.id)
-          sleep(1);
-    }
+   if (tty_tmp->interactive)
+   {
+      terms->work_state = INITIAL;
+      if (thread_create(&terms->gui_th.id, &ncurses_gui, NULL) < 0 )
+         g00dbye();
+      /* Wait until the ncurses GUI is over */
+      while(terms->work_state != STOPPED)
+         thread_usleep(100000);
+   }
+   else
+   {
+#endif
+#ifdef HAVE_GTK
+      if (tty_tmp->gtk)
+      {
+         terms->work_state = INITIAL;
+         if (thread_create(&terms->gui_gtk_th.id, &gtk_gui, NULL) < 0 )
+            g00dbye();
+         /* Wait until the GTK GUI is over */
+         while(terms->work_state != STOPPED)
+            thread_usleep(100000);
+      }
+#endif
+#ifdef HAS_CURSES
+   }
 #endif
 
 #ifdef HAVE_REMOTE_ADMIN
-    if (tty_tmp->daemonize)
-    {
-       /* Ok, now that console (ncurses) is finished
-        * we can become a true daemon... */
-       become_daemon(parent_id);
-       
-       /* Wait until some important thread exits due to fatal_error...*/
-       while (fatal_error == 4)
-       {
-           usleep(150000);
-       }
-       g00dbye();
-    }
+   if (tty_tmp->daemonize)
+   {
+      /* Ok, now that console (ncurses) is finished
+       * we can become a true daemon... */
+      become_daemon(parent_id);
+
+      /* Wait until some important thread exits due to fatal_error...*/
+      while (fatal_error == 4)
+         thread_usleep(100000);
+   }
 #endif
 
-    g00dbye();
+   g00dbye();
 
-    exit(1);
+   exit(1);
 }
 
 
@@ -289,6 +311,8 @@ th_tty_peer(void *args)
    struct term_node *term_node=NULL;
    sigset_t mask;
    
+   terms->work_state = RUNNING;
+   
 write_log(0, "\n th_tty_peer thread = %d...\n",(int)pthread_self());
    
    sigfillset(&mask);
@@ -300,8 +324,11 @@ write_log(0, "\n th_tty_peer thread = %d...\n",(int)pthread_self());
    }   
 
    if (pthread_mutex_lock(&terms->mutex) != 0)
+   {
       thread_error("th_tty_peer pthread_mutex_lock",errno);
-
+      th_tty_peer_exit(NULL);
+   }
+   
    fail = term_add_node(&term_node, TERM_TTY, (int)NULL, pthread_self());
 
    if (fail == -1)
@@ -344,6 +371,14 @@ write_log(0, "\n th_tty_peer thread = %d...\n",(int)pthread_self());
 
     /* This is a tty so, man... ;) */
     strncpy(term_node->from_ip, "127.0.0.1", sizeof(term_node->from_ip));
+
+   /* Parse config file */
+   if (strlen(tty_tmp->config_file))
+      if (parser_read_config_file(tty_tmp, term_node) < 0)
+      {
+         write_log(0, "Error reading configuration file\n");
+         th_tty_peer_exit(term_node);
+      }
    
     if (init_attribs(term_node) < 0)        
     {
@@ -363,13 +398,7 @@ write_log(0, "\n th_tty_peer thread = %d...\n",(int)pthread_self());
         write_log(0, "Warning, proto %d has no init_attribs function!!\n", arguments->proto_index);
 
     /* Choose a parser */
-	if (protocols[arguments->proto_index].parser)
-	{
-        fail = (*protocols[arguments->proto_index].parser)
-              (term_node, arguments->count, arguments->argv_tmp, 
-              term_node->protocol[arguments->proto_index].tmp_data );
-	} else
-		write_log(0, "Warning, proto %d has no parser function!!\n", arguments->proto_index);
+    fail = parser_cl_proto(term_node, arguments->count, arguments->argv_tmp, arguments->proto_index);
 
    if (pthread_mutex_unlock(&terms->mutex) != 0)
       thread_error("th_tty_peer pthread_mutex_unlock",errno);
@@ -396,13 +425,15 @@ write_log(0, "\n th_tty_peer thread = %d...\n",(int)pthread_self());
 void
 th_tty_peer_exit(struct term_node *term_node)
 {
-   int i;
+   dlist_t *p;
+   struct interface_data *iface_data;
 
    if (term_node)
    {
-      for(i=0; i< MAX_INTERFACES; i++)
-         if (term_node->used_ints[i])
-            interfaces_del(i);
+      for (p = term_node->used_ints->list; p; p = dlist_next(term_node->used_ints->list, p)) {
+         iface_data = (struct interface_data *) dlist_data(p);
+         interfaces_disable(iface_data->ifname);
+      }
       
       attack_kill_th(term_node,ALL_ATTACK_THREADS);
       
@@ -417,6 +448,8 @@ th_tty_peer_exit(struct term_node *term_node)
 
    write_log(0,"\n th_tty_peer %d finished...\n",(int)pthread_self());      
 
+   terms->work_state = STOPPED;
+
    pthread_exit(NULL); 
 }
 
@@ -440,57 +473,65 @@ doloop(struct term_node *node, int mode)
 
     if (term_tty->attack >= 0) 
     {
-        printf("<*> Starting %s attack %s...\n", 
-            (theattack[term_tty->attack].type)?"DOS":"NONDOS",
-            theattack[term_tty->attack].s); 
+        if (theattack[term_tty->attack].nparams)
+        {
+            printf("\n<*> Ouch!! At the moment the command line interface doesn't support attacks <*>\n");
+            printf("<*> that needs parameters and the one you've choosed needs %d <*>\n",
+                         theattack[term_tty->attack].nparams);
+        }
+        else
+        {
+            printf("<*> Starting %s attack %s...\n", 
+                (theattack[term_tty->attack].type)?"DOS":"NONDOS",
+                theattack[term_tty->attack].s); 
 
-        if (attack_launch(node, mode, term_tty->attack, NULL, 0) < 0)
-            write_log(1, "Error launching attack %d (mode %d)!!\n", 
-		        term_tty->attack, mode);
+            if (attack_launch(node, mode, term_tty->attack, NULL, 0) < 0)
+                write_log(1, "Error launching attack %d (mode %d)!!\n", 
+                            term_tty->attack, mode);
 
-        fflush(stdin); fflush(stdout);
-        setvbuf(stdout, NULL, _IONBF, 0);
-        tcgetattr(0,&old_term);
-        tcgetattr(0,&term);
-        term.c_cc[VMIN]  = 1;
-        term.c_cc[VTIME] = _POSIX_VDISABLE;
-        term.c_lflag &= ~ICANON;
-        term.c_lflag &= ~ECHO;
-        tcsetattr(0,TCSANOW,&term);
+            fflush(stdin); fflush(stdout);
+            setvbuf(stdout, NULL, _IONBF, 0);
+            tcgetattr(0,&old_term);
+            tcgetattr(0,&term);
+            term.c_cc[VMIN]  = 1;
+            term.c_cc[VTIME] = _POSIX_VDISABLE;
+            term.c_lflag &= ~ICANON;
+            term.c_lflag &= ~ECHO;
+            tcsetattr(0,TCSANOW,&term);
 
-        if (theattack[term_tty->attack].single == CONTINOUS) {
-            printf("<*> Press any key to stop the attack <*>\n");
-            fail = 0;
-            while(!fail && !node->thread.stop)
-            {
-                FD_ZERO(&read_set);
-                FD_SET(0, &read_set);
-                timeout.tv_sec  = 0;
-                timeout.tv_usec = 200000;
-                if ( (ret=select(1, &read_set, NULL, NULL, &timeout) ) == -1 )
+            if (theattack[term_tty->attack].single == CONTINOUS) {
+                printf("<*> Press any key to stop the attack <*>\n");
+                fail = 0;
+                while(!fail && !node->thread.stop)
                 {
-                   thread_error("network_peer_th select()",errno);
-                   continue;
-                }
+                    FD_ZERO(&read_set);
+                    FD_SET(0, &read_set);
+                    timeout.tv_sec  = 0;
+                    timeout.tv_usec = 200000;
+                    if ( (ret=select(1, &read_set, NULL, NULL, &timeout) ) == -1 )
+                    {
+                       thread_error("network_peer_th select()",errno);
+                       continue;
+                    }
 
-                if ( !ret )  /* Timeout, decrement timers... */
-                   continue;
-                else
-                {
-                   if (FD_ISSET(0, &read_set))
-                   {
-                      getchar();
-                      fail = 1;
-                   }
-                }
-             }
-         } else
-            /* Command line, only one attack (0), let's wait for its conclusion... */
-            while (node->protocol[mode].attacks[0].up)
-                sleep(1);
-         
-         tcsetattr(0,TCSANOW, &old_term);
-         
+                    if ( !ret )  /* Timeout, decrement timers... */
+                       continue;
+                    else
+                    {
+                       if (FD_ISSET(0, &read_set))
+                       {
+                          getchar();
+                          fail = 1;
+                       }
+                    }
+                 }
+             } else
+                /* Command line, only one attack (0), let's wait for its conclusion... */
+                while (node->protocol[mode].attacks[0].up)
+                    thread_usleep(150000);
+             
+             tcsetattr(0,TCSANOW, &old_term);
+        }         
     } /* if term_tty->attack */
 }
 
@@ -512,7 +553,7 @@ write_log(0,"\n th_uptime thread = %d\n",(int)pthread_self());
    if (pthread_sigmask(SIG_BLOCK, &mask, NULL))
    {
       thread_error("th_uptime pthread_sigmask()",errno);
-      admin_th_listen_exit(NULL,0);
+      th_uptime_exit();
    }
 
    if (pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL))
@@ -591,7 +632,7 @@ th_uptime_clean(void *data)
 void
 become_daemon(pid_t parent_id)
 {
-   usleep(900000); /* Just for giving time to parent... */
+   thread_usleep(800000); /* Just for giving time to parent... */
 
    kill(parent_id,SIGUSR1); /* Kill parent... */
    
@@ -613,6 +654,10 @@ void
 write_log( u_int16_t mode, char *msg, ... )
 {
     va_list ap; /* Variable argument list...*/
+#ifdef HAVE_GTK
+	char buffer[4096];
+	GtkTextIter iter2;
+#endif
 
     va_start(ap,msg);
 
@@ -622,6 +667,15 @@ write_log( u_int16_t mode, char *msg, ... )
     if ((!mode || (mode == 1)) && (tty_tmp->log_file))
        vfprintf(tty_tmp->log_file, msg,ap);
 
+	/* Send yersinia log to the main_log gtk widget */
+#ifdef HAVE_GTK
+    if ((!mode || (mode == 1)) && ((tty_tmp->gtk) && (tty_tmp->buffer_log))) {
+       vsnprintf(buffer, 4096, msg,ap);
+	   gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (tty_tmp->buffer_log), &iter2, 0);
+	   gtk_text_buffer_insert(GTK_TEXT_BUFFER(tty_tmp->buffer_log), &iter2, buffer, -1);
+	}
+#endif
+
     va_end(ap);
 }
 
@@ -710,6 +764,12 @@ g00dbye(void)
        thread_destroy(&terms->gui_th);
 #endif
 
+#ifdef HAVE_GTK
+    if (tty_tmp->gtk && (terms->gui_gtk_th.id != 0)) {
+       thread_destroy(&terms->gui_gtk_th);
+    }
+#endif
+
 #ifdef HAVE_REMOTE_ADMIN
     if (tty_tmp->daemonize)
        admin_exit();
@@ -737,10 +797,10 @@ g00dbye(void)
       show_vty_motd();
    }
 
+   finish_log();
+
    if (tty_tmp)
       free(tty_tmp);
-   
-   finish_log();
 
    exit(0);
 }
@@ -761,7 +821,7 @@ init_log(void)
 
     gethostname(this_name, sizeof(this_name));
 
-    write_log(0,"# %s v%s started in %s at %s", PACKAGE, VERSION, 
+    write_log(0,"# %s v%s started in %s on %s", PACKAGE, VERSION, 
                    this_name, ctime(&this_time));
 
     write_log(0,"\n");
@@ -777,7 +837,7 @@ finish_log(void)
     time_t this_time;
     this_time = time(NULL);
 
-    write_log(0,"# %s finished at %s\n",PACKAGE, ctime(&this_time));
+    write_log(0,"# %s finished on %s\n",PACKAGE, ctime(&this_time));
 
     fflush(tty_tmp->log_file);
     if (fclose(tty_tmp->log_file) < 0)
@@ -797,11 +857,14 @@ init_attribs(struct term_node *node)
 
     result = 0;
 
-    for (i = 0; i < MAX_PROTOCOLS; i++) {
-		if (protocols[i].init_attribs)
-	        result = result | (*protocols[i].init_attribs)(node);
-		else
-			write_log(0, "Warning: protocol %d has no init_attribs function!!\n", i);
+    for (i = 0; i < MAX_PROTOCOLS; i++) 
+    {
+        if (!protocols[i].visible)
+           continue;
+        if (protocols[i].init_attribs)
+	   result = result | (*protocols[i].init_attribs)(node);
+        else
+           write_log(0, "Warning: protocol %d has no init_attribs function!!\n", i);
     }
 
     return result;
@@ -846,7 +909,6 @@ handle_signals_parent( void )
 #if !defined(OPENBSD) && !defined(FREEBSD) && !defined(DARWIN)
     posix_signal( SIGPOLL, SIG_IGN );
 #endif
-    posix_signal( SIGCHLD, SIG_IGN );
     posix_signal( SIGUSR1, final_parent );
     posix_signal( SIGUSR2, SIG_IGN );
     posix_signal( SIGINT,  SIG_IGN );
@@ -929,22 +991,3 @@ final_parent(int signal)
 }
 
 
-#ifndef HAVE_USLEEP
-int
-usleep(unsigned long useconds)
-{
-   struct timeval timeout;
-   int ret;
-
-   timeout.tv_sec  = 0;
-   timeout.tv_usec = useconds;
-
-   if (useconds > 999999)
-      useconds=999999;
-
-   ret = select(0,NULL,NULL,NULL,&timeout);
-
-   return ret;
-}
-#endif
-/* vim:set tabstop=4:set expandtab:set shiftwidth=4:set textwidth=78: */
